@@ -25,7 +25,7 @@
  * http://www.steadystate.com/css/
  * mailto:css@steadystate.co.uk
  *
- * $Id: CSSStyleSheetImpl.java,v 1.4 2008/01/03 13:35:40 xamjadmin Exp $
+ * $Id: CSSStyleSheetImpl.java,v 1.5 2008/08/10 22:54:43 xamjadmin Exp $
  */
 
 package com.steadystate.css.dom;
@@ -46,7 +46,7 @@ import com.steadystate.css.parser.*;
  * @version $Release$
  */
 public class CSSStyleSheetImpl implements CSSStyleSheet, Serializable {
-
+	public static final String KEY_DISABLED_CHANGED = "styleSheet.disabled.changed";
     private boolean _disabled = false;
     private Node _ownerNode = null;
     private StyleSheet _parentStyleSheet = null;
@@ -73,9 +73,19 @@ public class CSSStyleSheetImpl implements CSSStyleSheet, Serializable {
      * by generating an event for the main application.
      */
     public void setDisabled(boolean disabled) {
-        _disabled = disabled;
+    	if(this._disabled != disabled) {
+    		_disabled = disabled;
+    		Node node = this.getOwnerNode();
+    		if(node != null) {
+    			node.setUserData(KEY_DISABLED_CHANGED, Boolean.valueOf(disabled), null);
+    		}
+    	}
     }
 
+    public void setDisabledOnly(boolean disabled) {
+    	this._disabled = disabled;
+    }
+    
     public Node getOwnerNode() {
         return _ownerNode;
     }
