@@ -25,7 +25,7 @@
  * http://www.steadystate.com/css/
  * mailto:css@steadystate.co.uk
  *
- * $Id: RectImpl.java,v 1.4 2008/01/03 13:35:40 xamjadmin Exp $
+ * $Id: RectImpl.java,v 1.5 2009/01/18 14:30:22 xamjadmin Exp $
  */
  
 package com.steadystate.css.dom;
@@ -50,14 +50,14 @@ public class RectImpl implements Rect, Serializable {
     public RectImpl(LexicalUnit lu) {
         LexicalUnit next = lu;
         _left = new CSSValueImpl(next, true);
-        next = next.getNextLexicalUnit();
-        next = next.getNextLexicalUnit();
+        next = next == null ? null : next.getNextLexicalUnit();
+        next = next == null ? null : next.getNextLexicalUnit();
         _top = new CSSValueImpl(next, true);
-        next = next.getNextLexicalUnit();
-        next = next.getNextLexicalUnit();
+        next = next == null ? null : next.getNextLexicalUnit();
+        next = next == null ? null : next.getNextLexicalUnit();
         _right = new CSSValueImpl(next, true);
-        next = next.getNextLexicalUnit();
-        next = next.getNextLexicalUnit();
+        next = next == null ? null : next.getNextLexicalUnit();
+        next = next == null ? null : next.getNextLexicalUnit();
         _bottom = new CSSValueImpl(next, true);
     }
   
