@@ -25,7 +25,7 @@
  * http://www.steadystate.com/css/
  * mailto:css@steadystate.co.uk
  *
- * $Id: LexicalUnitImpl.java,v 1.3 2006/06/18 04:15:16 xamjadmin Exp $
+ * $Id: LexicalUnitImpl.java,v 1.5 2008/02/24 13:37:09 xamjadmin Exp $
  */
 
 package com.steadystate.css.parser;
@@ -271,6 +271,9 @@ public class LexicalUnitImpl implements LexicalUnit, Serializable {
         case SAC_OPERATOR_TILDE:
             sb.append("~");
             break;
+        case SAC_OPERATOR_EQUALS:
+            sb.append("=");
+            break;
         case SAC_INHERIT:
             sb.append("inherit");
             break;
@@ -345,6 +348,11 @@ public class LexicalUnitImpl implements LexicalUnit, Serializable {
             appendParams(sb, _params);
             sb.append(")");
             break;
+        case SAC_ALPHA_FUNCTION:
+        	sb.append(getFunctionName() + "(");
+        	appendParams(sb, _params);
+            sb.append(")");
+            break;
         }
         return sb.toString();
     }
@@ -660,7 +668,11 @@ public class LexicalUnitImpl implements LexicalUnit, Serializable {
     public static LexicalUnit createCounter(LexicalUnit prev, LexicalUnit params) {
         return new LexicalUnitImpl(prev, LexicalUnit.SAC_COUNTER_FUNCTION, "counter", params);
     }
-    
+
+    public static LexicalUnit createAlpha(LexicalUnit prev, LexicalUnit params) {
+        return new LexicalUnitImpl(prev, LexicalUnit.SAC_ALPHA_FUNCTION, "alpha", params);
+    }
+
     public static LexicalUnit createCounters(LexicalUnit prev, LexicalUnit params) {
         return new LexicalUnitImpl(prev, LexicalUnit.SAC_COUNTERS_FUNCTION, "counters", params);
     }
