@@ -53,4 +53,22 @@ public interface BrowserFrame {
 	 * Gets the {@link HtmlRendererContext} of the frame.
 	 */
 	public HtmlRendererContext getHtmlRendererContext();
+	
+	/**
+	 * Sets the default margin insets of the browser frame.
+	 * @param insets The margin insets.
+	 */
+	public void setDefaultMarginInsets(java.awt.Insets insets);
+	
+	/**
+	 * Sets the default horizontal overflow of the browser frame.
+	 * @param overflowX See constants in {@link org.lobobrowser.html.style.RenderState}.
+	 */
+	public void setDefaultOverflowX(int overflowX);
+
+	/**
+     * Sets the default vertical overflow of the browser frame.
+     * @param overflowY See constants in {@link org.lobobrowser.html.style.RenderState}.
+     */
+	public void setDefaultOverflowY(int overflowY);
 }
