@@ -73,6 +73,10 @@ public class DescendentHTMLCollection extends AbstractScriptableDelegate impleme
 					if(id != null && id.length() != 0) {
 						itemsByName.put(id, element);
 					}
+					String name = element.getAttribute("name");
+					if(name != null && name.length() != 0 && !name.equals(id)) {
+						itemsByName.put(name, element);
+					}
 				}
 			}
 		}
