@@ -1,5 +1,8 @@
 package org.lobobrowser.html.domimpl;
 
+/**
+ * A listener of document changes.
+ */
 public interface DocumentNotificationListener {
 	/**
 	 * Called if a property related to the node's
@@ -27,7 +30,8 @@ public interface DocumentNotificationListener {
 	/**
 	 * This is called when the node has changed, but
 	 * it is unclear if it's a size change or a look
-	 * change.
+	 * change. Typically, a node attribute has changed,
+	 * but the set of child nodes has not changed.
 	 * @param node
 	 */
 	public void invalidated(NodeImpl node);
@@ -38,6 +42,12 @@ public interface DocumentNotificationListener {
 	 * @param node
 	 */
 	public void nodeLoaded(NodeImpl node);
+	
+	/**
+	 * The children of the node might have changed.
+	 * @param node
+	 */
+	public void structureInvalidated(NodeImpl node);
 
 	/**
 	 * Called when a external script (a SCRIPT tag with a src attribute) 
@@ -46,6 +56,7 @@ public interface DocumentNotificationListener {
 	 */
 	public void externalScriptLoading(NodeImpl node);
 
+	
 	/**
 	 * This is called when the whole document
 	 * is potentially invalid, e.g. when a new
