@@ -91,6 +91,8 @@ class ElementFactory {
 		
 		builders.put("U", new HTMLElementBuilder.Underline());
 		builders.put("STRIKE", new HTMLElementBuilder.Strike());
+		builders.put("SUP", new HTMLElementBuilder.Sup());
+		builders.put("SUB", new HTMLElementBuilder.Sub());
 		
 		HTMLElementBuilder em = new HTMLElementBuilder.Em();
 		builders.put("I", em);
