@@ -90,7 +90,8 @@ public class ElementImpl extends NodeImpl implements Element {
 	private String id;
 
 	public String getId() {
-		return this.id;
+		String id = this.id;
+		return id == null ? "" : id;
 	}
 
 	public void setId(String id) {
@@ -259,13 +260,16 @@ public class ElementImpl extends NodeImpl implements Element {
 			// Note that the value of name is used
 			// as an ID, but the value of ID is not
 			// used as a name.
-			String oldId = this.id;
-			this.id = value;
+			if(!isName) {	
+				this.id = value;
+			}
 			HTMLDocumentImpl document = (HTMLDocumentImpl) this.document;
 			if (document != null) {
-				if (oldId != null) {
-					document.removeElementById(oldId);
-				}
+				//// Do not remove old ID. Consider scenario where both
+				//// name and ID are provided in an element.
+//				if (oldId != null) {
+//					document.removeElementById(oldId);
+//				}
 				document.setElementById(value, this);
 				if(isName) {
 					String oldName = this.getAttribute("name");
@@ -473,18 +477,6 @@ public class ElementImpl extends NodeImpl implements Element {
 		return sb.toString();
 	}
 
-	/**
-	 * Attempts to convert the subtree starting at this point to a close text
-	 * representation. BR elements are converted to line breaks, and so forth.
-	 */
-	public String getInnerText() {
-		StringBuffer buffer = new StringBuffer();
-		synchronized(this.treeLock) {
-			this.appendInnerTextImpl(buffer);
-		}
-		return buffer.toString();
-	}
-	
 	public void setInnerText(String newText) {
 		org.w3c.dom.Document document = this.document;
 		if(document == null) {
@@ -502,31 +494,19 @@ public class ElementImpl extends NodeImpl implements Element {
 		this.appendChild(textNode);
 	}
 
-	protected void appendInnerTextImpl(StringBuffer buffer) {
-		ArrayList nl = this.nodeList;
-		if (nl == null) {
-			return;
-		}
-		int size = nl.size();
-		if (size == 0) {
-			return;
-		}
-		for (int i = 0; i < size; i++) {
-			Node child = (Node) nl.get(i);
-			if (child instanceof ElementImpl) {
-				((ElementImpl) child).appendInnerTextImpl(buffer);
-			}
-			if(child instanceof Comment) {
-				// skip
-			}
-			else if (child instanceof Text) {
-				buffer.append(((Text) child).getTextContent());
-			}
-		}
-	}
-
 	protected Node createSimilarNode() {
 		HTMLDocumentImpl doc = (HTMLDocumentImpl) this.document;
 		return doc == null ? null : doc.createElement(this.getTagName());
 	}
+	
+	protected String htmlEncodeChildText(String text) {
+		if(org.lobobrowser.html.parser.HtmlParser.isDecodeEntities(this.name)) {
+			return Strings.strictHtmlEncode(text, false);			
+		}
+		else {
+			return text;
+		}
+	}
+	
+
 }
