@@ -60,22 +60,38 @@ public class HTMLBaseFontElementImpl extends HTMLAbstractUIElement implements
 	}
 	
 	protected RenderState createRenderState(RenderState prevRenderState) {
-		String face = this.getAttribute("face");
 		String size = this.getAttribute("size");
-		String color = this.getAttribute("color");
-		if(face != null) {
-			prevRenderState = new FontNameRenderState(prevRenderState, face);
-		}
 		if(size != null) {
 			int fontNumber = HtmlValues.getFontNumberOldStyle(size, prevRenderState);
 			float fontSize = HtmlValues.getFontSize(fontNumber);
-			prevRenderState = new FontSizeRenderState(prevRenderState, fontSize);
 			prevRenderState = new BaseFontRenderState(prevRenderState, fontNumber);
 		}
-		if(color != null) {
-			prevRenderState = new ColorRenderState(prevRenderState, ColorFactory.getInstance().getColor(color));
-		}
-		return prevRenderState;
+		return super.createRenderState(prevRenderState);
 	}
 
+    protected AbstractCSS2Properties createDefaultStyleSheet() {
+        String fontFamily = this.getAttribute("face");
+        String color = this.getAttribute("color");
+        String size = this.getAttribute("size");
+        ModelNode parentModelNode = this.getParentModelNode();
+        RenderState parentRS = parentModelNode == null ? null : parentModelNode.getRenderState();
+        String fontSize = null;
+        if(parentRS != null) {
+            int fontNumber = HtmlValues.getFontNumberOldStyle(size, parentRS);
+            fontSize = HtmlValues.getFontSizeSpec(fontNumber);
+        }        
+        ComputedCSS2Properties css = new ComputedCSS2Properties(this);
+        if(fontSize != null) {
+            css.internalSetLC("font-size", fontSize);
+        }
+        if(fontFamily != null) {
+            css.internalSetLC("font-family", fontFamily);
+        }
+        if(color != null) {
+            css.internalSetLC("color", color);
+        }
+        return css;
+    }
+
+	    
 }
