@@ -28,6 +28,6 @@ public class HTMLBlockQuoteElementImpl extends HTMLAbstractUIElement {
 	}
 
 	protected RenderState createRenderState(RenderState prevRenderState) {
-		return new BlockRenderState(prevRenderState, this);
+		return new BlockQuoteRenderState(prevRenderState, this);
 	}
 }
