@@ -42,4 +42,30 @@ public class HTMLDivElementImpl extends HTMLAbstractUIElement implements
 	protected RenderState createRenderState(RenderState prevRenderState) {
 		return new BlockRenderState(prevRenderState, this);
 	}
+	
+	protected void appendInnerTextImpl(StringBuffer buffer) {
+		int length = buffer.length();
+		int lineBreaks;
+		if(length == 0) {
+			lineBreaks = 2;
+		}
+		else {
+			int start = length - 2;
+			if (start < 0) {
+				start = 0;
+			}
+			lineBreaks = 0;
+			for(int i = start; i < length; i++) {
+				char ch = buffer.charAt(i);
+				if(ch == '\n') {
+					lineBreaks++;
+				}
+			}
+		}
+		for(int i = 0; i < 1 - lineBreaks; i++) {
+			buffer.append("\r\n");
+		}
+		super.appendInnerTextImpl(buffer);
+		buffer.append("\r\n");
+	}
 }
