@@ -355,7 +355,8 @@ public class HTMLDocumentImpl extends NodeImpl implements HTMLDocument, Document
 	}
 
 	/**
-	 * Loads the document from the reader provided when it was constructed.
+	 * Loads the document from the reader provided when the
+	 * current instance of <code>HTMLDocumentImpl</code> was constructed.
 	 * It then closes the reader.
 	 * @throws IOException
 	 * @throws SAXException
@@ -824,6 +825,29 @@ public class HTMLDocumentImpl extends NodeImpl implements HTMLDocument, Document
 		}
 		this.allInvalidated();
 	}
+
+	public void allInvalidated(boolean forgetRenderStates) {
+		if(forgetRenderStates) {
+			synchronized(this.treeLock) {
+				this.styleSheetAggregator = null;
+				// Need to invalidate all children up to
+				// this point.
+				this.forgetRenderState();
+				//TODO: this might be ineffcient.
+				ArrayList nl = this.nodeList;
+				if(nl != null) {
+					Iterator i = nl.iterator();
+					while(i.hasNext()) {
+						Object node = i.next();
+						if(node instanceof HTMLElementImpl) {
+							((HTMLElementImpl) node).forgetStyle(true);
+						}
+					}
+				}
+			}
+		}
+		this.allInvalidated();
+	}
 	
 	public Collection getStyleSheets(){
 		return this.styleSheets;
@@ -1033,6 +1057,10 @@ public class HTMLDocumentImpl extends NodeImpl implements HTMLDocument, Document
 		}
 	}
 
+	/**
+	 * Informs listeners that the whole document has been
+	 * invalidated.
+	 */
 	public void allInvalidated() {
 		ArrayList listenersList = this.documentNotificationListeners;
 		int size;
@@ -1278,7 +1306,7 @@ public class HTMLDocumentImpl extends NodeImpl implements HTMLDocument, Document
 			if(!(node instanceof Element)) {
 				return false;
 			}
-			String n = this.name;
+			String n = this.name;			
 			return n.equalsIgnoreCase(((Element) node).getTagName());
 		}
 	}
