@@ -320,4 +320,17 @@ public abstract class HTMLElementBuilder {
 			return new HTMLNonStandardElement(name);
 		}
 	}
+	
+	public static class Sup extends HTMLElementBuilder {
+		public HTMLElementImpl build(String name) {
+			return new HTMLSuperscriptElementImpl(name,1);
+		}
+	}
+
+	public static class Sub extends HTMLElementBuilder {
+		public HTMLElementImpl build(String name) {
+			return new HTMLSuperscriptElementImpl(name,-1);
+		}
+	}
+
 }
