@@ -97,6 +97,7 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 		}
 		// Can't do the following in synchronized block (reverse locking order with document).
 		// First, add declarations from stylesheet
+		sds = this.createDefaultStyleSheet();
 		sds = this.addStyleSheetDeclarations(sds, this.getPseudoNames());
 		// Now add local style if any.
 		AbstractCSS2Properties localStyle = this.getStyle();
@@ -152,6 +153,11 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 		return sds;
 	}
 
+	protected AbstractCSS2Properties createDefaultStyleSheet() {
+	    // Override to provide element defaults.
+	    return null;
+	}
+	
 	private Map computedStyles;
 	
 	public AbstractCSS2Properties getComputedStyle(String pseudoElement) {
@@ -170,7 +176,8 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 		// Can't do the following in synchronized block (reverse locking order with document).
 		// First, add declarations from stylesheet
 		Set pes = pseudoElement.length() == 0 ? null : Collections.singleton(pseudoElement);
-		AbstractCSS2Properties sds = this.addStyleSheetDeclarations(null, pes);
+		AbstractCSS2Properties sds = this.createDefaultStyleSheet();
+		sds = this.addStyleSheetDeclarations(sds, pes);
 		// Now add local style if any.
 		AbstractCSS2Properties localStyle = this.getStyle();
 		if(sds == null) {
@@ -245,8 +252,7 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 	}
 	
 	public boolean getAttributeAsBoolean(String name) {
-		String value = this.getAttribute(name);
-		return name.equalsIgnoreCase(value);
+		return this.getAttribute(name) != null;
 	}
 	
 	protected void assignAttributeField(String normalName, String value) {
@@ -450,7 +456,7 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 			return null;
 		}
 		StyleSheetAggregator ssa = doc.getStyleSheetAggregator();
-		return ssa.getStyleDeclarations(this, elementName, id, className, pseudoNames);
+		return ssa.getActiveStyleDeclarations(this, elementName, id, className, pseudoNames);
 	}
 	
 	public void informInvalid() {
@@ -517,6 +523,52 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 		}
 	}
 
+	public HTMLElementImpl getParentWithClass(String elementTL, String classTL) {
+		Object nodeObj = this.getParentNode();
+		if(nodeObj instanceof HTMLElementImpl) {
+			HTMLElementImpl parentElement = (HTMLElementImpl) nodeObj;
+			String pelementTL = parentElement.getTagName().toLowerCase();
+			if(("*".equals(elementTL) || elementTL.equals(pelementTL)) && parentElement.classMatch(classTL)) {
+				return parentElement;
+			}
+		}
+		return null;
+	}
+
+	public HTMLElementImpl getPreceedingSiblingElement() {
+		Node parentNode = this.getParentNode();
+		if(parentNode == null) {
+			return null;
+		}
+		NodeList childNodes = parentNode.getChildNodes();
+		if(childNodes == null) {
+			return null;
+		}
+		int length = childNodes.getLength();
+		HTMLElementImpl priorElement = null;
+		for(int i = 0; i < length; i++) {
+			Node child = childNodes.item(i);
+			if(child == this) {
+				return priorElement;
+			}
+			if(child instanceof HTMLElementImpl) {
+				priorElement = (HTMLElementImpl) child;
+			}
+		}
+		return null;
+	}
+	
+	public HTMLElementImpl getPreceedingSiblingWithClass(String elementTL, String classTL) {
+		HTMLElementImpl psibling = this.getPreceedingSiblingElement();
+		if(psibling != null) {
+			String pelementTL = psibling.getTagName().toLowerCase();
+			if(("*".equals(elementTL) || elementTL.equals(pelementTL)) && psibling.classMatch(classTL)) {
+				return psibling;
+			}
+		}
+		return null;
+	}
+
 	public HTMLElementImpl getAncestorWithId(String elementTL, String idTL) {
 		Object nodeObj = this.getParentNode();
 		if(nodeObj instanceof HTMLElementImpl) {
@@ -534,6 +586,33 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 		}
 	}
 
+	public HTMLElementImpl getParentWithId(String elementTL, String idTL) {
+		Object nodeObj = this.getParentNode();
+		if(nodeObj instanceof HTMLElementImpl) {
+			HTMLElementImpl parentElement = (HTMLElementImpl) nodeObj;
+			String pelementTL = parentElement.getTagName().toLowerCase();
+			String pid = parentElement.getId();
+			String pidTL = pid == null ? null : pid.toLowerCase();
+			if(("*".equals(elementTL) || elementTL.equals(pelementTL)) && idTL.equals(pidTL)) {
+				return parentElement;
+			}
+		}
+		return null;
+	}
+
+	public HTMLElementImpl getPreceedingSiblingWithId(String elementTL, String idTL) {
+		HTMLElementImpl psibling = this.getPreceedingSiblingElement();
+		if(psibling != null) {
+			String pelementTL = psibling.getTagName().toLowerCase();
+			String pid = psibling.getId();
+			String pidTL = pid == null ? null : pid.toLowerCase();
+			if(("*".equals(elementTL) || elementTL.equals(pelementTL)) && idTL.equals(pidTL)) {
+				return psibling;
+			}
+		}
+		return null;
+	}
+
 	public HTMLElementImpl getAncestor(String elementTL) {
 		Object nodeObj = this.getParentNode();
 		if(nodeObj instanceof HTMLElementImpl) {
@@ -551,7 +630,36 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 			return null;
 		}
 	}	
-	
+
+	public HTMLElementImpl getParent(String elementTL) {
+		Object nodeObj = this.getParentNode();
+		if(nodeObj instanceof HTMLElementImpl) {
+			HTMLElementImpl parentElement = (HTMLElementImpl) nodeObj;
+			if("*".equals(elementTL)) {
+				return parentElement;
+			}
+			String pelementTL = parentElement.getTagName().toLowerCase();
+			if(elementTL.equals(pelementTL)) {
+				return parentElement;
+			}
+		}
+		return null;
+	}	
+
+	public HTMLElementImpl getPreceedingSibling(String elementTL) {
+		HTMLElementImpl psibling = this.getPreceedingSiblingElement();
+		if(psibling != null) {
+			if("*".equals(elementTL)) {
+				return psibling;
+			}
+			String pelementTL = psibling.getTagName().toLowerCase();
+			if(elementTL.equals(pelementTL)) {
+				return psibling;
+			}
+		}
+		return null;
+	}	
+
 	protected Object getAncestorForJavaClass(Class javaClass) {
 		Object nodeObj = this.getParentNode();
 		if(nodeObj == null || javaClass.isInstance(nodeObj)) {
@@ -613,7 +721,7 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 					buffer.append(' ');
 					buffer.append(entry.getKey());
 					buffer.append("=\"");
-					buffer.append(Strings.strictHtmlEncode(value));
+					buffer.append(Strings.strictHtmlEncode(value, true));
 					buffer.append("\"");				
 				}
 			}
@@ -637,39 +745,25 @@ public class HTMLElementImpl extends ElementImpl implements HTMLElement, CSS2Pro
 	}
 	
 	public int getOffsetTop() {
-		AbstractCSS2Properties style = this.getCurrentStyle();
-		if(style == null) {
-			return 0;
-		}
-		String topText = style.getTop();
-		return topText == null ? 0 : HtmlValues.getPixelSize(topText, this.getRenderState(), 0);
+		//TODO: Sometimes this can be called while parsing, and
+		//browsers generally give the right answer.
+		UINode uiNode = this.getUINode();
+		return uiNode == null ? 0 : uiNode.getBoundsRelativeToBlock().y;
 	}
 	
 	public int getOffsetLeft() {
-		AbstractCSS2Properties style = this.getCurrentStyle();
-		if(style == null) {
-			return 0;
-		}
-		String leftText = style.getLeft();
-		return leftText == null ? 0 : HtmlValues.getPixelSize(leftText, this.getRenderState(), 0);
+		UINode uiNode = this.getUINode();
+		return uiNode == null ? 0 : uiNode.getBoundsRelativeToBlock().x;
 	}
 
 	public int getOffsetWidth() {
-		AbstractCSS2Properties style = this.getCurrentStyle();
-		if(style == null) {
-			return 0;
-		}
-		String valueText = style.getWidth();
-		return valueText == null ? 0 : HtmlValues.getPixelSize(valueText, this.getRenderState(), 0);
+		UINode uiNode = this.getUINode();
+		return uiNode == null ? 0 : uiNode.getBoundsRelativeToBlock().width;
 	}
 
 	public int getOffsetHeight() {
-		AbstractCSS2Properties style = this.getCurrentStyle();
-		if(style == null) {
-			return 0;
-		}
-		String valueText = style.getHeight();
-		return valueText == null ? 0 : HtmlValues.getPixelSize(valueText, this.getRenderState(), 0);
+		UINode uiNode = this.getUINode();
+		return uiNode == null ? 0 : uiNode.getBoundsRelativeToBlock().height;
 	}
 
 	public AbstractCSS2Properties getParentStyle() {
