@@ -20,10 +20,11 @@
 */
 package org.lobobrowser.html.domimpl;
 
+import org.lobobrowser.html.style.AbstractCSS2Properties;
 import org.lobobrowser.html.style.BaseFontRenderState;
 import org.lobobrowser.html.style.ColorRenderState;
+import org.lobobrowser.html.style.ComputedCSS2Properties;
 import org.lobobrowser.html.style.FontNameRenderState;
-import org.lobobrowser.html.style.FontSizeRenderState;
 import org.lobobrowser.html.style.HtmlValues;
 import org.lobobrowser.html.style.RenderState;
 import org.lobobrowser.util.gui.ColorFactory;
@@ -60,20 +61,33 @@ public class HTMLFontElementImpl extends HTMLAbstractUIElement implements
 	}
 
 	protected RenderState createRenderState(RenderState prevRenderState) {
-		String face = this.getAttribute("face");
-		String size = this.getAttribute("size");
-		String color = this.getAttribute("color");
-		if(face != null) {
-			prevRenderState = new FontNameRenderState(prevRenderState, face);
-		}
-		if(size != null) {
-			int fontNumber = HtmlValues.getFontNumberOldStyle(size, prevRenderState);
-			float fontSize = HtmlValues.getFontSize(fontNumber);
-			prevRenderState = new FontSizeRenderState(prevRenderState, fontSize);
-		}
-		if(color != null) {
-			prevRenderState = new ColorRenderState(prevRenderState, ColorFactory.getInstance().getColor(color));
-		}
-		return prevRenderState;
+	    return super.createRenderState(prevRenderState);
 	}
+	
+    protected AbstractCSS2Properties createDefaultStyleSheet() {
+        String fontFamily = this.getAttribute("face");
+        String color = this.getAttribute("color");
+        String size = this.getAttribute("size");
+        String fontSize = null;
+        if(size != null) {
+            ModelNode parentModelNode = this.getParentModelNode();
+            RenderState parentRS = parentModelNode == null ? null : parentModelNode.getRenderState();
+            if(parentRS != null) {
+                int fontNumber = HtmlValues.getFontNumberOldStyle(size, parentRS);
+                fontSize = HtmlValues.getFontSizeSpec(fontNumber);
+            }        
+        }
+        ComputedCSS2Properties css = new ComputedCSS2Properties(this);
+        if(fontSize != null) {
+            css.internalSetLC("font-size", fontSize);
+        }
+        if(fontFamily != null) {
+            css.internalSetLC("font-family", fontFamily);
+        }
+        if(color != null) {
+            css.internalSetLC("color", color);
+        }
+        return css;
+    }
+
 }
