@@ -34,18 +34,19 @@ public class HTMLFontSizeChangeElementImpl extends HTMLAbstractUIElement {
 	}
 
 	protected RenderState createRenderState(RenderState prevRenderState) {
-		int fontSize;
-		if(prevRenderState == null) {
-			fontSize = 13;
-		}
-		else {
-			fontSize = prevRenderState.getFont().getSize();
-			fontSize += (this.fontChange * 2);
-			if(fontSize < 1) {
-				fontSize = 1;
-			}
-		}
-		prevRenderState = new FontSizeRenderState(prevRenderState, fontSize);
 		return super.createRenderState(prevRenderState);
 	}
+	
+    protected AbstractCSS2Properties createDefaultStyleSheet() {
+        ModelNode parentModelNode = this.getParentModelNode();
+        RenderState parentRS = parentModelNode == null ? null : parentModelNode.getRenderState();
+        String fontSize = null;
+        int prevFontSize = parentRS != null ? parentRS.getFont().getSize() : HtmlValues.DEFAULT_FONT_SIZE_INT;
+        int newFontSize = prevFontSize + (this.fontChange * 2);
+        fontSize = newFontSize + "px";
+        ComputedCSS2Properties css = new ComputedCSS2Properties(this);
+        css.internalSetLC("font-size", fontSize);
+        return css;
+    }
+
 }
