@@ -66,6 +66,6 @@ public class HTMLOListElementImpl extends HTMLAbstractUIElement implements
 	}
 
 	protected RenderState createRenderState(RenderState prevRenderState) {
-		return new DisplayRenderState(prevRenderState, this, RenderState.DISPLAY_LIST_ITEM);
+		return new ListRenderState(prevRenderState, this);
 	}
 }
