@@ -2,23 +2,22 @@ package org.lobobrowser.html.domimpl;
 
 import org.w3c.dom.DOMException;
 import org.w3c.dom.Node;
-import org.w3c.dom.html2.HTMLOptionsCollection;
+import org.w3c.dom.html2.*;
 
 public class HTMLOptionsCollectionImpl extends DescendentHTMLCollection implements HTMLOptionsCollection {
-	private static final NodeFilter OPTION_FILTER = new OptionFilter();
+	public static final NodeFilter OPTION_FILTER = new OptionFilter();
 	
 	public HTMLOptionsCollectionImpl(HTMLElementImpl selectElement) {
-		super(selectElement, OPTION_FILTER);
+		super(selectElement, OPTION_FILTER, selectElement.treeLock, false);
 	}
 	
 	public void setLength(int length) throws DOMException {
-		//TODO: ???
 		throw new UnsupportedOperationException();
 	}
 
 	private static class OptionFilter implements NodeFilter {
 		public boolean accept(Node node) {
-			return "option".equalsIgnoreCase(node.getNodeName());
+			return node instanceof HTMLOptionElement;
 		}
 	}
 }
