@@ -44,6 +44,6 @@ public class HTMLPElementImpl extends HTMLAbstractUIElement implements
 	}
 
 	protected RenderState createRenderState(RenderState prevRenderState) {
-		return new BlockRenderState(prevRenderState, this);
+		return new ParagraphRenderState(prevRenderState, this);
 	}
 }
