@@ -195,13 +195,17 @@ public class HTMLScriptElementImpl extends HTMLElementImpl implements
 						logger.info("addNotify(): Evaluated (or attempted to evaluate) Javascript in " + (time2 - time1) + " ms.");
 					}
 				} catch(EcmaError ecmaError) {
-					this.warn("Javascript error at " + ecmaError.getSourceName() + ":" + ecmaError.getLineNumber() + ": " + ecmaError.getMessage());
+					logger.log(Level.WARNING, "Javascript error at " + ecmaError.getSourceName() + ":" + ecmaError.getLineNumber() + ": " + ecmaError.getMessage(), ecmaError);
 				} catch(Throwable err) {
-					this.warn("Unable to evaluate Javascript code", err);
+					logger.log(Level.WARNING, "Unable to evaluate Javascript code", err);
 				}
 			} finally {
 				Context.exit();
 			}
 		}
 	}
+	
+	protected void appendInnerTextImpl(StringBuffer buffer) {
+		// nop
+	}
 }
