@@ -33,6 +33,8 @@ import com.steadystate.css.parser.CSSOMParser;
 
 public class HTMLStyleElementImpl extends HTMLElementImpl implements
 		HTMLStyleElement {
+	private CSSStyleSheet styleSheet; 
+	
 	public HTMLStyleElementImpl() {
 		super("STYLE", true);
 	}
@@ -48,6 +50,10 @@ public class HTMLStyleElementImpl extends HTMLElementImpl implements
 	
 	public void setDisabled(boolean disabled) {
 		this.disabled = disabled;
+		CSSStyleSheet sheet = this.styleSheet;
+		if(sheet != null) {
+			sheet.setDisabled(disabled);
+		}
 	}
 	
 	public String getMedia() {
@@ -70,10 +76,14 @@ public class HTMLStyleElementImpl extends HTMLElementImpl implements
 		if(org.lobobrowser.html.parser.HtmlParser.MODIFYING_KEY.equals(key) && data != Boolean.TRUE) {
 			this.processStyle();
 		}
+		else if(com.steadystate.css.dom.CSSStyleSheetImpl.KEY_DISABLED_CHANGED.equals(key)) {
+			this.informDocumentInvalid();
+		}
 		return super.setUserData(key, data, handler);
 	}
 
 	protected void processStyle() {
+		this.styleSheet = null;
 		if(CSSUtilities.matchesMedia(this.getMedia(), this.getUserAgentContext())) {
 			String text = this.getRawInnerText(true);
 			if(text != null && !"".equals(text)) {
@@ -87,10 +97,22 @@ public class HTMLStyleElementImpl extends HTMLElementImpl implements
 					sheet.setOwnerNode(this);
 					sheet.setHref(baseURI);
 					doc.addStyleSheet(sheet);
+					this.styleSheet = sheet;
+					if(sheet instanceof CSSStyleSheetImpl) {
+						CSSStyleSheetImpl sheetImpl = (CSSStyleSheetImpl) sheet; 
+						sheetImpl.setDisabledOnly(this.disabled);
+					}
+					else {
+						sheet.setDisabled(this.disabled);
+					}
 				} catch(Throwable err) {
 					this.warn("Unable to parse style sheet", err);
 				}
 			}
 		}
 	}
+	
+	protected void appendInnerTextImpl(StringBuffer buffer) {
+		// nop
+	}
 }
