@@ -39,6 +39,7 @@ public interface InputContext {
 	public int getTabIndex();
 	public void setTabIndex(int tabIndex);
 	public String getValue();
+	public String[] getValues();
 	public void setValue(String value);
 	public void blur();
 	public void focus();
@@ -48,8 +49,6 @@ public interface InputContext {
 	public int getCols();
 	public void setRows(int rows);
 	public void setCols(int cols);
-	public boolean getMultiple();
-	public void setMultiple(boolean value);
 	public int getSelectedIndex();
 	public void setSelectedIndex(int value);
 	public int getVisibleSize();
