@@ -26,7 +26,6 @@ package org.lobobrowser.html.domimpl;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.*;
-import java.util.Map.Entry;
 import java.util.logging.*;
 
 import org.lobobrowser.html.*;
@@ -1057,6 +1056,14 @@ public abstract class NodeImpl extends AbstractScriptableDelegate implements Nod
 		}
 	}
 
+	public void informDocumentInvalid() {
+		// This is called when an attribute or child changes.
+		HTMLDocumentImpl doc = (HTMLDocumentImpl) this.document;
+		if(doc != null) {
+			doc.allInvalidated(true);
+		}
+	}
+
 	private RenderState renderState = INVALID_RENDER_STATE;
 	
 	public RenderState getRenderState() {
@@ -1135,7 +1142,7 @@ public abstract class NodeImpl extends AbstractScriptableDelegate implements Nod
 				}
 				else if (child instanceof Text) {
 					String text = ((Text) child).getTextContent();
-					String encText = Strings.strictHtmlEncode(text);
+					String encText = this.htmlEncodeChildText(text);
 					buffer.append(encText);
 				}
 				else if (child instanceof ProcessingInstruction) {
@@ -1144,4 +1151,43 @@ public abstract class NodeImpl extends AbstractScriptableDelegate implements Nod
 			}
 		}
 	}
+	
+	protected String htmlEncodeChildText(String text) {
+		return Strings.strictHtmlEncode(text, false);
+	}
+
+	/**
+	 * Attempts to convert the subtree starting at this point to a close text
+	 * representation. BR elements are converted to line breaks, and so forth.
+	 */
+	public String getInnerText() {
+		StringBuffer buffer = new StringBuffer();
+		synchronized(this.treeLock) {
+			this.appendInnerTextImpl(buffer);
+		}
+		return buffer.toString();
+	}
+
+	protected void appendInnerTextImpl(StringBuffer buffer) {
+		ArrayList nl = this.nodeList;
+		if (nl == null) {
+			return;
+		}
+		int size = nl.size();
+		if (size == 0) {
+			return;
+		}
+		for (int i = 0; i < size; i++) {
+			Node child = (Node) nl.get(i);
+			if (child instanceof ElementImpl) {
+				((ElementImpl) child).appendInnerTextImpl(buffer);
+			}
+			if(child instanceof Comment) {
+				// skip
+			}
+			else if (child instanceof Text) {
+				buffer.append(((Text) child).getTextContent());
+			}
+		}
+	}
 }
