@@ -32,6 +32,7 @@ import javax.swing.*;
 import org.lobobrowser.html.*;
 import org.lobobrowser.html.domimpl.*;
 import org.lobobrowser.html.renderer.*;
+import org.lobobrowser.html.style.RenderState;
 import org.lobobrowser.util.*;
 import org.lobobrowser.util.gui.ColorFactory;
 import org.w3c.dom.*;
@@ -41,8 +42,7 @@ import java.util.*;
 /**
  * A Swing component that renders a HTML block, given 
  * by a DOM root or an internal element, typically a DIV.
- * This component <i>cannot</i> render FRAMESETs, which is done
- * by {@link FrameSetPanel}. <code>HtmlBlockPanel</code>
+ * This component <i>cannot</i> render FRAMESETs. <code>HtmlBlockPanel</code>
  * is used by {@link HtmlPanel} whenever the DOM
  * is determined <i>not</i> to be a FRAMESET.
  * @see HtmlPanel
@@ -60,6 +60,10 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 	protected RenderableSpot endSelection;
 	protected RBlock rblock;
 	protected int preferredWidth = -1;
+    protected Insets defaultMarginInsets = null;    
+    //protected Insets defaultPaddingInsets = null; 
+    protected int defaultOverflowX = RenderState.OVERFLOW_AUTO;
+    protected int defaultOverflowY = RenderState.OVERFLOW_SCROLL;
 
 	public HtmlBlockPanel(UserAgentContext pcontext, HtmlRendererContext rcontext, FrameContext frameContext) {
 		this(ColorFactory.TRANSPARENT, false, pcontext, rcontext, frameContext);
@@ -141,6 +145,18 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 		}
 	}
 
+	public void scrollBy(int xOffset, int yOffset) {
+		RBlock block = this.rblock;
+		if(block != null) {
+			if(xOffset != 0) {	
+				block.scrollBy(JScrollBar.HORIZONTAL, xOffset);
+			}
+			if(yOffset != 0) {
+				block.scrollBy(JScrollBar.VERTICAL, yOffset);
+			}
+		}
+	}
+
 	/**
 	 * Scrolls the body area to the node given, if it is
 	 * part of the current document.
@@ -279,9 +295,21 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 		if(pw != -1) {
 			final RBlock block = this.rblock;
 			if(block != null) {
-				// Potentially does layout outside of GUI thread.
-				// Do not provide default overflow.
-				block.layout(pw, 0, RBlock.OVERFLOW_NONE);
+			    // Layout should always be done in the GUI thread.
+			    if(EventQueue.isDispatchThread()) {
+			        block.layout(pw, 0, false, false, RenderState.OVERFLOW_VISIBLE, RenderState.OVERFLOW_VISIBLE, true);
+			    }
+			    else {
+			        try {
+			            EventQueue.invokeAndWait(new Runnable() {
+			                public void run() {
+			                    block.layout(pw, 0, false, false, RenderState.OVERFLOW_VISIBLE, RenderState.OVERFLOW_VISIBLE, true);			                
+			                }
+			            });
+			        } catch(Exception err) {
+			            logger.log(Level.SEVERE, "Unable to do preferred size layout.", err);
+			        }
+			    }
 				// Adjust for permanent vertical scrollbar.
 				int newPw = Math.max(block.width + block.getVScrollBarWidth(), pw);
 				return new Dimension(newPw, block.height);
@@ -306,11 +334,13 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 		}
 	}
 	
-	protected Insets defaultPaddingInsets = null;
-
-	public void setDefaultPaddingInsets(Insets insets) {
-		this.defaultPaddingInsets = insets;
-	}
+//	public void setDefaultPaddingInsets(Insets insets) {
+//		this.defaultPaddingInsets = insets;
+//		RBlock block = this.rblock;
+//		if(block != null) {
+//		    block.setDefaultPaddingInsets(insets);
+//		}
+//	}
 	
 	public int getFirstLineHeight() {
 		RBlock block = this.rblock;
@@ -348,8 +378,11 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 	 */
 	public void setRootNode(NodeImpl node) {
 		if(node != null) {
-			RBlock block = new RBlock(node, 0, this.ucontext, this.rcontext, this.frameContext, this, RBlock.OVERFLOW_VERTICAL);
-			block.setDefaultPaddingInsets(this.defaultPaddingInsets);
+			RBlock block = new RBlock(node, 0, this.ucontext, this.rcontext, this.frameContext, this);
+			block.setDefaultMarginInsets(this.defaultMarginInsets);
+			//block.setDefaultPaddingInsets(this.defaultPaddingInsets);
+			block.setDefaultOverflowX(this.defaultOverflowX);
+			block.setDefaultOverflowY(this.defaultOverflowY);
 			node.setUINode(block);
 			this.rblock = block;
 		}
@@ -544,9 +577,7 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 			RBlock block = this.rblock;
 			if(block != null) {
 				ModelNode rootNode = block.getModelNode();
-				block.layout(size.width, size.height);
-				// Must also adjust here
-				block.adjust(size.width, size.height, true, true, null);
+				block.layout(size.width, size.height, true, true, null, false);
 				//Only set origin
 				block.setOrigin(0, 0);
 				block.updateWidgetBounds(0, 0);
@@ -815,4 +846,60 @@ public class HtmlBlockPanel extends JComponent implements NodeRenderer, Renderab
 			}
 		}
 	}
+
+    public Insets getDefaultMarginInsets() {
+        return defaultMarginInsets;
+    }
+
+    /**
+     * Sets the default margin insets. Note that in the root block,
+     * the margin behaves like padding.
+     * @param defaultMarginInsets The default margin insets.
+     */
+    public void setDefaultMarginInsets(Insets defaultMarginInsets) {
+        if(!Objects.equals(this.defaultMarginInsets, defaultMarginInsets)) {
+            this.defaultMarginInsets = defaultMarginInsets;
+            RBlock block = this.rblock;
+            if(block != null) {
+                block.setDefaultMarginInsets(defaultMarginInsets);
+                block.relayoutIfValid();
+            }
+        }
+    }
+
+//    public Insets getDefaultPaddingInsets() {
+//        return defaultPaddingInsets;
+//    }
+//
+    public int getDefaultOverflowX() {
+        return defaultOverflowX;
+    }
+
+    public void setDefaultOverflowX(int defaultOverflowX) {
+        if(defaultOverflowX != this.defaultOverflowX) {
+            this.defaultOverflowX = defaultOverflowX;
+            RBlock block = this.rblock;
+            if(block != null) {
+                block.setDefaultOverflowX(defaultOverflowX);
+                block.relayoutIfValid();
+            }
+        }
+    } 
+
+    public int getDefaultOverflowY() {
+        return defaultOverflowY;
+    }
+
+    public void setDefaultOverflowY(int defaultOverflowY) {
+        if(this.defaultOverflowY != defaultOverflowY) {
+            this.defaultOverflowY = defaultOverflowY;
+            RBlock block = this.rblock;
+            if(block != null) {
+                block.setDefaultOverflowY(defaultOverflowY);
+                block.relayoutIfValid();
+            }
+        }
+    }
+    
+    
 }
\ No newline at end of file
