@@ -33,6 +33,7 @@ import org.lobobrowser.html.*;
 import org.lobobrowser.html.domimpl.*;
 import org.lobobrowser.html.parser.*;
 import org.lobobrowser.html.renderer.*;
+import org.lobobrowser.html.style.RenderState;
 import org.lobobrowser.util.EventDispatch2;
 import org.lobobrowser.util.gui.WrapperLayout;
 import org.w3c.dom.Document;
@@ -45,7 +46,9 @@ import org.w3c.dom.html2.*;
  * either {@link HtmlBlockPanel} or {@link FrameSetPanel}
  * internally, depending on whether the document
  * is determined to be a FRAMESET or not.
- * @author J. H. S.
+ * <p>
+ * Invoke method {@link #setDocument(Document, HtmlRendererContext)}
+ * in order to schedule a document for rendering.
  */
 public class HtmlPanel extends JComponent implements FrameContext {
 	private final EventDispatch2 selectionDispatch = new SelectionDispatch();
@@ -58,9 +61,13 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	private volatile NodeRenderer nodeRenderer = null;
 	private volatile NodeImpl rootNode;
 	private volatile int preferredWidth = -1;
+	private volatile Insets defaultMarginInsets = new Insets(8, 8, 8, 8);
+	private volatile int defaultOverflowX = RenderState.OVERFLOW_AUTO;
+	private volatile int defaultOverflowY = RenderState.OVERFLOW_SCROLL;
 
 	protected volatile HtmlBlockPanel htmlBlockPanel;
 	protected volatile FrameSetPanel frameSetPanel;
+	
 
 	/**
 	 * Constructs an <code>HtmlPanel</code>.
@@ -167,9 +174,11 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	private void setUpAsBlock(UserAgentContext ucontext, HtmlRendererContext rcontext) {
 		HtmlBlockPanel shp = this.createHtmlBlockPanel(ucontext, rcontext);
 		shp.setPreferredWidth(this.preferredWidth);
+		shp.setDefaultMarginInsets(this.defaultMarginInsets);
+		shp.setDefaultOverflowX(this.defaultOverflowX);
+		shp.setDefaultOverflowY(this.defaultOverflowY);
 		this.htmlBlockPanel = shp;
 		this.frameSetPanel = null;
-		shp.setDefaultPaddingInsets(new Insets(8, 8, 8, 8));
 		this.removeAll();
 		this.add(shp);
 		this.nodeRenderer = shp;
@@ -223,10 +232,30 @@ public class HtmlPanel extends JComponent implements FrameContext {
 		}
 	}
 
+	public void scrollBy(final int x, final int y) {
+		if(EventQueue.isDispatchThread()) {
+			this.scrollByImpl(x, y);
+		}
+		else {
+			EventQueue.invokeLater(new Runnable() {
+				public void run() {
+					scrollByImpl(x, y);					
+				}
+			});
+		}
+	}
+
 	private void scrollImpl(int x, int y) {
 		this.scrollTo(new Rectangle(x, y, 16, 16), false, false);		
 	}
-	
+
+	private void scrollByImpl(int xOffset, int yOffset) {
+		HtmlBlockPanel bp = this.htmlBlockPanel;
+		if(bp != null) {
+			bp.scrollBy(xOffset, yOffset);
+		}
+	}
+
 	/**
 	 * Clears the current document if any.
 	 * If called outside the GUI thread, the operation
@@ -374,6 +403,7 @@ public class HtmlPanel extends JComponent implements FrameContext {
 	 * @param uri A base URI used to resolve item URIs.
 	 * @param rcontext The {@link HtmlRendererContext} instance.
 	 * @see org.lobobrowser.html.test.SimpleHtmlRendererContext
+	 * @see #setDocument(Document, HtmlRendererContext)
 	 */
 	public void setHtml(String htmlSource, String uri, HtmlRendererContext rcontext) {
 		try {
@@ -608,6 +638,49 @@ public class HtmlPanel extends JComponent implements FrameContext {
 		this.selectionDispatch.removeListener(listener);
 	}
 	
+    /**
+     * Sets the default margin insets. Note that in the root block,
+     * the margin behaves like padding.
+     * <p>
+     * This method has no effect on FRAMESETs.
+     * @param insets The default margin insets.
+     */
+    public void setDefaultMarginInsets(Insets insets) {
+        this.defaultMarginInsets = insets;
+        HtmlBlockPanel block = this.htmlBlockPanel;
+        if(block != null) {
+            block.setDefaultMarginInsets(insets);
+        }           
+    }
+
+    /**
+     * Sets the default horizontal overflow.
+     * <p>
+     * This method has no effect on FRAMESETs.
+     * @param overflow See {@link org.lobobrowser.html.style.RenderState}.
+     */
+    public void setDefaultOverflowX(int overflow) {
+        this.defaultOverflowX = overflow;
+        HtmlBlockPanel block = this.htmlBlockPanel;
+        if(block != null) {
+            block.setDefaultOverflowX(overflow);
+        }                   
+    }
+    
+    /**
+     * Sets the default vertical overflow.
+     * <p>
+     * This method has no effect on FRAMESETs.
+     * @param overflow See {@link org.lobobrowser.html.style.RenderState}.
+     */
+    public void setDefaultOverflowY(int overflow) {
+        this.defaultOverflowY = overflow;
+        HtmlBlockPanel block = this.htmlBlockPanel;
+        if(block != null) {
+            block.setDefaultOverflowY(overflow);
+        }                           
+    }
+	
 	private ArrayList notifications = new ArrayList(1);
 	
 	private void addNotification(DocumentNotification notification) {
