@@ -47,16 +47,25 @@ public class Event extends AbstractScriptableDelegate {
 		this.inputEvent = keyEvent;
 	}
 
+	public Event(String type, HTMLElement srcElement) {
+		this.type = type;
+		this.srcElement = srcElement;
+		this.inputEvent = null;
+	}
+
 	public boolean getAltKey() {
-		return this.inputEvent.isAltDown();
+		InputEvent ie = this.inputEvent;
+		return ie == null ? false : ie.isAltDown();
 	}
 	
 	public boolean getShiftKey() {
-		return this.inputEvent.isShiftDown();
+		InputEvent ie = this.inputEvent;
+		return ie == null ? false : ie.isShiftDown();
 	}
 	
 	public boolean getCtrlKey() {
-		return this.inputEvent.isControlDown();
+		InputEvent ie = this.inputEvent;
+		return ie == null ? false : ie.isControlDown();
 	}
 	
 	public int getButton() {
