@@ -11,8 +11,7 @@ public class Location extends AbstractScriptableDelegate {
 	private static final Logger logger = Logger.getLogger(Location.class.getName());
 	private final Window window;
 
-	public Location(final Window window) {
-		super();
+	Location(final Window window) {
 		this.window = window;
 	}
 	
