@@ -30,7 +30,7 @@ public class Navigator extends AbstractScriptableDelegate {
 	/**
 	 * @param context
 	 */
-	public Navigator(UserAgentContext context) {
+	Navigator(UserAgentContext context) {
 		super();
 		this.context = context;
 	}
