@@ -47,6 +47,8 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 	private static final Map CONTEXT_WINDOWS = new WeakHashMap();
 	//private static final JavaClassWrapper IMAGE_WRAPPER = JavaClassWrapperFactory.getInstance().getClassWrapper(Image.class);	
 	private static final JavaClassWrapper XMLHTTPREQUEST_WRAPPER = JavaClassWrapperFactory.getInstance().getClassWrapper(XMLHttpRequest.class);	
+
+	private static int timerIdCounter = 0;
 	
 	private final HtmlRendererContext rcontext;
 	private final UserAgentContext uaContext;
@@ -65,10 +67,20 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		this.uaContext = uaContext;
 	}
 
+	private static int generateTimerID() {
+	    synchronized(logger) {
+	        return timerIdCounter++;
+	    }
+	}
+	
 	public HtmlRendererContext getHtmlRendererContext() {
 		return this.rcontext;
 	}
 	
+	public UserAgentContext getUserAgentContext() {
+	    return this.uaContext;
+	}
+	
 	private void clearState() {
 		Scriptable s = this.getWindowScope();
 		if(s != null) {
@@ -116,41 +128,54 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		return this.document;
 	}
 	
-	private void putTask(Integer timeoutID, Timer timer) {
+	private void putAndStartTask(Integer timeoutID, Timer timer, Object retained) {
+		TaskWrapper oldTaskWrapper = null;
 		synchronized(this) {
 			Map taskMap = this.taskMap;
 			if(taskMap == null) {
-				taskMap = new HashMap();
+				taskMap = new HashMap(4);
 				this.taskMap = taskMap;
 			}
-			taskMap.put(timeoutID, timer);
-		}		
+			else {
+				oldTaskWrapper = (TaskWrapper) taskMap.get(timeoutID);
+			}
+			taskMap.put(timeoutID, new TaskWrapper(timer, retained));
+		}	
+		// Do this outside synchronized block, just in case.
+		if(oldTaskWrapper != null) {
+			oldTaskWrapper.timer.stop();
+		}
+		timer.start();
 	}
 	
 	private void forgetTask(Integer timeoutID, boolean cancel) {
+		TaskWrapper oldTimer = null;
 		synchronized(this) {
 			Map taskMap = this.taskMap;
 			if(taskMap != null) {
-				Timer timer = (Timer) taskMap.remove(timeoutID);
-				if(timer != null && cancel) {
-					timer.stop();
-				}
+				oldTimer = (TaskWrapper) taskMap.remove(timeoutID);
 			}			
 		}
+		if(oldTimer != null && cancel) {
+			oldTimer.timer.stop();
+		}
 	}
 	
 	private void forgetAllTasks() {
+		TaskWrapper[] oldTaskWrappers = null;
 		synchronized(this) {
 			Map taskMap = this.taskMap;
 			if(taskMap != null) {
-				Iterator i = taskMap.values().iterator();
-				while(i.hasNext()) {
-					Timer timer = (Timer) i.next();
-					timer.stop();
-				}
+				oldTaskWrappers = (TaskWrapper[]) taskMap.values().toArray(new TaskWrapper[0]);
 				this.taskMap = null;
 			}
 		}
+		if(oldTaskWrappers != null) {
+			for(int i = 0; i < oldTaskWrappers.length; i++) {
+				TaskWrapper taskWrapper = oldTaskWrappers[i];
+				taskWrapper.timer.stop();
+			}
+		}
 	}
 
 	//	private Timer getTask(Long timeoutID) {
@@ -174,32 +199,22 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
       *      definition</a>
       * @todo Make proper and refactore with {@link Window#setTimeout(Function, double)}.
       */
-     public int setInterval(final Function aFunction, double aTimeInMs)
+     public int setInterval(final Function aFunction, final double aTimeInMs)
      {
-         final int timeID = ID.generateInt();
-         final Integer timeIDInt = new Integer(timeID);
-         ActionListener task = new ActionListener() {
-             public void actionPerformed(ActionEvent e)
-             {
-                 // This executes in the GUI thread and that's good.
-                 Window.this.forgetTask(timeIDInt, false);
-                 HTMLDocumentImpl doc = (HTMLDocumentImpl) document;
-                 if (doc == null)
-                 {
-                     //  FIXME : Make a proper method to stop all timer launched via setInterval, other than window.setDocument(null)
-                     throw new IllegalStateException("Cannot perform operation when document is unset.");
-                 }
-                 Executor.executeFunction(getWindowScope(), aFunction, doc.getDocumentURL(), uaContext);
-             }
-         };
          if (aTimeInMs > Integer.MAX_VALUE || aTimeInMs < 0)
          {
              throw new IllegalArgumentException("Timeout value " + aTimeInMs + " is not supported.");
          }
-         Timer timer = new Timer((int) aTimeInMs, task);
+         final int timeID = generateTimerID();
+         final Integer timeIDInt = new Integer(timeID);
+         ActionListener task = new FunctionTimerTask(this, timeIDInt, aFunction, false);
+         int t = (int) aTimeInMs;
+         if(t < 1) {
+             t = 1;
+         }
+         Timer timer = new Timer(t, task);
          timer.setRepeats(true); // The only difference with setTimeout
-         timer.start();
-         this.putTask(timeIDInt, timer);
+         this.putAndStartTask(timeIDInt, timer, aFunction);
          return timeID;
      }
  
@@ -215,30 +230,20 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
       */
      public int setInterval(final String aExpression, double aTimeInMs)
      {
-    	 final int timeID = ID.generateInt();
+         if (aTimeInMs > Integer.MAX_VALUE || aTimeInMs < 0)
+         {
+             throw new IllegalArgumentException("Timeout value " + aTimeInMs + " is not supported.");
+         }
+    	 final int timeID = generateTimerID();
     	 final Integer timeIDInt = new Integer(timeID);
-    	 ActionListener task = new ActionListener() {
-    		 public void actionPerformed(ActionEvent e)
-    		 {
-    			 // This executes in the GUI thread and that's good.
-    			 Window.this.forgetTask(timeIDInt, false);
-    			 HTMLDocumentImpl doc = (HTMLDocumentImpl) document;
-    			 if (doc == null)
-    			 {
-    				 //  FIXME : Make a proper method to stop all timer launched via setInterval, other than window.setDocument(null)
-    				 throw new IllegalStateException("Cannot perform operation when document is unset.");
-    			 }
-    			 Window.this.eval(aExpression);
-    		 }
-    	 };
-    	 if (aTimeInMs > Integer.MAX_VALUE || aTimeInMs < 0)
-    	 {
-    		 throw new IllegalArgumentException("Timeout value " + aTimeInMs + " is not supported.");
+    	 ActionListener task = new ExpressionTimerTask(this, timeIDInt, aExpression, false); 
+    	 int t = (int) aTimeInMs;
+    	 if(t < 1) {
+    	     t = 1;
     	 }
-    	 Timer timer = new Timer((int) aTimeInMs, task);
+    	 Timer timer = new Timer(t, task);
     	 timer.setRepeats(false); // The only difference with setTimeout
-    	 timer.start();
-    	 this.putTask(timeIDInt, timer);
+    	 this.putAndStartTask(timeIDInt, timer, null);
     	 return timeID;
      }
 
@@ -430,6 +435,10 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		}
 	}
 
+	public Window open(String url) {
+	    return this.open(url, "window:" + String.valueOf(ID.generateLong()));
+	}
+
 	public Window open(String url, String windowName) {
 		return this.open(url, windowName, "", false);
 	}
@@ -456,58 +465,65 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		}
 	}
 
-	public void scroll(int x, int y) {
+	public void scrollTo(int x, int y) {
 		HtmlRendererContext rcontext = this.rcontext;
 		if(rcontext != null) {
 			rcontext.scroll(x, y);
 		}
 	}
-	
-	public int setTimeout(final String expr, double millis) {
-		final int timeID = ID.generateInt();
-		final Integer timeIDInt = new Integer(timeID);
-		ActionListener task = new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				// This executes in the GUI thread and that's good.
-				Window.this.forgetTask(timeIDInt, false);
-				try {
-					Window.this.eval(expr);
-				} catch(Throwable thrown) {
-					logger.log(Level.WARNING, "setTimeout(): Error evaluating expression.", thrown);
-				}
-			}			
-		};
-		if(millis > Integer.MAX_VALUE || millis < 0) {
-			throw new IllegalArgumentException("Timeout value " + millis + " is not supported.");
+
+	public void scrollBy(int x, int y) {
+		HtmlRendererContext rcontext = this.rcontext;
+		if(rcontext != null) {
+			rcontext.scrollBy(x, y);
 		}
-		Timer timer = new Timer((int) millis, task);
+	}
+	
+	public void resizeTo(int width, int height) {
+		HtmlRendererContext rcontext = this.rcontext;
+		if(rcontext != null) {
+			rcontext.resizeTo(width, height);
+		}		
+	}
+	
+	public void resizeBy(int byWidth, int byHeight) {
+		HtmlRendererContext rcontext = this.rcontext;
+		if(rcontext != null) {
+			rcontext.resizeBy(byWidth, byHeight);
+		}				
+	}
+
+	public int setTimeout(final String expr, double millis) {
+        if(millis > Integer.MAX_VALUE || millis < 0) {
+            throw new IllegalArgumentException("Timeout value " + millis + " is not supported.");
+        }
+		final int timeID = generateTimerID();
+		final Integer timeIDInt = new Integer(timeID);
+		ActionListener task = new ExpressionTimerTask(this, timeIDInt, expr, true); 
+		int t = (int) millis;
+		if(t < 1) {
+		    t = 1;
+		}
+		Timer timer = new Timer(t, task);
 		timer.setRepeats(false);
-		timer.start();
-		this.putTask(timeIDInt, timer);
+		this.putAndStartTask(timeIDInt, timer, null);
 		return timeID;
 	}
 	
 	public int setTimeout(final Function function, double millis) {
-		final int timeID = ID.generateInt();
+        if(millis > Integer.MAX_VALUE || millis < 0) {
+            throw new IllegalArgumentException("Timeout value " + millis + " is not supported.");
+        }
+		final int timeID = generateTimerID();
 		final Integer timeIDInt = new Integer(timeID);
-		ActionListener task = new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				// This executes in the GUI thread and that's good.
-				Window.this.forgetTask(timeIDInt, false);
-				HTMLDocumentImpl doc = (HTMLDocumentImpl) document;
-				if(doc == null) {
-					throw new IllegalStateException("Cannot perform operation when document is unset.");
-				}
-				Executor.executeFunction(getWindowScope(), function, doc.getDocumentURL(), uaContext);
-			}			
-		};
-		if(millis > Integer.MAX_VALUE || millis < 0) {
-			throw new IllegalArgumentException("Timeout value " + millis + " is not supported.");
+		ActionListener task = new FunctionTimerTask(this, timeIDInt, function, true);
+		int t = (int) millis;
+		if(t < 1) {
+		    t = 1;
 		}
-		Timer timer = new Timer((int) millis, task);
+		Timer timer = new Timer(t, task);
 		timer.setRepeats(false);
-		timer.start();
-		this.putTask(timeIDInt, timer);
+		this.putAndStartTask(timeIDInt, timer, function);
 		return timeID;
 	}
 
@@ -539,12 +555,25 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		return null;
 	}
 	
+	private int length;
+	private boolean lengthSet = false;
+	
 	/**
 	 * Gets the number of frames.
 	 */
 	public int getLength() {
-		HTMLCollection frames = this.getFrames();
-		return frames == null ? 0 : frames.getLength();
+		if(this.lengthSet) {
+			return this.length;
+		}
+		else {
+			HTMLCollection frames = this.getFrames();
+			return frames == null ? 0 : frames.getLength();
+		}
+	}
+	
+	public void setLength(int length) {
+		this.lengthSet = true;
+		this.length = length;
 	}
 	
 	public String getName() {
@@ -661,6 +690,19 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		this.getLocation().setHref(location);
 	}
 	
+	private History history; 
+	
+	public History getHistory() {
+        synchronized(this) {
+            History history = this.history;
+            if(history == null) {
+                history = new History(this);
+                this.history = history;
+            }
+            return history;
+        }	    
+	}
+	
 	public CSS2Properties getComputedStyle(HTMLElement element, String pseudoElement) {
 		if(element instanceof HTMLElementImpl) {
 			return ((HTMLElementImpl) element).getComputedStyle(pseudoElement);
@@ -699,5 +741,129 @@ public class Window extends AbstractScriptableDelegate implements AbstractView
 		this.onunload = onunload;
 	}
 	
+	
+	public org.w3c.dom.Node namedItem(String name) {
+		// Bug 1928758: Element IDs are named objects in context. 
+		HTMLDocumentImpl doc = this.document;
+		if(doc == null) {
+			return null;
+		}
+		org.w3c.dom.Node node = doc.getElementById(name);
+		if(node != null) {
+			return node;
+		}
+		return null;
+	}
+	
+	public void forceGC() {
+	    System.gc();
+	}
+	
+	private static abstract class WeakWindowTask implements ActionListener {
+	    private final WeakReference windowRef;
+
+	    public WeakWindowTask(Window window) {
+	        this.windowRef = new WeakReference(window);
+	    }
+	    
+	    protected Window getWindow() {
+	        WeakReference ref = this.windowRef;
+	        return ref == null ? null : (Window) ref.get();
+	    }	    
+	}
+	
+	private static class FunctionTimerTask extends WeakWindowTask {
+	    // Implemented as a static WeakWindowTask to allow the Window
+	    // to get garbage collected, especially in infinite loop 
+	    // scenarios.
+	    private final Integer timeIDInt;
+	    private final WeakReference functionRef;
+	    private final boolean removeTask;
+	    
+	    public FunctionTimerTask(Window window, Integer timeIDInt, Function function, boolean removeTask) {
+	        super(window);
+	        this.timeIDInt = timeIDInt;
+	        this.functionRef = new WeakReference(function);
+	        this.removeTask = removeTask;
+	    }
+	    
+	    public void actionPerformed(ActionEvent e) {
+	        // This executes in the GUI thread and that's good.
+	        try {
+	            Window window = this.getWindow(); 
+	            if(window == null) {
+	                if(logger.isLoggable(Level.INFO)) {
+	                    logger.info("actionPerformed(): Window is no longer available.");
+	                }
+	                return;
+	            }
+	            if(this.removeTask) {
+	                window.forgetTask(this.timeIDInt, false);
+	            }
+	            HTMLDocumentImpl doc = (HTMLDocumentImpl) window.getDocument();
+	            if(doc == null) {
+	                throw new IllegalStateException("Cannot perform operation when document is unset.");
+	            }
+	            Function function = (Function) this.functionRef.get();
+	            if(function == null) {
+	                throw new IllegalStateException("Cannot perform operation. Function is no longer available.");
+	            }
+	            Executor.executeFunction(window.getWindowScope(), function, doc.getDocumentURL(), window.getUserAgentContext());
+	        } catch(Throwable err) {
+	            logger.log(Level.WARNING, "actionPerformed()", err);
+	        }
+	    }           
+	}
+	
+	private static class ExpressionTimerTask extends WeakWindowTask {
+	    // Implemented as a static WeakWindowTask to allow the Window
+	    // to get garbage collected, especially in infinite loop 
+	    // scenarios.
+	    private final Integer timeIDInt;
+	    private final String expression;
+	    private final boolean removeTask;
+
+	    public ExpressionTimerTask(Window window, Integer timeIDInt, String expression, boolean removeTask) {
+	        super(window);
+	        this.timeIDInt = timeIDInt;
+	        this.expression = expression;
+	        this.removeTask = removeTask;
+	    }
+
+	    public void actionPerformed(ActionEvent e) {
+	        // This executes in the GUI thread and that's good.
+	        try {
+	            Window window = this.getWindow(); 
+	            if(window == null) {
+                    if(logger.isLoggable(Level.INFO)) {
+                        logger.info("actionPerformed(): Window is no longer available.");
+                    }
+	                return;
+	            }
+	            if(this.removeTask) {
+	                window.forgetTask(this.timeIDInt, false);
+	            }
+	            HTMLDocumentImpl doc = (HTMLDocumentImpl) window.getDocument();
+	            if(doc == null) {
+	                throw new IllegalStateException("Cannot perform operation when document is unset.");
+	            }
+	            window.eval(this.expression);
+	        } catch(Throwable err) {
+	            logger.log(Level.WARNING, "actionPerformed()", err);
+	        }
+	    }           
+	}
+
+	private static class TaskWrapper {
+	    public final Timer timer;
+	    private final Object retained;
+	    
+        public TaskWrapper(Timer timer, Object retained) {
+            super();
+            this.timer = timer;
+            this.retained = retained;
+        }
+	}
+	
 }
 
