@@ -30,6 +30,7 @@ class ElementInfo {
 	public final boolean childElementOk;
 	public final Set stopTags;
 	public final boolean noScriptElement;
+	public final boolean decodeEntities;
 	
 	public static final int END_ELEMENT_FORBIDDEN = 0;
 	public static final int END_ELEMENT_OPTIONAL = 1;
@@ -44,6 +45,7 @@ class ElementInfo {
 		this.endElementType = type;
 		this.stopTags = null;
 		this.noScriptElement = false;
+		this.decodeEntities = true;
 	}
 
 	/**
@@ -55,6 +57,7 @@ class ElementInfo {
 		this.endElementType = type;
 		this.stopTags = stopTags;
 		this.noScriptElement = false;
+		this.decodeEntities = true;
 	}
 
 	public ElementInfo(boolean ok, int type, Set stopTags, boolean noScriptElement) {
@@ -62,5 +65,14 @@ class ElementInfo {
 		this.endElementType = type;
 		this.stopTags = stopTags;
 		this.noScriptElement = noScriptElement;
+		this.decodeEntities = true;
+	}
+	
+	public ElementInfo(boolean ok, int type, boolean decodeEntities) {
+		this.childElementOk = ok;
+		this.endElementType = type;
+		this.stopTags = null;
+		this.noScriptElement = false;
+		this.decodeEntities = decodeEntities;
 	}
 }
