@@ -33,7 +33,6 @@ import org.w3c.dom.DOMException;
 import org.xml.sax.ErrorHandler;
 import org.xml.sax.Locator;
 import org.xml.sax.SAXException;
-import org.xml.sax.SAXParseException;
 import org.lobobrowser.html.*;
 import org.lobobrowser.html.io.*;
 
@@ -381,10 +380,10 @@ public class HtmlParser {
 		paragraphStopElements.add("OL");
 		ElementInfo paragraphElement = new ElementInfo(true, ElementInfo.END_ELEMENT_OPTIONAL, paragraphStopElements);
 
-		Set liStopElements = new HashSet();
-		liStopElements.add("LI");
-		liStopElements.add("UL");
-		liStopElements.add("OL");
+//		Set liStopElements = new HashSet();
+//		liStopElements.add("LI");
+//		liStopElements.add("UL");
+//		liStopElements.add("OL");
 		
 		elementInfos.put("SCRIPT", onlyText);
 		elementInfos.put("STYLE", onlyText);
@@ -409,6 +408,12 @@ public class HtmlParser {
 		elementInfos.put("TD", tableCellElement);
 		elementInfos.put("HEAD", headElement);
 		elementInfos.put("OPTION", optionElement);
+		
+		// Note: The specification states anchors have
+		// a required end element, but browsers generally behave
+		// as if it's optional.
+		elementInfos.put("A", optionalEndElement);
+		elementInfos.put("ANCHOR", optionalEndElement);
 		//TODO: Keep adding tags here		
 	}
 
@@ -457,6 +462,11 @@ public class HtmlParser {
 		this.systemId = null;
 	}
 
+	public static boolean isDecodeEntities(String elementName) {
+		ElementInfo einfo = (ElementInfo) ELEMENT_INFOS.get(elementName.toUpperCase());
+		return einfo == null ? true : einfo.decodeEntities;
+	}
+	
 	/**
 	 * Parses HTML from an input stream, assuming
 	 * the character set is ISO-8859-1.
@@ -568,7 +578,7 @@ public class HtmlParser {
 			return TOKEN_EOD;
 		}
 		if(textSb.length() != 0) {
-			int textLine = reader.getLineNumber();
+			//int textLine = reader.getLineNumber();
 			StringBuffer decText = this.entityDecode(textSb);
 			Node textNode = doc.createTextNode(decText.toString());
 			try {
@@ -580,7 +590,7 @@ public class HtmlParser {
 			}
 		}
 		if(this.justReadTagBegin) {
-			String tag = this.readTag(reader);
+			String tag = this.readTag(parent, reader);
 			if(tag == null) {
 				return TOKEN_EOD;
 			}
@@ -588,7 +598,7 @@ public class HtmlParser {
 			try {
 				if(tag.startsWith("!")) {
 					if("!--".equals(tag)) {
-						int commentLine = reader.getLineNumber();
+						//int commentLine = reader.getLineNumber();
 						StringBuffer comment = this.passEndOfComment(reader);
 						StringBuffer decText = this.entityDecode(comment);
 						parent.appendChild(doc.createComment(decText.toString()));
@@ -867,46 +877,105 @@ public class HtmlParser {
 	 * @param reader
 	 * @return
 	 */
-	private final String readTag(LineNumberReader reader) throws IOException {
+	private final String readTag(Node parent, LineNumberReader reader) throws IOException {
 		StringBuffer sb = new StringBuffer();
 		int chInt;
 		chInt = reader.read();
 		if(chInt != -1) {
 			boolean cont = true;
-			char ch = (char) chInt;
-			if(ch == '!') {
-				sb.append('!');
-				chInt = reader.read();
-				if(chInt != -1) {
-					ch = (char) chInt;
-					if(ch == '-') {
-						sb.append('-');
-						chInt = reader.read();
-						if(chInt != -1) {
-							ch = (char) chInt;
-							if(ch == '-') {
-								sb.append('-');
+			char ch;
+			LOOP:
+			for(;;) {
+				ch = (char) chInt;
+				if(Character.isLetter(ch)) {
+					// Speed up normal case
+					break LOOP;
+				}
+				else if(ch == '!') {
+					sb.append('!');
+					chInt = reader.read();
+					if(chInt != -1) {
+						ch = (char) chInt;
+						if(ch == '-') {
+							sb.append('-');
+							chInt = reader.read();
+							if(chInt != -1) {
+								ch = (char) chInt;
+								if(ch == '-') {
+									sb.append('-');
+									cont = false;
+								}
+							}
+							else {
 								cont = false;
 							}
 						}
-						else {
-							cont = false;
-						}
+					}
+					else {
+						cont = false;
 					}
 				}
-				else {
-					cont = false;
+				else if(ch == '/') {
+					sb.append(ch);
+					chInt = reader.read();
+					if(chInt != -1) {
+						ch = (char) chInt;
+					}
+					else {
+						cont = false;
+					}
 				}
-			}
-			else if(ch == '/') {
-				sb.append(ch);
-				chInt = reader.read();
-				if(chInt != -1) {
-					ch = (char) chInt;
+				else if(ch == '<') {
+					StringBuffer ltText = new StringBuffer(3);
+					ltText.append('<');
+					while((chInt = reader.read()) == '<') {
+						ltText.append('<');
+					}	
+					Document doc = this.document;
+					Node textNode = doc.createTextNode(ltText.toString());
+					try {
+						parent.appendChild(textNode);
+					} catch(DOMException de) {
+						if(parent.getNodeType() != Node.DOCUMENT_NODE || de.code != DOMException.HIERARCHY_REQUEST_ERR) {
+							logger.log(Level.WARNING, "parseToken(): Unable to append child to " + parent + ".", de);
+						}
+					}			
+					if(chInt == -1) {
+						cont = false;
+					}
+					else {
+						continue LOOP;
+					}
 				}
-				else {
-					cont = false;
+				else if(Character.isWhitespace(ch)) {
+					StringBuffer ltText = new StringBuffer();
+					ltText.append('<');
+					ltText.append(ch);
+					while((chInt = reader.read()) != -1) {
+						ch = (char) chInt;
+						if(ch == '<') {
+							chInt = reader.read();
+							break;
+						}
+						ltText.append(ch);
+					}	
+					Document doc = this.document;
+					Node textNode = doc.createTextNode(ltText.toString());
+					try {
+						parent.appendChild(textNode);
+					} catch(DOMException de) {
+						if(parent.getNodeType() != Node.DOCUMENT_NODE || de.code != DOMException.HIERARCHY_REQUEST_ERR) {
+							logger.log(Level.WARNING, "parseToken(): Unable to append child to " + parent + ".", de);
+						}
+					}			
+					if(chInt == -1) {
+						cont = false;
+					}
+					else {
+						continue LOOP;
+					}					
 				}
+				break LOOP;
 			}
 			if(cont) {
 				boolean lastCharSlash = false;
@@ -1184,7 +1253,10 @@ public class HtmlParser {
 				if(attributeName != null) {
 					String attributeNameStr = attributeName.toString();
 					if(attributeValue == null) {
-						element.setAttribute(attributeNameStr, null);
+						// Quotes are closed. There's a distinction
+						// between blank values and null in HTML, as
+						// processed by major browsers.
+						element.setAttribute(attributeNameStr, "");
 					}
 					else {
 						StringBuffer actualAttributeValue = this.entityDecode(attributeValue);
