@@ -36,6 +36,7 @@ import org.xml.sax.InputSource;
 public class InputSourceImpl extends InputSource {
 	/**
 	 * Constructs an <code>InputSourceImpl</code>.
+	 * @deprecated Use a constructor that takes either a stream or a reader.
 	 */
 	public InputSourceImpl() {
 		super();
@@ -43,13 +44,15 @@ public class InputSourceImpl extends InputSource {
 
 	/**
 	 * Constructs an <code>InputSourceImpl</code>.
-	 * Note that the parameter does not represent a
-	 * string to be parsed. To parse a string, use
-	 * a StringReader.
-	 * @param systemId The system ID of the input source.
+	 * <p>
+	 * It is valid to use this constructor, but it
+	 * is generally recommended that callers use
+	 * one of the constructors that take a reader
+	 * or an input stream instead.
+	 * @param uri The URI (or systemID) of the document.
 	 */
-	public InputSourceImpl(String systemId) {
-		super(systemId);
+	public InputSourceImpl(String uri) {
+		super(uri);
 	}
 
 	/**
