@@ -190,15 +190,17 @@ abstract class BaseBoundableRenderable extends BaseRenderable implements Boundab
 	protected boolean isValid() {
 		return this.layoutUpTreeCanBeInvalidated;
 	}
-
 	
-	protected final void relayoutImpl(boolean invalidateLocal) {
+	private final void relayoutImpl(boolean invalidateLocal, boolean onlyIfValid) {
+	    if(onlyIfValid && !this.layoutUpTreeCanBeInvalidated) {
+	        return;
+	    }
 		if(invalidateLocal) {
 			this.invalidateLayoutUpTree();
 		}
 		Renderable parent = this.parent;
 		if(parent instanceof BaseBoundableRenderable) {
-			((BaseBoundableRenderable) parent).relayoutImpl(false);
+			((BaseBoundableRenderable) parent).relayoutImpl(false, false);
 		}
 		else if(parent == null) {
 			// Has to be top RBlock.
@@ -218,17 +220,30 @@ abstract class BaseBoundableRenderable extends BaseRenderable implements Boundab
 	 */
 	public void relayout() {
 		if(EventQueue.isDispatchThread()) {
-			this.relayoutImpl(true);
+			this.relayoutImpl(true, false);
 		}
 		else {
 			EventQueue.invokeLater(new Runnable() {
 				public void run() {
-					relayout();
+					relayoutImpl(true, false);
 				}
 			});
 		}
 	}
-	
+
+    public void relayoutIfValid() {
+	    if(EventQueue.isDispatchThread()) {
+	        this.relayoutImpl(true, true);
+	    }
+	    else {
+	        EventQueue.invokeLater(new Runnable() {
+	            public void run() {
+	                relayoutImpl(true, true);
+	            }
+	        });
+	   }
+    }
+
 	/**
 	 * Parent for graphics coordinates.
 	 */
