@@ -29,26 +29,17 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.logging.Level;
 
+import org.lobobrowser.util.Strings;
 import org.lobobrowser.util.gui.*;
 import org.lobobrowser.html.HttpRequest;
 import org.lobobrowser.html.ReadyStateChangeListener;
 import org.lobobrowser.html.UserAgentContext;
 import org.lobobrowser.html.domimpl.*;
-import org.lobobrowser.html.style.BackgroundInfo;
-import org.lobobrowser.html.style.AbstractCSS2Properties;
-import org.lobobrowser.html.style.HtmlValues;
-import org.lobobrowser.html.style.RenderState;
+import org.lobobrowser.html.style.*;
 import org.lobobrowser.util.gui.ColorFactory;
 import org.w3c.dom.css.CSS2Properties;
 
 abstract class BaseElementRenderable extends BaseRCollection implements RElement, RenderableContainer, java.awt.image.ImageObserver {
-	public static final int OVERFLOW_NONE = 0;
-	public static final int OVERFLOW_SCROLL = 1;
-	public static final int OVERFLOW_AUTO = 2;
-	public static final int OVERFLOW_HIDDEN = 3;
-	public static final int OVERFLOW_VISIBLE = 4;
-	public static final int OVERFLOW_VERTICAL = 5;
-	
 	protected static final Integer INVALID_SIZE = new Integer(Integer.MIN_VALUE);
 	
 	/**
@@ -77,10 +68,14 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 	protected Color borderBottomColor;
 	protected Color borderRightColor;	
 	protected Insets borderInsets;
-	protected Insets borderStyles;
+	protected Insets marginInsets;
+    protected Insets paddingInsets;
+    protected BorderInfo borderInfo;
 	protected java.net.URL lastBackgroundImageUri;
-	protected Insets defaultMarginInsets;
-	private int cachedOverflow = -1;
+	protected Insets defaultMarginInsets = null;
+    protected Insets defaultPaddingInsets = null;
+    protected int overflowX;
+    protected int overflowY;
 	
 	protected final UserAgentContext userAgentContext;
 	
@@ -89,6 +84,14 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		this.userAgentContext = ucontext;
 	}
 
+    public void setDefaultPaddingInsets(Insets insets) {
+        this.defaultPaddingInsets = insets;
+    }
+
+    public void setDefaultMarginInsets(Insets insets) {
+        this.defaultMarginInsets = insets;
+    }
+
 	public float getAlignmentX() {
 		return 0.0f;
 	}
@@ -122,7 +125,12 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 	}
 
 	protected void invalidateLayoutLocal() {
-		this.cachedOverflow = -1;
+	    RenderState rs = this.modelNode.getRenderState();
+	    if(rs != null) {
+	        rs.invalidate();
+	    }
+	    this.overflowX = RenderState.OVERFLOW_NONE;
+        this.overflowY = RenderState.OVERFLOW_NONE;
 		this.declaredWidth = INVALID_SIZE;
 		this.declaredHeight = INVALID_SIZE;
 		this.lastAvailHeightForDeclared = -1;
@@ -134,16 +142,33 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 	private int lastAvailWidthForDeclared = -1;
 	private int lastAvailHeightForDeclared = -1;
 	
-	protected Integer getDeclaredWidth(RenderState renderState, int availWidth) {
+	protected Integer getDeclaredWidth(RenderState renderState, int actualAvailWidth) {
 		Integer dw = this.declaredWidth;
-		if(dw == INVALID_SIZE || availWidth != this.lastAvailWidthForDeclared) {
-			this.lastAvailWidthForDeclared = availWidth;
-			int dwInt = this.getDeclaredWidthImpl(renderState, availWidth);
+		if(dw == INVALID_SIZE || actualAvailWidth != this.lastAvailWidthForDeclared) {
+			this.lastAvailWidthForDeclared = actualAvailWidth;
+			int dwInt = this.getDeclaredWidthImpl(renderState, actualAvailWidth);
 			dw = dwInt == -1 ? null : new Integer(dwInt);
 			this.declaredWidth  = dw;
 		}
 		return dw;
 	}
+	
+	public final boolean hasDeclaredWidth() {
+        Integer dw = this.declaredWidth;
+        if(dw == INVALID_SIZE) {
+            Object rootNode = this.modelNode;
+            if(rootNode instanceof HTMLElementImpl) {
+                HTMLElementImpl element = (HTMLElementImpl) rootNode;
+                CSS2Properties props = element.getCurrentStyle();
+                if(props == null) {
+                    return false;
+                }
+                return !Strings.isBlank(props.getWidth());
+            }
+            return false;
+        }
+        return dw != null;
+	}
 		
 	private final int getDeclaredWidthImpl(RenderState renderState, int availWidth) {
 		Object rootNode = this.modelNode;
@@ -164,11 +189,11 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		}
 	}
 
-	protected Integer getDeclaredHeight(RenderState renderState, int availHeight) {
+	protected Integer getDeclaredHeight(RenderState renderState, int actualAvailHeight) {
 		Integer dh = this.declaredHeight;
-		if(dh == INVALID_SIZE || availHeight != this.lastAvailHeightForDeclared) {
-			this.lastAvailHeightForDeclared = availHeight;
-			int dhInt = this.getDeclaredHeightImpl(renderState, availHeight);
+		if(dh == INVALID_SIZE || actualAvailHeight != this.lastAvailHeightForDeclared) {
+			this.lastAvailHeightForDeclared = actualAvailHeight;
+			int dhInt = this.getDeclaredHeightImpl(renderState, actualAvailHeight);
 			dh = dhInt == -1 ? null : new Integer(dhInt);
 			this.declaredHeight  = dh;
 		}
@@ -194,53 +219,6 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		}		
 	}
 		
-	protected int getOverflow() {
-		int co = this.cachedOverflow;
-		if(co != -1) {
-			return co;
-		}
-		Object rootNode = this.modelNode;
-		if(rootNode instanceof HTMLElementImpl) {
-			HTMLElementImpl element = (HTMLElementImpl) rootNode;
-			CSS2Properties props = element.getCurrentStyle();
-			if(props == null) {
-				co = OVERFLOW_NONE;
-			}
-			else {
-				String overflowText = props.getOverflow();
-				if(overflowText == null) {
-					co = OVERFLOW_NONE;
-				}
-				else {
-					String overflowTextTL = overflowText.toLowerCase();
-					if("scroll".equals(overflowTextTL)) {
-						co = OVERFLOW_SCROLL;
-					}
-					else if("auto".equals(overflowTextTL)) {
-						co = OVERFLOW_AUTO;
-					}
-					else if("vertical".equals(overflowTextTL)) {
-						co = OVERFLOW_VERTICAL;
-					}
-					else if("hidden".equals(overflowTextTL)) {
-						co = OVERFLOW_HIDDEN;
-					}
-					else if("visible".equals(overflowTextTL)) {
-						co = OVERFLOW_VISIBLE;
-					}
-					else {
-						co = OVERFLOW_NONE;
-					}
-				}
-			}
-		}
-		else {
-			co = OVERFLOW_NONE;
-		}
-		this.cachedOverflow = co;
-		return co;
-	}
-	
 	/**
 	 * All overriders should call super implementation.
 	 */
@@ -251,17 +229,17 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 	 * Lays out children, and deals with "valid" state. Override doLayout method
 	 * instead of this one.
 	 */
-	public final void layout(int availWidth, int availHeight) {
+	public final void layout(int availWidth, int availHeight, boolean sizeOnly) {
 		// Must call doLayout regardless of validity state.
 		try {
-			this.doLayout(availWidth, availHeight);	
+			this.doLayout(availWidth, availHeight, sizeOnly);	
 		} finally {
 			this.layoutUpTreeCanBeInvalidated = true;
 			this.layoutDeepCanBeInvalidated = true;
 		}
 	}	
 	
-	protected abstract void doLayout(int availWidth, int availHeight);	
+	protected abstract void doLayout(int availWidth, int availHeight, boolean sizeOnly);	
 	
 	protected final void sendGUIComponentsToParent() {
 		// Ensures that parent has all the components
@@ -316,24 +294,79 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		this.updateWidgetBounds(guiPoint.x, guiPoint.y);
 	}
 
-	protected void applyStyle() {
+	public Rectangle getBoundsRelativeToBlock() {
+		RCollection parent = this;
+		int x = 0, y = 0;
+		while(parent != null) {
+			x += parent.getX();
+			y += parent.getY();
+			parent = parent.getParent();
+			if(parent instanceof RElement) {
+				break;
+			}
+		}
+		return new Rectangle(x, y, this.getWidth(), this.getHeight());
+	}
+	
+	protected void clearStyle(boolean isRootBlock) {
+        this.borderInfo = null;
+        this.borderInsets = null;
+        this.borderTopColor = null;
+        this.borderLeftColor = null;
+        this.borderBottomColor = null;
+        this.borderRightColor = null;
+        this.zIndex = 0;
+        this.backgroundColor = null;
+        this.backgroundImage = null;
+        this.lastBackgroundImageUri = null;
+        this.overflowX = RenderState.OVERFLOW_VISIBLE;
+        this.overflowY = RenderState.OVERFLOW_VISIBLE;
+        if(isRootBlock) {
+            // The margin of the root block behaves like extra padding.
+            Insets insets1 = this.defaultMarginInsets;
+            Insets insets2 = this.defaultPaddingInsets;
+            Insets finalInsets = insets1 == null ? null : new Insets(insets1.top, insets1.left, insets1.bottom, insets1.right);
+            if(insets2 != null) {
+                if(finalInsets == null) {
+                    finalInsets = new Insets(insets2.top, insets2.left, insets2.bottom, insets2.right);
+                }
+                else {
+                    finalInsets.top += insets2.top;
+                    finalInsets.bottom += insets2.bottom;
+                    finalInsets.left += insets2.left;
+                    finalInsets.right += insets2.right;
+                }
+            }
+            this.marginInsets = null;
+            this.paddingInsets = finalInsets;
+        }
+        else {
+            this.marginInsets = this.defaultMarginInsets;        
+            this.paddingInsets = this.defaultPaddingInsets;
+        }
+	}
+
+	protected void applyStyle(int availWidth, int availHeight) {
 		//TODO: Can be optimized if there's no style?
-		//Note: Overridden by table cell
+	    //TODO: There's part of this that doesn't depend on availWidth
+	    //      and availHeight, so it doesn't need to be redone on
+	    //      every resize.
+		//Note: Overridden by tables and lists.
 		Object rootNode = this.modelNode;
 		HTMLElementImpl rootElement;
-		if(rootNode instanceof HTMLDocumentImpl) {			
+		boolean isRootBlock;
+		if(rootNode instanceof HTMLDocumentImpl) {
+		    isRootBlock = true;
 			HTMLDocumentImpl doc = (HTMLDocumentImpl) rootNode;
 			// Need to get BODY tag, for bgcolor, etc.
 			rootElement = (HTMLElementImpl) doc.getBody();
 		}
 		else {
+		    isRootBlock = false;
 			rootElement = (HTMLElementImpl) rootNode;
 		}
 		if(rootElement == null) {
-			this.clearStyle();
-			this.backgroundColor = null;
-			this.backgroundImage = null;
-			this.lastBackgroundImageUri = null;
+		    this.clearStyle(isRootBlock);
 			return;
 		}
 		RenderState rs = rootElement.getRenderState();
@@ -354,40 +387,86 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		}
 		AbstractCSS2Properties props = rootElement.getCurrentStyle();
 		if(props == null) {
-			this.clearStyle();
+			this.clearStyle(isRootBlock);
 		}
 		else {
-			Insets borderStyles = HtmlValues.getBorderStyles(props);
-			this.borderStyles = borderStyles;
-			this.borderInsets = HtmlValues.getBorderInsets(borderStyles, props, rs);
-			String borderTopColorText = props.getBorderTopColor();
-			if(borderTopColorText != null) {
-				this.borderTopColor = ColorFactory.getInstance().getColor(borderTopColorText);
-			}
-			else {
-				this.borderTopColor = null;
-			}
-			String borderLeftColorText = props.getBorderLeftColor();
-			if(borderLeftColorText != null) {
-				this.borderLeftColor = ColorFactory.getInstance().getColor(borderLeftColorText);
-			}
-			else {
-				this.borderLeftColor = null;
-			}
-			String borderBottomColorText = props.getBorderBottomColor();
-			if(borderBottomColorText != null) {
-				this.borderBottomColor = ColorFactory.getInstance().getColor(borderBottomColorText);
-			}
-			else {
-				this.borderBottomColor = null;
-			}
-			String borderRightColorText = props.getBorderRightColor();
-			if(borderRightColorText != null) {
-				this.borderRightColor = ColorFactory.getInstance().getColor(borderRightColorText);
-			}
-			else {
-				this.borderRightColor = null;
+		    BorderInfo borderInfo = rs.getBorderInfo();
+            this.borderInfo = borderInfo;
+			HtmlInsets binsets = borderInfo == null ? null : borderInfo.insets;					
+			HtmlInsets minsets = rs.getMarginInsets();
+			HtmlInsets pinsets = rs.getPaddingInsets();
+			Insets defaultMarginInsets = this.defaultMarginInsets;
+			int dmleft = 0, dmright = 0, dmtop = 0, dmbottom = 0;
+			if(defaultMarginInsets != null) {
+			    dmleft = defaultMarginInsets.left;
+			    dmright = defaultMarginInsets.right;
+			    dmtop = defaultMarginInsets.top;
+			    dmbottom = defaultMarginInsets.bottom;
 			}
+			Insets defaultPaddingInsets = this.defaultPaddingInsets;
+            int dpleft = 0, dpright = 0, dptop = 0, dpbottom = 0;
+            if(defaultPaddingInsets != null) {
+                dpleft = defaultPaddingInsets.left;
+                dpright = defaultPaddingInsets.right;
+                dptop = defaultPaddingInsets.top;
+                dpbottom = defaultPaddingInsets.bottom;
+            }			
+            Insets borderInsets = binsets == null ? null : binsets.getAWTInsets(0, 0, 0, 0, availWidth, availHeight, 0, 0); 
+            if(borderInsets == null) {
+                borderInsets = RBlockViewport.ZERO_INSETS;
+            }
+            Insets paddingInsets = pinsets == null ? defaultPaddingInsets : pinsets.getAWTInsets(dptop, dpleft, dpbottom, dpright, availWidth, availHeight, 0, 0);
+            if(paddingInsets == null) {
+                paddingInsets = RBlockViewport.ZERO_INSETS;
+            }
+            Insets tentativeMarginInsets = minsets == null ? defaultMarginInsets : minsets.getAWTInsets(dmtop, dmleft, dmbottom, dmright, availWidth, availHeight, 0, 0);
+            if(tentativeMarginInsets == null) {
+                tentativeMarginInsets = RBlockViewport.ZERO_INSETS;
+            }
+            int actualAvailWidth = availWidth - paddingInsets.left - paddingInsets.right - borderInsets.left - borderInsets.right - tentativeMarginInsets.left - tentativeMarginInsets.right;
+            int actualAvailHeight = availHeight - paddingInsets.top - paddingInsets.bottom - borderInsets.top - borderInsets.bottom - tentativeMarginInsets.top - tentativeMarginInsets.bottom;            
+            Integer declaredWidth = this.getDeclaredWidth(rs, actualAvailWidth);
+            Integer declaredHeight = this.getDeclaredHeight(rs, actualAvailHeight);
+            int autoMarginX = 0, autoMarginY = 0;
+            if(declaredWidth != null) {
+                autoMarginX = (availWidth - declaredWidth.intValue() - (borderInsets == null ? 0 : borderInsets.left - borderInsets.right) - (paddingInsets == null ? 0 : paddingInsets.left - paddingInsets.right)) / 2; 
+            }
+            if(declaredHeight != null) {
+                autoMarginY = (availHeight - declaredHeight.intValue() - (borderInsets == null ? 0 : borderInsets.top - borderInsets.bottom) - (paddingInsets == null ? 0 : paddingInsets.top - paddingInsets.bottom)) / 2; 
+            }
+            this.borderInsets = borderInsets;
+            if(isRootBlock) {
+                // In the root block, the margin behaves like an extra padding.
+                Insets regularMarginInsets =  (autoMarginX == 0 && autoMarginY == 0) ? tentativeMarginInsets : (minsets == null ? defaultMarginInsets : minsets.getAWTInsets(dmtop, dmleft, dmbottom, dmright, availWidth, availHeight, autoMarginX, autoMarginY));
+                if(paddingInsets == null) {
+                    paddingInsets = RBlockViewport.ZERO_INSETS;
+                }
+                if(regularMarginInsets == null) {
+                    regularMarginInsets = RBlockViewport.ZERO_INSETS;
+                }
+                this.marginInsets = null;
+                this.paddingInsets = new Insets(
+                        paddingInsets.top + regularMarginInsets.top, 
+                        paddingInsets.left + regularMarginInsets.left, 
+                        paddingInsets.bottom + regularMarginInsets.bottom, 
+                        paddingInsets.right + regularMarginInsets.right);
+            }
+            else {
+                this.paddingInsets = paddingInsets;
+                this.marginInsets =  (autoMarginX == 0 && autoMarginY == 0) ? tentativeMarginInsets : (minsets == null ? defaultMarginInsets : minsets.getAWTInsets(dmtop, dmleft, dmbottom, dmright, availWidth, availHeight, autoMarginX, autoMarginY));
+            }
+            if(borderInfo != null) {
+                this.borderTopColor = borderInfo.topColor;
+                this.borderLeftColor = borderInfo.leftColor;
+                this.borderBottomColor = borderInfo.bottomColor;
+                this.borderRightColor = borderInfo.rightColor;
+            }
+            else {
+                this.borderTopColor = null; 
+                this.borderLeftColor = null;
+                this.borderBottomColor = null;
+                this.borderRightColor = null;                
+            }
 			String zIndex = props.getZIndex();
 			if(zIndex != null) {
 				try {
@@ -400,6 +479,8 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 			else {
 				this.zIndex = 0;
 			}
+			this.overflowX = rs.getOverflowX(); 
+			this.overflowY = rs.getOverflowY();
 		}
 
 		// Check if background image needs to be loaded
@@ -489,7 +570,7 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		int startY = 0;
 		ModelNode node = this.modelNode;
 		RenderState rs = node.getRenderState();
-		Insets marginInsets = this.getMarginInsets(rs);
+		Insets marginInsets = this.marginInsets;
 		if(marginInsets != null) {
 			totalWidth -= (marginInsets.left + marginInsets.right);
 			totalHeight -= (marginInsets.top + marginInsets.bottom);
@@ -513,10 +594,10 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 			// by the content area.
 			Rectangle clipBounds = g.getClipBounds();
 			if(!clientRegion.contains(clipBounds)) {
-				Insets borderStyles = this.borderStyles;
+			    BorderInfo borderInfo = this.borderInfo;
 				if(btop > 0) {
 					g.setColor(this.getBorderTopColor());
-					int borderStyle = borderStyles == null ? HtmlValues.BORDER_STYLE_SOLID : borderStyles.top;
+					int borderStyle = borderInfo == null ? HtmlValues.BORDER_STYLE_SOLID : borderInfo.topStyle;
 					for(int i = 0; i < btop; i++) {
 						int leftOffset = (i * bleft) / btop;
 						int rightOffset = (i * bright) / btop;
@@ -529,7 +610,7 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 					}
 				}
 				if(bright > 0) {
-					int borderStyle = borderStyles == null ? HtmlValues.BORDER_STYLE_SOLID : borderStyles.right;
+					int borderStyle = borderInfo == null ? HtmlValues.BORDER_STYLE_SOLID : borderInfo.rightStyle;
 					g.setColor(this.getBorderRightColor());
 					int lastX = startX + totalWidth - 1;
 					for(int i = 0; i < bright; i++) {
@@ -544,7 +625,7 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 					}				
 				}
 				if(bbottom > 0) {
-					int borderStyle = borderStyles == null ? HtmlValues.BORDER_STYLE_SOLID : borderStyles.bottom;
+					int borderStyle = borderInfo == null ? HtmlValues.BORDER_STYLE_SOLID : borderInfo.bottomStyle;
 					g.setColor(this.getBorderBottomColor());
 					int lastY = startY + totalHeight - 1;
 					for(int i = 0; i < bbottom; i++) {
@@ -559,7 +640,7 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 					}				
 				}
 				if(bleft > 0) {
-					int borderStyle = borderStyles == null ? HtmlValues.BORDER_STYLE_SOLID : borderStyles.left;
+					int borderStyle = borderInfo == null ? HtmlValues.BORDER_STYLE_SOLID : borderInfo.leftStyle;
 					g.setColor(this.getBorderLeftColor());
 					for(int i = 0; i < bleft; i++) {
 						int topOffset = (i * btop) / bleft;
@@ -675,24 +756,6 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 		}
 	}
 
-	void clearStyle() {
-		this.borderStyles = null;
-		this.borderInsets = null;
-		this.borderTopColor = null;
-		this.borderLeftColor = null;
-		this.borderBottomColor = null;
-		this.borderRightColor = null;
-		this.zIndex = 0;
-	}
-
-	protected final Insets getMarginInsets(RenderState rs) {
-		Insets mi = rs.getMarginInsets();
-		if(mi == null) {
-			return this.defaultMarginInsets;
-		}
-		return mi;
-	}	
-	
 	public boolean imageUpdate(Image img, int infoflags, int x, int y, int w, int h) {
 		// This is so that a loading image doesn't cause
 		// too many repaint events.
@@ -710,7 +773,7 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 	 */
 	public Insets getInsets(boolean hscroll, boolean vscroll) {
 		RenderState rs = this.modelNode.getRenderState();
-		Insets mi = this.getMarginInsets(rs);
+		Insets mi = this.marginInsets;
 		Insets bi = this.borderInsets;
 		int top = 0;
 		int bottom = 0;
@@ -791,4 +854,84 @@ abstract class BaseElementRenderable extends BaseRCollection implements RElement
 	public boolean isContainedByNode() {
 		return true;
 	}
+
+    public int getCollapsibleMarginBottom() {
+        int cm;
+        Insets paddingInsets = this.paddingInsets;
+        if(paddingInsets != null && paddingInsets.bottom > 0) {
+            cm = 0;
+        }
+        else {
+            Insets borderInsets = this.borderInsets;
+            if(borderInsets != null && borderInsets.bottom > 0) {
+                cm = 0;
+            }
+            else {
+                cm = this.getMarginBottom();
+            }
+        }
+        if(this.isMarginBoundary()) {
+            RenderState rs = this.modelNode.getRenderState();
+            if(rs != null) {
+                FontMetrics fm = rs.getFontMetrics();
+                int fontHeight = fm.getHeight();
+                if(fontHeight > cm) {
+                    cm = fontHeight;
+                }
+            }
+        }
+        return cm;
+    }
+    
+    protected boolean isMarginBoundary() {
+        return (this.overflowY != RenderState.OVERFLOW_VISIBLE && this.overflowX != RenderState.OVERFLOW_NONE) || this.modelNode instanceof HTMLDocumentImpl;
+    }
+
+    public int getCollapsibleMarginTop() {
+        int cm;
+        Insets paddingInsets = this.paddingInsets;
+        if(paddingInsets != null && paddingInsets.top > 0) {
+            cm = 0;
+        }
+        else {
+            Insets borderInsets = this.borderInsets;
+            if(borderInsets != null && borderInsets.top > 0) {
+                cm = 0;
+            }
+            else {
+                cm = this.getMarginTop();
+            }
+        }
+        if(this.isMarginBoundary()) {
+            RenderState rs = this.modelNode.getRenderState();
+            if(rs != null) {
+                FontMetrics fm = rs.getFontMetrics();
+                int fontHeight = fm.getHeight();
+                if(fontHeight > cm) {
+                    cm = fontHeight;
+                }
+            }
+        }
+        return cm;
+    }
+
+    public int getMarginBottom() {
+        Insets marginInsets = this.marginInsets;
+        return marginInsets == null ? 0 : marginInsets.bottom;
+    }
+
+    public int getMarginLeft() {
+        Insets marginInsets = this.marginInsets;
+        return marginInsets == null ? 0 : marginInsets.left;
+    }
+
+    public int getMarginRight() {
+        Insets marginInsets = this.marginInsets;
+        return marginInsets == null ? 0 : marginInsets.right;
+    }
+
+    public int getMarginTop() {
+        Insets marginInsets = this.marginInsets;
+        return marginInsets == null ? 0 : marginInsets.top;
+    }
 }
