@@ -37,6 +37,7 @@ abstract class BaseInputControl extends BaseControl implements InputContext {
 	}
 	
 	public void reset(int availWidth, int availHeight) {
+		super.reset(availWidth, availHeight);
 		String sizeText = this.controlElement.getAttribute("size");
 		if(sizeText != null) {
 			try {
@@ -114,6 +115,14 @@ abstract class BaseInputControl extends BaseControl implements InputContext {
 		return this.value;
 	}
 
+	/**
+	 * Returns <code>null</code>. It should be
+	 * overridden by controls that support multiple values.
+	 */
+	public String[] getValues() {
+		return null;
+	}
+	
 	/* (non-Javadoc)
 	 * @see org.xamjwg.html.domimpl.InputContext#select()
 	 */
