@@ -16,20 +16,18 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 package org.lobobrowser.html.renderer;
 
 public class DelayedPair {
-	public final int alignment;
 	public final RenderableContainer targetParent;
 	public final BoundableRenderable child;
 	public final int x;
 	public final int y;
 
-	public DelayedPair(final int alignment, final RenderableContainer parent, final BoundableRenderable child, int x, int y) {
+	public DelayedPair(final RenderableContainer parent, final BoundableRenderable child, int x, int y) {
 		super();
-		this.alignment = alignment;
 		this.targetParent = parent;
 		this.child = child;
 		this.x = x;
