@@ -16,12 +16,34 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 package org.lobobrowser.html.renderer;
 
 interface FloatingBounds {
+	/**
+	 * The offset from the left at the given Y, not counting insets.
+	 */
 	public int getLeft(int y);
+
+	/**
+	 * The offset from the right at the given Y, not counting insets.
+	 */
 	public int getRight(int y);
+	
+	/**
+	 * The Y at which the float clears starting at the given Y.
+	 */
 	public int getClearY(int y);
+	
+	/**
+	 * The Y at which the first float clears. It does
+	 * not recurse to check other floats.
+	 */
+	public int getFirstClearY(int y);
+	
+	public int getLeftClearY(int y);
+	public int getRightClearY(int y);
+	
+	public int getMaxY();
 }
