@@ -152,6 +152,10 @@ class FloatingViewportBounds implements FloatingBounds {
 	}
 	
 	public boolean equals(Object other) {
+        // Important for layout caching.
+	    if(other == this) {
+	        return true;
+	    }
 		if(!(other instanceof FloatingViewportBounds)) {
 			return false;
 		}
@@ -162,4 +166,8 @@ class FloatingViewportBounds implements FloatingBounds {
 				olm.offsetFromBorder == this.offsetFromBorder &&
 				org.lobobrowser.util.Objects.equals(olm.prevBounds, this.prevBounds);
 	}
+
+    public int hashCode() {
+        return (this.leftFloat ? 1 : 0) ^ this.y ^ this.height ^ this.offsetFromBorder;
+    }
 }
