@@ -7,7 +7,7 @@ import org.mozilla.javascript.*;
 import java.util.logging.*;
 import java.awt.event.*;
 
-class HtmlController implements Controller {
+class HtmlController {
 	private static final Logger logger = Logger.getLogger(HtmlController.class.getName());
 	private static final HtmlController instance = new HtmlController();
 	
@@ -46,6 +46,12 @@ class HtmlController implements Controller {
 					return false;
 				}
 			}
+			HtmlRendererContext rcontext = uiElement.getHtmlRendererContext();
+			if(rcontext != null) {
+				if(!rcontext.onMouseClick(uiElement, event)) {
+					return false;
+				}
+			}
 		}
 		if(node instanceof HTMLLinkElementImpl) {
 			((HTMLLinkElementImpl) node).navigate();
@@ -102,7 +108,11 @@ class HtmlController implements Controller {
 			}
 			HtmlRendererContext rcontext = uiElement.getHtmlRendererContext();
 			if(rcontext != null) {
-				rcontext.onContextMenu(uiElement, event);
+				// Needs to be done after Javascript, so the script
+				// is able to prevent it.
+				if(!rcontext.onContextMenu(uiElement, event)) {
+					return false;
+				}
 			}
 		}
 		ModelNode parent = node.getParentModelNode();
@@ -172,6 +182,12 @@ class HtmlController implements Controller {
 					return false;
 				}
 			}
+			HtmlRendererContext rcontext = uiElement.getHtmlRendererContext();
+			if(rcontext != null) {
+				if(!rcontext.onDoubleClick(uiElement, event)) {
+					return false;
+				}
+			}
 		}
 		ModelNode parent = node.getParentModelNode();
 		if(parent == null) {
@@ -295,4 +311,19 @@ class HtmlController implements Controller {
 		// No propagate
 		return false;
 	}	
+	
+	public boolean onChange(ModelNode node) {
+		if(node instanceof HTMLSelectElementImpl) {
+			HTMLSelectElementImpl uiElement = (HTMLSelectElementImpl) node;
+			Function f = uiElement.getOnchange();
+			if(f != null) {
+				Event jsEvent = new Event("change", uiElement);
+				if(!Executor.executeFunction(uiElement, f, jsEvent)) {
+					return false;
+				}
+			}
+		}
+		// No propagate
+		return false;
+	}
 }
