@@ -114,15 +114,45 @@ class ImgControl extends BaseControl implements ImageListener {
 	public Dimension createPreferredSize(int dw, int dh) {
 		Image img = this.image;
 		if(dw == -1) {
-			dw = img == null ? -1 : img.getWidth(this);
-			if(dw == -1) {
-				dw = 0;
+			if(dh != -1) {
+				int iw = img == null ? -1 : img.getWidth(this);
+				int ih = img == null ? -1 : img.getHeight(this);
+				if(ih == 0) {
+					dw = iw == -1 ? 0 : iw;
+				}
+				else if(iw == -1 || ih == -1) {
+					dw = 0;
+				}
+				else {
+					dw = dh * iw / ih;
+				}
+			}
+			else {
+				dw = img == null ? -1 : img.getWidth(this);
+				if(dw == -1) {
+					dw = 0;
+				}
 			}
 		}
 		if(dh == -1) {
-			dh = img == null ? -1 : img.getHeight(this);			
-			if(dh == -1) {
-				dh = 0;
+			if(dw != -1) {
+				int iw = img == null ? -1 : img.getWidth(this);
+				int ih = img == null ? -1 : img.getHeight(this);
+				if(iw == 0) {
+					dh = ih == -1 ? 0 : ih;
+				}
+				else if(iw == -1 || ih == -1) {
+					dh = 0;
+				}
+				else {
+					dh = dw * ih / iw;
+				}
+			}
+			else {
+				dh = img == null ? -1 : img.getHeight(this);			
+				if(dh == -1) {
+					dh = 0;
+				}
 			}
 		}
 		return new Dimension(dw, dh);
@@ -143,28 +173,6 @@ class ImgControl extends BaseControl implements ImageListener {
 		}
 	}
 	
-//	private void checkImgSrc() {
-//		ModelNode rc = this.controlElement;
-//		if(rc instanceof HTMLImageElement) {
-//			HTMLImageElement ie = (HTMLImageElement) rc;
-//			String newSrc = ie.getSrc();
-//			if(newSrc == null) {
-//				this.lastSrc = null;
-//				this.image = null;
-//			}
-//			else if(!newSrc.equals(this.lastSrc)) {
-//				this.lastSrc = newSrc;
-//				this.image = null;
-//				this.loadImage(newSrc);
-//			}
-//		}		
-//	}
-//	
-//	public void invalidate() {
-//		super.invalidate();
-//		this.checkImgSrc();
-//	}
-		
 	/* (non-Javadoc)
 	 * @see java.awt.Component#imageUpdate(java.awt.Image, int, int, int, int, int)
 	 */
