@@ -37,6 +37,7 @@ class InputButtonControl extends BaseInputControl {
 		super(modelNode);
 		this.setLayout(WrapperLayout.getInstance());
 		JButton widget = new JButton();
+		widget.setContentAreaFilled(false);
 		this.widget = widget;
 		this.add(widget);
 		widget.addActionListener(new ActionListener() {
@@ -48,9 +49,16 @@ class InputButtonControl extends BaseInputControl {
 	
 	public void reset(int availWidth, int availHeight) {
 		super.reset(availWidth, availHeight);
+		RUIControl ruiControl = this.ruicontrol;
+		JButton button = this.widget;
+		button.setContentAreaFilled(!ruiControl.hasBackground());
+		java.awt.Color foregroundColor = ruiControl.getForegroundColor();
+		if(foregroundColor != null) {
+			button.setForeground(foregroundColor);
+		}
 		HTMLInputElementImpl element = (HTMLInputElementImpl) this.controlElement;
 		String text = element.getAttribute("value");
-		if(text == null || text == "") {
+		if(text == null || text.length() == 0) {
 			String type = element.getType();
 			if("submit".equalsIgnoreCase(type)) {
 				text = "Submit Query";
@@ -62,7 +70,7 @@ class InputButtonControl extends BaseInputControl {
 				text = "";
 			}
 		}
-		this.widget.setText(text);		
+		button.setText(text);		
 	}
 	
 	/* (non-Javadoc)
