@@ -40,8 +40,7 @@ class InputCheckboxControl extends BaseInputControl {
 		//Note: Value attribute cannot be set in reset() method.
 		//Otherwise, layout revalidation causes typed values to
 		//be lost (including revalidation due to hover.)
-		String checkedText = this.controlElement.getAttribute("checked");
-	    checkBox.setSelected("checked".equalsIgnoreCase(checkedText));
+	    checkBox.setSelected(this.controlElement.getAttributeAsBoolean("checked"));
 
 		this.add(checkBox);
 	}	
@@ -80,8 +79,7 @@ class InputCheckboxControl extends BaseInputControl {
 	}
 	
 	public void resetInput() {
-		String checkedText = this.controlElement.getAttribute("checked");
-		this.widget.setSelected("checked".equalsIgnoreCase(checkedText));		
+		this.widget.setSelected(this.controlElement.getAttributeAsBoolean("checked"));		
 	}
 	
 	public String getValue() {
