@@ -60,8 +60,7 @@ class InputRadioControl extends BaseInputControl {
 		else {
 			this.buttonGroup = null;
 		}
-		String checkedText = controlElement.getAttribute("checked");
-		radio.setSelected("checked".equalsIgnoreCase(checkedText));		
+		radio.setSelected(controlElement.getAttributeAsBoolean("checked"));		
 
 		this.add(radio);
 	}	
@@ -102,8 +101,7 @@ class InputRadioControl extends BaseInputControl {
 	}
 	
 	public void resetInput() {
-		String checkedText = controlElement.getAttribute("checked");
-		this.widget.setSelected("checked".equalsIgnoreCase(checkedText));				
+		this.widget.setSelected(this.controlElement.getAttributeAsBoolean("checked"));				
 	}
 	
 	public String getValue() {
