@@ -16,7 +16,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 /*
  * Created on Nov 6, 2005
@@ -30,5 +30,5 @@ import java.awt.Insets;
 import org.lobobrowser.html.domimpl.HTMLElementImpl;
 
 interface MarkupLayout {
-	public void layoutMarkup(RBlockViewport bodyLayout, RenderableContainer container, Dimension containerSize, Insets insets, HTMLElementImpl markupElement);
+	public void layoutMarkup(RBlockViewport bodyLayout, HTMLElementImpl markupElement);
 }
