@@ -25,11 +25,13 @@ class PositionedRenderable {
 	public final BoundableRenderable renderable;
 	public final boolean verticalAlignable;
 	public final int ordinal;
+	public final boolean isFloat;
 	
-	public PositionedRenderable(final BoundableRenderable renderable, final boolean verticalAlignable, int ordinal) {
+	public PositionedRenderable(final BoundableRenderable renderable, final boolean verticalAlignable, int ordinal, boolean isFloat) {
 		super();
 		this.renderable = renderable;
 		this.verticalAlignable = verticalAlignable;
 		this.ordinal = ordinal;
+		this.isFloat = isFloat;
 	}		
 }
\ No newline at end of file
