@@ -33,8 +33,9 @@ public interface RElement extends RCollection, UINode, BoundableRenderable {
 	 * The RElement is expected to set its own dimensions, but not its origin.
 	 * @param availWidth The available width from the parent's canvas.
 	 * @param availHeight The available height from the parent's canvas.
+	 * @param sizeOnly Whether the layout is for sizing determination only.
 	 */
-	public void layout(int availWidth, int availHeight);
+	public void layout(int availWidth, int availHeight, boolean sizeOnly);
 	
 	/**
 	 * Vertical alignment for elements rendered in a line.
@@ -42,6 +43,14 @@ public interface RElement extends RCollection, UINode, BoundableRenderable {
 	 */
 	public int getVAlign();
 	
+	public int getMarginTop();
+    public int getMarginLeft();
+    public int getMarginBottom();
+    public int getMarginRight();
+    
+    public int getCollapsibleMarginTop();
+    public int getCollapsibleMarginBottom();
+	
 	public static final int VALIGN_TOP = 0;
 	public static final int VALIGN_MIDDLE = 1;
 	public static final int VALIGN_BOTTOM = 2;
