@@ -29,25 +29,7 @@ public class RImgControl extends RUIControl {
 		super(me, widget, container, frameContext, ucontext);
 	}
 
-	protected void applyStyle() {
-		super.applyStyle();
-		//TODO: Should borderInsets be moved to RenderState?
-		if(this.borderInsets == null) {
-			HTMLElementImpl imageElement = (HTMLElementImpl) this.modelNode;
-			String borderText = imageElement.getAttribute("border");
-			if(borderText != null && borderText.length() != 0) {
-				int border;
-				try {
-					//TODO: Percentages supported?
-					border = Integer.parseInt(borderText);
-				} catch(NumberFormatException nfe) {
-					// Ignore?
-					border = 0;
-				}
-				this.borderInsets = new Insets(border, border, border, border);
-			}
-		}
+	protected void applyStyle(int availWidth, int availHeight) {
+		super.applyStyle(availWidth, availHeight);
 	}
-
-	
 }
