@@ -43,7 +43,10 @@ class RLine extends BaseRCollection {
 	 */
 	private int xoffset;
 	
-	public RLine(ModelNode modelNode, RenderableContainer container, int x, int y, int desiredMaxWidth, int height) {
+	private boolean allowOverflow = false;
+	private boolean firstAllowOverflowWord = false;
+	
+	public RLine(ModelNode modelNode, RenderableContainer container, int x, int y, int desiredMaxWidth, int height, boolean initialAllowOverflow) {
 		// Note that in the case of RLine, modelNode is the context node
 		// at the beginning of the line, not a node that encloses the whole line.
 		super(container, modelNode);
@@ -53,6 +56,23 @@ class RLine extends BaseRCollection {
 		this.desiredMaxWidth = desiredMaxWidth;
 		// Layout here can always be "invalidated"
 		this.layoutUpTreeCanBeInvalidated = true;
+		this.allowOverflow = initialAllowOverflow;
+	}
+	
+	
+	public void setAllowOverflow(boolean flag) {
+	    if(flag != this.allowOverflow) {
+	        this.allowOverflow = flag;
+	        if(flag) {
+	            // Set to true only if allowOverflow was
+	            // previously false.
+	            this.firstAllowOverflowWord = true;
+	        }
+	    }
+	}
+	
+	public boolean isAllowOverflow() {
+	    return this.allowOverflow;
 	}
 	
 	/**
@@ -145,15 +165,15 @@ class RLine extends BaseRCollection {
 	 * Note that RLine does not set sizes, but only origins.
 	 * @throws OverflowException Thrown if the renderable overflows the line. All overflowing renderables are added to the exception.
 	 */
-	public final void add(Renderable renderable, int overflowType) throws OverflowException {
+	public final void add(Renderable renderable) throws OverflowException {
 		if(renderable instanceof RWord) {
-			this.addWord((RWord) renderable, overflowType);
+			this.addWord((RWord) renderable);
 		}
 		else if(renderable instanceof RBlank) {
 			this.addBlank((RBlank) renderable);
 		}
 		else if(renderable instanceof RUIControl) {
-			this.addElement((RElement) renderable, overflowType);
+			this.addElement((RElement) renderable);
 		}
 		else if(renderable instanceof RSpacing) {
 			this.addSpacing((RSpacing) renderable);
@@ -169,15 +189,16 @@ class RLine extends BaseRCollection {
 		}
 	}
 	
-	public static final int OVERFLOW_ALLOW_ALL = 0;
-	public static final int OVERFLOW_ALLOW_FIRST = 1;
-	public static final int OVERFLOW_ALLOW_NONE = 2;
-		
-	public final void addWord(RWord rword, int overflowType) throws OverflowException {
+	public final void addWord(RWord rword) throws OverflowException {
 		// Check if it fits horzizontally
 		int offset = this.xoffset;
 		int wiwidth = rword.width;
-		if(overflowType != OVERFLOW_ALLOW_ALL && (offset != 0 || overflowType == OVERFLOW_ALLOW_NONE) && (offset + wiwidth > this.desiredMaxWidth)) {
+		boolean allowOverflow = this.allowOverflow;
+		boolean firstAllowOverflowWord = this.firstAllowOverflowWord;
+        if(allowOverflow && firstAllowOverflowWord) {
+            this.firstAllowOverflowWord = false;
+        }
+		if((!allowOverflow || firstAllowOverflowWord) && offset != 0 && (offset + wiwidth > this.desiredMaxWidth)) {
 			ArrayList renderables = this.renderables;
 			ArrayList overflow = null;
 			boolean cancel = false;
@@ -217,7 +238,16 @@ class RLine extends BaseRCollection {
 					break;
 				}
 			}
-			if(!cancel) {
+			if(cancel) {
+			   // Oops. Need to undo overflow.
+			   if(overflow != null) {
+			       Iterator i = overflow.iterator();
+			       while(i.hasNext()) {
+			          renderables.add(i.next()); 
+			       }
+			   }
+			}
+			else {
 				this.xoffset = newOffset;
 				this.width = newWidth;
 				if(overflow == null) {
@@ -309,12 +339,17 @@ class RLine extends BaseRCollection {
 		relement.setY(yoffset);
 	}
 	
-	private final void addElement(RElement relement, int overflowType) throws OverflowException {
+	private final void addElement(RElement relement) throws OverflowException {
 		// Check if it fits horizontally
 		int origXOffset = this.xoffset;
 		int desiredMaxWidth = this.desiredMaxWidth;
 		int pw = relement.getWidth();
-		if(overflowType != OVERFLOW_ALLOW_ALL && (origXOffset != 0 || overflowType == OVERFLOW_ALLOW_NONE) && (origXOffset + pw > desiredMaxWidth)) {
+        boolean allowOverflow = this.allowOverflow;
+        boolean firstAllowOverflowWord = this.firstAllowOverflowWord;
+        if(allowOverflow && firstAllowOverflowWord) {
+            this.firstAllowOverflowWord = false;
+        }
+		if((!allowOverflow || firstAllowOverflowWord) && origXOffset != 0  && (origXOffset + pw > desiredMaxWidth)) {
 			throw new OverflowException(Collections.singleton(relement));
 		}
 		//Note: Renderable for widget doesn't paint the widget, but
