@@ -30,29 +30,18 @@ import org.lobobrowser.html.style.RenderState;
 class RList extends BaseRListElement {
 	public RList(NodeImpl modelNode, int listNesting, UserAgentContext pcontext, HtmlRendererContext rcontext, FrameContext frameContext, RenderableContainer parentContainer, RCollection parent) {
 		super(modelNode, listNesting, pcontext, rcontext, frameContext,	parentContainer);
-		this.defaultMarginInsets = new java.awt.Insets(5, 0, 5, 0);
+		//this.defaultMarginInsets = new java.awt.Insets(5, 0, 5, 0);
 	}
 
-	private int counterStart;
-	
-	protected void applyStyle() {
-		super.applyStyle();
-		this.counterStart = 1;
-		Object rootNode = this.modelNode;
-		if(!(rootNode instanceof HTMLElementImpl)) {
-			return;
-		}
-		HTMLElementImpl rootElement = (HTMLElementImpl) rootNode;
-		String startText = rootElement.getAttribute("start");
-		if(startText != null) {
-			try {
-				this.counterStart = Integer.parseInt(startText);
-			} catch(NumberFormatException nfe) {
-				// ignore
-			}
-		}
+	protected void applyStyle(int availWidth, int availHeight) {
+		super.applyStyle(availWidth, availHeight);
 		ListStyle listStyle = this.listStyle;
 		if(listStyle == null || listStyle.type == ListStyle.TYPE_UNSET) {
+			Object rootNode = this.modelNode;
+			if(!(rootNode instanceof HTMLElementImpl)) {
+				return;
+			}
+			HTMLElementImpl rootElement = (HTMLElementImpl) rootNode;
 			if(listStyle == null) {
 				listStyle = new ListStyle();
 				this.listStyle = listStyle;
@@ -75,9 +64,23 @@ class RList extends BaseRListElement {
 		}	
 	}
 
-	public void doLayout(int availWidth, int availHeight, FloatingBounds floatBounds, int tentativeY, int defaultOverflow) {
+	public void doLayout(int availWidth, int availHeight, boolean expandWidth, boolean expandHeight, FloatingBoundsSource floatBoundsSource, int defaultOverflowX, int defaultOverflowY, boolean sizeOnly) {
 		RenderState renderState = this.modelNode.getRenderState();
-		renderState.resetCount(DEFAULT_COUNTER_NAME, this.listNesting, this.counterStart);
-		super.doLayout(availWidth, availHeight, floatBounds, tentativeY, defaultOverflow);
+		int counterStart = 1;
+		Object rootNode = this.modelNode;
+		if(!(rootNode instanceof HTMLElementImpl)) {
+			return;
+		}
+		HTMLElementImpl rootElement = (HTMLElementImpl) rootNode;
+		String startText = rootElement.getAttribute("start");
+		if(startText != null) {
+			try {
+				counterStart = Integer.parseInt(startText);
+			} catch(NumberFormatException nfe) {
+				// ignore
+			}
+		}
+		renderState.resetCount(DEFAULT_COUNTER_NAME, this.listNesting, counterStart);
+		super.doLayout(availWidth, availHeight, expandWidth, expandHeight, floatBoundsSource, defaultOverflowX, defaultOverflowY, sizeOnly);
 	}
 }	
\ No newline at end of file
