@@ -37,7 +37,7 @@ class RListItem extends BaseRListElement {
 
 	public RListItem(NodeImpl modelNode, int listNesting, UserAgentContext pcontext, HtmlRendererContext rcontext, FrameContext frameContext, RenderableContainer parentContainer, RCollection parent) {
 		super(modelNode, listNesting, pcontext, rcontext, frameContext, parentContainer);
-		this.defaultMarginInsets = new java.awt.Insets(0, BULLET_SPACE_WIDTH, 0, 0);
+		//this.defaultMarginInsets = new java.awt.Insets(0, BULLET_SPACE_WIDTH, 0, 0);
 	}
 	
 	public int getViewportListNesting(int blockNesting) {
@@ -74,8 +74,8 @@ class RListItem extends BaseRListElement {
 
 	private int count; 
 	
-	public void doLayout(int availWidth, int availHeight, FloatingBounds floatBounds, int tentativeY, int defaultOverflow) {
-		super.doLayout(availWidth, availHeight, floatBounds, tentativeY, defaultOverflow);
+	public void doLayout(int availWidth, int availHeight, boolean expandWidth, boolean expandHeight, FloatingBoundsSource floatBoundsSource, int defaultOverflowX, int defaultOverflowY, boolean sizeOnly) {
+		super.doLayout(availWidth, availHeight, expandWidth, expandHeight, floatBoundsSource, defaultOverflowX, defaultOverflowY, sizeOnly);
 		//Note: Count must be calculated even if layout is valid.
 		RenderState renderState = this.modelNode.getRenderState();
 		Integer value = this.getValue();
@@ -92,65 +92,67 @@ class RListItem extends BaseRListElement {
 	public void paint(Graphics g) {
 		super.paint(g);
 		RenderState rs = this.modelNode.getRenderState();
-		Insets marginInsets = this.getMarginInsets(rs);
+		Insets marginInsets = this.marginInsets;
 		RBlockViewport layout = this.bodyLayout;
 		if(layout != null) {
 			ListStyle listStyle = this.listStyle;
 			int bulletType = listStyle == null ? ListStyle.TYPE_UNSET : listStyle.type;
-			if(bulletType == ListStyle.TYPE_UNSET) {
-				RCollection parent = this.getOriginalOrCurrentParent();
-				if(!(parent instanceof RList)) {
-					parent = parent.getOriginalOrCurrentParent();
+			if(bulletType != ListStyle.TYPE_NONE) {
+				if(bulletType == ListStyle.TYPE_UNSET) {
+					RCollection parent = this.getOriginalOrCurrentParent();
+					if(!(parent instanceof RList)) {
+						parent = parent.getOriginalOrCurrentParent();
+					}
+					if(parent instanceof RList) {
+						ListStyle parentListStyle = ((RList) parent).listStyle;
+						bulletType = parentListStyle == null ? ListStyle.TYPE_DISC : parentListStyle.type;
+					}
+					else {
+						bulletType = ListStyle.TYPE_DISC;
+					}
 				}
-				if(parent instanceof RList) {
-					ListStyle parentListStyle = ((RList) parent).listStyle;
-					bulletType = parentListStyle == null ? ListStyle.TYPE_DISC : parentListStyle.type;
+				// Paint bullets
+				Color prevColor = g.getColor();
+				g.setColor(rs.getColor());
+				try {
+					Insets insets = this.getInsets(this.hasHScrollBar, this.hasVScrollBar);
+					Insets paddingInsets = this.paddingInsets;
+					int baselineOffset = layout.getFirstBaselineOffset();
+					int bulletRight = (marginInsets == null ? 0 : marginInsets.left) - BULLET_RMARGIN;
+					int bulletBottom = insets.top + baselineOffset + (paddingInsets == null ? 0 : paddingInsets.top);
+					int bulletTop = bulletBottom - BULLET_HEIGHT;
+					int bulletLeft = bulletRight - BULLET_WIDTH;
+					int bulletNumber = this.count;
+					String numberText = null;
+					switch(bulletType) {
+					case ListStyle.TYPE_DECIMAL:
+						numberText = bulletNumber + ".";
+						break;
+					case ListStyle.TYPE_LOWER_ALPHA:
+						numberText = ((char) ('a' + bulletNumber)) + ".";
+						break;
+					case ListStyle.TYPE_UPPER_ALPHA:
+						numberText = ((char) ('A' + bulletNumber)) + ".";
+						break;
+					case ListStyle.TYPE_DISC:
+						g.fillOval(bulletLeft, bulletTop, BULLET_WIDTH, BULLET_HEIGHT);
+						break;					
+					case ListStyle.TYPE_CIRCLE:
+						g.drawOval(bulletLeft, bulletTop, BULLET_WIDTH, BULLET_HEIGHT);
+						break;
+					case ListStyle.TYPE_SQUARE:
+						g.fillRect(bulletLeft, bulletTop, BULLET_WIDTH, BULLET_HEIGHT);					
+						break;
+					}
+					if(numberText != null) {
+						FontMetrics fm = g.getFontMetrics();
+						int numberLeft = bulletRight - fm.stringWidth(numberText);
+						int numberY = bulletBottom;
+						g.drawString(numberText, numberLeft, numberY);
+					}
+				} finally {
+					g.setColor(prevColor);
 				}
-				else {
-					bulletType = ListStyle.TYPE_DISC;
-				}
-			}
-			// Paint bullets
-			Color prevColor = g.getColor();
-			g.setColor(Color.BLACK);
-			try {
-				Insets insets = this.getInsets(this.hasHScrollBar, this.hasVScrollBar);
-				Insets paddingInsets = this.getPaddingInsets(rs);
-				int baselineOffset = layout.getFirstBaselineOffset();
-				int bulletRight = marginInsets.left - BULLET_RMARGIN;
-				int bulletBottom = insets.top + baselineOffset + (paddingInsets == null ? 0 : paddingInsets.top);
-				int bulletTop = bulletBottom - BULLET_HEIGHT;
-				int bulletLeft = bulletRight - BULLET_WIDTH;
-				int bulletNumber = this.count;
-				String numberText = null;
-				switch(bulletType) {
-				case ListStyle.TYPE_DECIMAL:
-					numberText = bulletNumber + ".";
-					break;
-				case ListStyle.TYPE_LOWER_ALPHA:
-					numberText = ((char) ('a' + bulletNumber)) + ".";
-					break;
-				case ListStyle.TYPE_UPPER_ALPHA:
-					numberText = ((char) ('A' + bulletNumber)) + ".";
-					break;
-				case ListStyle.TYPE_DISC:
-					g.fillOval(bulletLeft, bulletTop, BULLET_WIDTH, BULLET_HEIGHT);
-					break;					
-				case ListStyle.TYPE_CIRCLE:
-					g.drawOval(bulletLeft, bulletTop, BULLET_WIDTH, BULLET_HEIGHT);
-					break;
-				case ListStyle.TYPE_SQUARE:
-					g.fillRect(bulletLeft, bulletTop, BULLET_WIDTH, BULLET_HEIGHT);					
-					break;
-				}
-				if(numberText != null) {
-					FontMetrics fm = g.getFontMetrics();
-					int numberLeft = bulletRight - fm.stringWidth(numberText);
-					int numberY = bulletBottom;
-					g.drawString(numberText, numberLeft, numberY);
-				}
-			} finally {
-				g.setColor(prevColor);
 			}
 		}
 	}
