@@ -25,10 +25,7 @@ package org.lobobrowser.html.renderer;
 
 import java.awt.*;
 import java.awt.event.MouseEvent;
-import java.util.Collection;
-import java.util.Iterator;
-import java.util.SortedSet;
-import java.util.TreeSet;
+import java.util.*;
 
 import org.lobobrowser.html.*;
 import org.lobobrowser.util.*;
@@ -38,9 +35,13 @@ import org.lobobrowser.html.style.RenderState;
 import org.w3c.dom.css.*;
 
 class RTable extends BaseElementRenderable {
+    private static final int MAX_CACHE_SIZE = 10;
+    private final Map cachedLayout = new HashMap(5);
 	private final TableMatrix tableMatrix;
 	private SortedSet positionedRenderables;
 	private int otherOrdinal;
+	private LayoutKey lastLayoutKey = null;
+	private LayoutValue lastLayoutValue = null;
 	
 	public RTable(HTMLElementImpl modelNode, UserAgentContext pcontext, HtmlRendererContext rcontext, FrameContext frameContext, RenderableContainer container) {
 		super(container, modelNode, pcontext);
@@ -52,69 +53,6 @@ class RTable extends BaseElementRenderable {
 		return VALIGN_BASELINE;
 	}
 
-	protected void applyStyle() {
-		super.applyStyle();
-		Insets bi = this.borderInsets;
-		if(bi == null) {
-			bi = new Insets(0,0,0,0);
-		}
-		HTMLElementImpl element = (HTMLElementImpl) this.modelNode;
-		String borderText = element.getAttribute("border");
-		int border = 0;
-		if(borderText != null) {
-			try {
-				border = Integer.parseInt(borderText);
-				if(border < 0) {
-					border = 0;
-				}
-			} catch(NumberFormatException nfe) {
-				// ignore
-			}
-			CSS2Properties props = element.getCurrentStyle();
-			if(props != null) {
-				int top, left, bottom, right;
-				boolean changed = false;
-				if(Strings.isBlank(props.getBorderTopWidth())) {
-					changed = true;
-					top = border;
-				}
-				else {
-					top = bi.top;
-				}
-				if(Strings.isBlank(props.getBorderLeftWidth())) {
-					changed = true;
-					left = border;
-				}
-				else {
-					left = bi.left;
-				}
-				if(Strings.isBlank(props.getBorderBottomWidth())) {
-					changed = true;
-					bottom = border;
-				}
-				else {
-					bottom = bi.bottom;
-				}
-				if(Strings.isBlank(props.getBorderRightWidth())) {
-					changed = true;
-					right = border;
-				}
-				else {
-					right = bi.right;
-				}
-				if(changed) {
-					this.borderInsets = new Insets(top, left, bottom, right);
-				}
- 			}
-		}
-		if(this.borderTopColor == null && this.borderLeftColor == null) {
-			this.borderTopColor = this.borderLeftColor = Color.LIGHT_GRAY;
-		}
-		if(this.borderBottomColor == null && this.borderRightColor == null) {
-			this.borderBottomColor = this.borderRightColor = Color.DARK_GRAY;
-		}
-	}
-
 	public void paint(Graphics g) {
 		RenderState rs = this.modelNode.getRenderState();
 		if(rs != null && rs.getVisibility() != RenderState.VISIBILITY_VISIBLE) {
@@ -141,13 +79,26 @@ class RTable extends BaseElementRenderable {
 			super.paint(g);
 		}
 	}
-	
-	private volatile int lastAvailWidth = -1;
-	private volatile int lastAvailHeight = -1;
-//	private volatile Dimension layoutSize = null;
-	
-	public void doLayout(int availWidth, int availHeight) {
-		if (availWidth != this.lastAvailWidth || availHeight != this.lastAvailHeight) {
+		
+	public void doLayout(int availWidth, int availHeight, boolean sizeOnly) {
+	    Map cachedLayout = this.cachedLayout;
+	    RenderState rs = this.modelNode.getRenderState();
+	    int whitespace = rs == null ? RenderState.WS_NORMAL : rs.getWhiteSpace();
+	    Font font = rs == null ? null : rs.getFont();
+	    LayoutKey layoutKey = new LayoutKey(availWidth, availHeight, whitespace, font);
+	    LayoutValue layoutValue;
+	    if(sizeOnly) {
+	        layoutValue = (LayoutValue) cachedLayout.get(layoutKey);
+	    }
+	    else {
+	        if(Objects.equals(layoutKey, this.lastLayoutKey)) {
+	            layoutValue = this.lastLayoutValue;
+	        }
+	        else {
+	            layoutValue = null;
+	        }
+	    }
+	    if(layoutValue == null) {
 			Collection prs = this.positionedRenderables;
 			if(prs != null) {
 				prs.clear();
@@ -155,17 +106,13 @@ class RTable extends BaseElementRenderable {
 			this.otherOrdinal = 0;
 			this.clearGUIComponents();
 			this.clearDelayedPairs();
-			this.applyStyle();
-			this.lastAvailHeight = availHeight;
-			this.lastAvailWidth = availWidth;
+			this.applyStyle(availWidth, availHeight);
 			TableMatrix tm = this.tableMatrix;
 			Insets insets = this.getInsets(false, false);
 			tm.reset(insets, availWidth, availHeight);
 			//TODO: No scrollbars
-			tm.build(availWidth, availHeight);
+			tm.build(availWidth, availHeight, sizeOnly);
 			tm.doLayout(insets);
-			this.width = tm.getTableWidth();
-			this.height = tm.getTableHeight();
 			
 			// Import applicable delayed pairs.
 			// Only needs to be done if layout was
@@ -181,29 +128,32 @@ class RTable extends BaseElementRenderable {
 					}
 				}
 			}
-			
-		} else {
-			// Nothing to do - dimensions already set.
-		}
+			layoutValue = new LayoutValue(tm.getTableWidth(), tm.getTableHeight());
+			if(sizeOnly) {
+			    if(cachedLayout.size() > MAX_CACHE_SIZE) {
+			        // Unlikely, but we should ensure it's bounded.
+			        cachedLayout.clear();
+			    }
+			    cachedLayout.put(layoutKey, layoutValue);
+			    this.lastLayoutKey = null;
+			    this.lastLayoutValue = null;
+			}
+			else {
+			    this.lastLayoutKey = layoutKey;
+			    this.lastLayoutValue = layoutValue;
+			}
+	    }
+	    this.width = layoutValue.width;
+	    this.height = layoutValue.height;
 		this.sendGUIComponentsToParent();
 		this.sendDelayedPairsToParent();
 	}
-	
-//	/* (non-Javadoc)
-//	 * @see org.xamjwg.html.renderer.UIControl#paintSelection(java.awt.Graphics, boolean, org.xamjwg.html.renderer.RenderablePoint, org.xamjwg.html.renderer.RenderablePoint)
-//	 */
-//	public boolean paintSelection(Graphics g, boolean inSelection, RenderableSpot startPoint, RenderableSpot endPoint) {
-//		return this.tableMatrix.paintSelection(g, inSelection, startPoint, endPoint);
-//	}
-//
-//	public boolean extractSelectionText(StringBuffer buffer, boolean inSelection, RenderableSpot startPoint, RenderableSpot endPoint) {
-//		return this.tableMatrix.extractSelectionText(buffer, inSelection, startPoint, endPoint);
-//	}
-	
+
 	public void invalidateLayoutLocal() {
 		super.invalidateLayoutLocal();
-		this.lastAvailHeight = -1;
-		this.lastAvailWidth = -1;
+		this.cachedLayout.clear();
+		this.lastLayoutKey = null;
+		this.lastLayoutValue = null;
 	}
 	
 	/* (non-Javadoc)
@@ -362,14 +312,14 @@ class RTable extends BaseElementRenderable {
 		return this.container.getPaintedBackgroundColor();
 	}
 	
-	private final void addPositionedRenderable(BoundableRenderable renderable, boolean verticalAlignable) {
+	private final void addPositionedRenderable(BoundableRenderable renderable, boolean verticalAlignable, boolean isFloat) {
 		// Expected to be called only in GUI thread.
 		SortedSet others = this.positionedRenderables;
 		if(others == null) {
 			others = new TreeSet(new ZIndexComparator());
 			this.positionedRenderables = others;
 		}
-		others.add(new PositionedRenderable(renderable, verticalAlignable, this.otherOrdinal++));
+		others.add(new PositionedRenderable(renderable, verticalAlignable, this.otherOrdinal++, isFloat));
 		renderable.setParent(this);
 		if(renderable instanceof RUIControl) {
 			this.container.addComponent(((RUIControl) renderable).widget.getComponent());
@@ -379,10 +329,55 @@ class RTable extends BaseElementRenderable {
 	private void importDelayedPair(DelayedPair pair) {
 		BoundableRenderable r = pair.child;
 		r.setOrigin(pair.x, pair.y);
-		this.addPositionedRenderable(r, false);
+		this.addPositionedRenderable(r, false, false);
 	}
-	
+		
 	public String toString() {
 		return "RTable[this=" + System.identityHashCode(this) + ",node=" + this.modelNode + "]";
 	}
+	
+	private static class LayoutKey {
+	    public final int availWidth;
+	    public final int availHeight;
+	    public final int whitespace;
+	    public final Font font;
+
+	    public LayoutKey(int availWidth, int availHeight, int whitespace, Font font) {
+	        super();
+	        this.availWidth = availWidth;
+	        this.availHeight = availHeight;
+	        this.whitespace = whitespace;
+	        this.font = font;
+	    }
+
+	    public boolean equals(Object obj) {
+	        if(obj == this) {
+	            return true;
+	        }
+	        if(!(obj instanceof LayoutKey)) {
+	            return false;
+	        }
+	        LayoutKey other = (LayoutKey) obj;
+	        return other.availWidth == this.availWidth &&
+	        other.availHeight == this.availHeight &&
+	        other.whitespace == this.whitespace &&
+	        Objects.equals(other.font, this.font);
+	    }
+	    
+	    public int hashCode() {
+	        Font font = this.font;
+	        return (this.availWidth * 1000 + this.availHeight) ^ (font == null ? 0 : font.hashCode()); 
+	    }
+	}
+
+	private static class LayoutValue {
+	    public final int width;
+	    public final int height;
+
+	    public LayoutValue(int width, int height) {
+	        this.width = width;
+	        this.height = height;
+	    }
+	}
+
 }
