@@ -34,7 +34,7 @@ import org.lobobrowser.html.style.TableCellRenderState;
 class RTableCell extends RBlock {
 	private final HTMLTableCellElementImpl cellElement;
 	private VirtualCell topLeftVirtualCell; 
-	private int cellPadding;
+	//private int cellPadding;
 	
 	/**
 	 * @param element
@@ -48,21 +48,19 @@ class RTableCell extends RBlock {
 	 * @param width The width available, including insets.
 	 * @param height The height available, including insets.
 	 */
-	protected Dimension doCellLayout(int width, int height, boolean isFinal) {
+	protected Dimension doCellLayout(int width, int height, boolean expandWidth, boolean expandHeight, boolean sizeOnly) {
 		try {
-			this.doLayout(width, height);
-			if(isFinal) {
-				this.adjust(width, height, true, true, null);
-			}
+			this.layout(width, height, expandWidth, expandHeight, null, sizeOnly);
 			return new Dimension(this.width, this.height);
 		} finally {
 			this.layoutUpTreeCanBeInvalidated = true;
+			this.layoutDeepCanBeInvalidated = true;
 		}
 	}
 
-	public void setCellPadding(int value) {
-		this.cellPadding = value;
-	}
+//	public void setCellPadding(int value) {
+//		this.cellPadding = value;
+//	}
 	
     protected Integer getDeclaredHeight(RenderState renderState, int availHeight) {
     	// Overridden since height declaration is handled by table.
@@ -129,6 +127,10 @@ class RTableCell extends RBlock {
 		return rs;
 	}
 
+	public void setRowSpan(int rowSpan) {
+		this.rowSpan = rowSpan;
+	}
+	
 	public String getHeightText() {
 		return this.cellElement.getHeight();
 	}
@@ -186,4 +188,8 @@ class RTableCell extends RBlock {
 		}
 		this.setBounds(x, y, width, height);
 	}
+	
+    protected boolean isMarginBoundary() {
+        return true;
+    }       
 }
