@@ -16,7 +16,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 /*
  * Created on Apr 16, 2005
@@ -29,14 +29,9 @@ import org.lobobrowser.html.domimpl.*;
 
 /**
  * Represents a renderer (view) node.
- * @author J. H. S.
  */
 public interface Renderable {
 	public static final Renderable[] EMPTY_ARRAY = new Renderable[0];
 	public void paint(Graphics g);
-	public void invalidateLayoutUpTree();
-	public int getOrdinal();
-	public void setOrdinal(int ordinal);
-	public int getZIndex();
 	public ModelNode getModelNode();
 }
