@@ -51,6 +51,10 @@ class TableMatrix {
     private SizeInfo[] rowSizes;
     private int tableWidth;
     private int tableHeight;
+    
+    /* This is so that we can draw the lines inside the table
+     * that appear when a border attribute is used.
+     */
     private int hasOldStyleBorder;
 
     /**
@@ -106,6 +110,7 @@ class TableMatrix {
 		ROWS.clear();
 		ALL_CELLS.clear();
 		ROW_ELEMENTS.clear();
+		//TODO: Does it need this old-style border?
 		String borderText = this.tableElement.getAttribute("border");
 		int border = 0;
 		if(borderText != null) {
@@ -160,10 +165,10 @@ class TableMatrix {
 		this.hasOldStyleBorder = border > 0 ? 1 : 0;
 	}
 	
-	public void build(int availWidth, int availHeight) {
+	public void build(int availWidth, int availHeight, boolean sizeOnly) {
 		int hasBorder = this.hasOldStyleBorder;
 		this.determineColumnSizes(hasBorder, this.cellSpacingX, this.cellSpacingY, availWidth);
-		this.determineRowSizes(hasBorder, this.cellSpacingY, availHeight);
+		this.determineRowSizes(hasBorder, this.cellSpacingY, availHeight, sizeOnly);
 	}
 
 	private final HTMLTableRowElementImpl getParentRow(HTMLTableCellElementImpl cellNode) {
@@ -184,7 +189,9 @@ class TableMatrix {
 			AbstractCSS2Properties props = element.getCurrentStyle();
 			String widthText = props == null ? null : props.getWidth();
 			if(widthText == null) {
-				return new HtmlLength(element.getAttribute("width"));
+			    String widthAttr=element.getAttribute("width");
+			    if (widthAttr==null) return null;
+				return new HtmlLength(widthAttr);
 			} 
 			else {
 				return new HtmlLength(HtmlValues.getPixelSize(widthText, element.getRenderState(), 0, availWidth));
@@ -199,7 +206,13 @@ class TableMatrix {
 			AbstractCSS2Properties props = element.getCurrentStyle();
 			String heightText = props == null ? null : props.getHeight();
 			if(heightText == null) {
-				return new HtmlLength(element.getAttribute("height"));
+			    String ha = element.getAttribute("height");
+			    if(ha == null) {
+			        return null;
+			    }
+			    else {
+			        return new HtmlLength(ha);
+			    }
 			} 
 			else {
 				return new HtmlLength(HtmlValues.getPixelSize(heightText, element.getRenderState(), 0, availHeight));
@@ -214,17 +227,6 @@ class TableMatrix {
 	 */
 	private void populateRows() {
 		HTMLElementImpl te = this.tableElement;
-		String cellPaddingText = te.getAttribute("cellpadding");
-		int cellPadding = 1;
-		if(cellPaddingText != null) {
-			try {
-				cellPadding = Integer.parseInt(cellPaddingText);
-				//TODO: Both cellpadding and cellspacing take percentages.
-				//TODO: Set cellpadding during layout, not in reset.
-			} catch(NumberFormatException nfe) {
-				// ignore
-			}
-		}
 		ArrayList rows = this.ROWS;
 		ArrayList rowElements = this.ROW_ELEMENTS;
 		ArrayList allCells = this.ALL_CELLS;
@@ -235,6 +237,10 @@ class TableMatrix {
 		while(ci.hasNext()) {
 			HTMLTableCellElementImpl columnNode = (HTMLTableCellElementImpl) ci.next();
 			HTMLTableRowElementImpl rowElement = this.getParentRow(columnNode);
+			if(rowElement != null && rowElement.getRenderState().getDisplay() == RenderState.DISPLAY_NONE) {
+				// Skip row [ 2047122 ] 
+				continue;
+			}
 			ArrayList row;
 			if(rowElement != null) {
 				currentNullRow = null;
@@ -268,7 +274,6 @@ class TableMatrix {
 				ac.setParent(this.relement);
 				columnNode.setUINode(ac);
 			}
-			ac.setCellPadding(cellPadding);
 			VirtualCell vc = new VirtualCell(ac, true);
 			ac.setTopLeftVirtualCell(vc);
 			row.add(vc);
@@ -300,10 +305,11 @@ class TableMatrix {
 						rowspan = 1;
 					}
 					
-					// Make sure row exists
+					// Can't go beyond last row (Fix bug #2022584)
 					int targetRows = r + rowspan;
-					while(rows.size() < targetRows) {
-						rows.add(new ArrayList());
+					if(numRows < targetRows) {
+						rowspan = numRows - r;
+						ac.setRowSpan(rowspan);
 					}
 					
 					numRows = rows.size();
@@ -681,7 +687,7 @@ class TableMatrix {
 							vcActualWidth += columnSizes[firstCol + x].actualSize;
 						}
 						//TODO: better height possible
-						Dimension size = ac.doCellLayout(vcActualWidth, 0, false);
+						Dimension size = ac.doCellLayout(vcActualWidth, 0, true, true, true);
 						int vcRenderWidth = size.width;
 
 						int denominator = (vcActualWidth - cellExtras);
@@ -705,7 +711,7 @@ class TableMatrix {
 					}
 					else {
 						//TODO: better height possible
-						Dimension size = ac.doCellLayout(actualSize, 0, false);
+						Dimension size = ac.doCellLayout(actualSize, 0, true, true, true);
 						if(size.width > colSize.layoutSize) {
 							colSize.layoutSize = size.width;
 						}
@@ -923,7 +929,7 @@ class TableMatrix {
 				if(!prevOverrideNoWrap) {
 					state.overrideNoWrap = !widthDeclared;
 				}
-				size = cell.doCellLayout(cellsTotalWidth, 0, false);
+				size = cell.doCellLayout(cellsTotalWidth, 0, true, true, true);
 			} finally {
 				state.overrideNoWrap = prevOverrideNoWrap;
 			}
@@ -977,7 +983,7 @@ class TableMatrix {
 		}
 	}
 
-	private void determineRowSizes(int hasBorder, int cellSpacing, int availHeight) {
+	private void determineRowSizes(int hasBorder, int cellSpacing, int availHeight, boolean sizeOnly) {
 		HtmlLength tableHeightLength = TableMatrix.getHeightLength(this.tableElement, availHeight);
 		int tableHeight;
 		SizeInfo[] rowSizes = this.rowSizes;
@@ -985,18 +991,18 @@ class TableMatrix {
 		int heightsOfExtras = this.heightsOfExtras;
 		if(tableHeightLength != null) {
 			tableHeight = tableHeightLength.getLength(availHeight);
-			this.determineRowSizesFixedTH(hasBorder, cellSpacing, availHeight, tableHeight);
+			this.determineRowSizesFixedTH(hasBorder, cellSpacing, availHeight, tableHeight, sizeOnly);
 		}
 		else {
 			tableHeight = heightsOfExtras;
 			for(int row = 0; row < numRows; row++) {
 				tableHeight += rowSizes[row].minSize;
 			}
-			this.determineRowSizesFlexibleTH(hasBorder, cellSpacing, availHeight);
+			this.determineRowSizesFlexibleTH(hasBorder, cellSpacing, availHeight, sizeOnly);
 		}
 	}
 	
-	private void determineRowSizesFixedTH(int hasBorder, int cellSpacing, int availHeight, int tableHeight) {
+	private void determineRowSizesFixedTH(int hasBorder, int cellSpacing, int availHeight, int tableHeight, boolean sizeOnly) {
 		SizeInfo[] rowSizes = this.rowSizes;
 		int numRows = rowSizes.length;
 		int heightsOfExtras = this.heightsOfExtras;
@@ -1127,10 +1133,10 @@ class TableMatrix {
 		// by setting a single height of subcell. Verify that IE only sets height
 		// of subcells when height of row or table are specified.
 		
-		this.finalRender(hasBorder, cellSpacing);
+		this.finalRender(hasBorder, cellSpacing, sizeOnly);
 	}
 
-	private void determineRowSizesFlexibleTH(int hasBorder, int cellSpacing, int availHeight) {
+	private void determineRowSizesFlexibleTH(int hasBorder, int cellSpacing, int availHeight, boolean sizeOnly) {
 		SizeInfo[] rowSizes = this.rowSizes;
 		int numRows = rowSizes.length;
 		int heightsOfExtras = this.heightsOfExtras;
@@ -1191,14 +1197,14 @@ class TableMatrix {
 		this.tableHeight = heightUsedByAbsolute + heightUsedByNoSize + heightUsedByPercent + heightsOfExtras;
 		
 		// Do a final render to set actual cell sizes		
-		this.finalRender(hasBorder, cellSpacing);
+		this.finalRender(hasBorder, cellSpacing, sizeOnly);
 	}
 
 	/**
 	 * This method renders each cell using already set actual column widths.
 	 * It sets minimum row heights based on this.
 	 */
-	private final void finalRender(int hasBorder, int cellSpacing) {
+	private final void finalRender(int hasBorder, int cellSpacing, boolean sizeOnly) {
 		// finalRender needs to adjust actualSize of columns and rows
 		// given that things might change as we render one last time.
 		ArrayList allCells = this.ALL_CELLS;
@@ -1231,7 +1237,7 @@ class TableMatrix {
 			else {
 				totalCellHeight = rowSizes[row].actualSize;
 			}
-			Dimension size = cell.doCellLayout(totalCellWidth, totalCellHeight, true);
+			Dimension size = cell.doCellLayout(totalCellWidth, totalCellHeight, true, true, sizeOnly); 
 			if(size.width > totalCellWidth) {
 				if(colSpan == 1) {
 					colSizes[col].actualSize = size.width;
@@ -1251,6 +1257,63 @@ class TableMatrix {
 		}
 	}
 
+//	public final void adjust() {
+//	    // finalRender needs to adjust actualSize of columns and rows
+//	    // given that things might change as we render one last time.
+//	    int hasBorder = this.hasOldStyleBorder;
+//	    int cellSpacingX = this.cellSpacingX;
+//	    int cellSpacingY = this.cellSpacingY;
+//	    ArrayList allCells = this.ALL_CELLS;
+//	    SizeInfo[] colSizes = this.columnSizes;
+//	    SizeInfo[] rowSizes = this.rowSizes;
+//	    int numCells = allCells.size();
+//	    for(int i = 0; i < numCells; i++) {
+//	        RTableCell cell = (RTableCell) allCells.get(i);
+//	        int col = cell.getVirtualColumn();
+//	        int colSpan = cell.getColSpan();
+//	        int totalCellWidth;
+//	        if(colSpan > 1) {
+//	            totalCellWidth = (colSpan - 1) * (cellSpacingX + 2 * hasBorder);
+//	            for(int x = 0; x < colSpan; x++) {
+//	                totalCellWidth += colSizes[col + x].actualSize;
+//	            }
+//	        }
+//	        else {
+//	            totalCellWidth = colSizes[col].actualSize;
+//	        }
+//	        int row = cell.getVirtualRow();
+//	        int rowSpan = cell.getRowSpan();
+//	        int totalCellHeight;
+//	        if(rowSpan > 1) {
+//	            totalCellHeight = (rowSpan - 1) * (cellSpacingY + 2 * hasBorder);
+//	            for(int y = 0; y < rowSpan; y++) {
+//	                totalCellHeight += rowSizes[row + y].actualSize;
+//	            }
+//	        }
+//	        else {
+//	            totalCellHeight = rowSizes[row].actualSize;
+//	        }
+//	        cell.adjust();
+//	        Dimension size = cell.getSize();
+//	        if(size.width > totalCellWidth) {
+//	            if(colSpan == 1) {
+//	                colSizes[col].actualSize = size.width;
+//	            }
+//	            else {
+//	                colSizes[col].actualSize += (size.width - totalCellWidth);
+//	            }
+//	        }
+//	        if(size.height > totalCellHeight) {
+//	            if(rowSpan == 1) {
+//	                rowSizes[row].actualSize = size.height;
+//	            }
+//	            else {
+//	                rowSizes[row].actualSize += (size.height - totalCellHeight);
+//	            }
+//	        }           
+//	    }
+//	}
+
 	/**
 	 * Sets bounds of each cell's component, and sumps up table width
 	 * and height.
