@@ -149,7 +149,7 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 	public static final String TEXT_TRANSFORM = "text-transform";
 	public static final String TOP = "top";
 	public static final String UNICODE_BIDI = "unicode-bidi";
-	public static final String VERTICAL_ALIGN = "vertical_align";
+	public static final String VERTICAL_ALIGN = "vertical-align";
 	public static final String VISIBILITY = "visibility";
 	public static final String VOICE_FAMILY = "voice-family";
 	public static final String VOLUME = "volume";
@@ -226,6 +226,10 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 //		return this.getPropertyValueLC(lowerCase);
 //	}
 
+	public final String getPropertyValue(String name) {
+	    return this.getPropertyValueLC(name.toLowerCase());
+	}
+
 	private final String getPropertyValueLC(String lowerCaseName) {
 		Map vm = this.valueMap;
 		synchronized(this) {
@@ -703,23 +707,19 @@ public abstract class AbstractCSS2Properties extends AbstractScriptableDelegate
 		return this.getPropertyValueLC(OUTLINE_WIDTH);
 	}
 
-	public String getOverflow() {
-		
+	public String getOverflow() {		
 		return this.getPropertyValueLC(OVERFLOW);
 	}
 
-	public String getPadding() {
-		
+	public String getPadding() {		
 		return this.getPropertyValueLC(PADDING);
 	}
 
-	public String getPaddingBottom() {
-		
+	public String getPaddingBottom() {		
 		return this.getPropertyValueLC(PADDING_BOTTOM);
 	}
 
 	public String getPaddingLeft() {
-		
 		return this.getPropertyValueLC(PADDING_LEFT);
 	}
 
