@@ -16,7 +16,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 package org.lobobrowser.html.style;
 
@@ -27,5 +27,6 @@ public interface CSS2PropertiesContext {
 	public void informPositionInvalid();
 	public void informLayoutInvalid();
 	public void informInvalid();
-	public CSS2PropertiesImpl getParentStyle();
+	public AbstractCSS2Properties getParentStyle();
+	public String getDocumentBaseURI();
 }
