@@ -32,4 +32,10 @@ public class ComputedCSS2Properties extends AbstractCSS2Properties {
 	protected void checkSetProperty() {
 		throw new java.lang.IllegalAccessError("Style properties cannot be set in this instance.");
 	}
+	
+	public void internalSetLC(String lowerCaseName, String value) {
+	    // Should only be called right after creation of the CSS object.
+	    // Properties need to be "unimportant" otherwise they won't get overridden.
+	    super.setPropertyValueLCAlt(lowerCaseName, value, false);
+	}
 }
