@@ -16,7 +16,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 package org.lobobrowser.html.style;
 
@@ -25,5 +25,5 @@ class FontInfo {
 	public String fontStyle;
 	public String fontVariant;
 	public String fontWeight;
-	public Float fontSize;
+	public String fontSize;
 }
\ No newline at end of file
