@@ -24,17 +24,32 @@ package org.lobobrowser.html.style;
 import java.awt.Font;
 import java.awt.FontMetrics;
 import java.awt.Toolkit;
+import java.awt.font.TextAttribute;
 import java.util.HashMap;
 import java.util.Map;
 
+import org.lobobrowser.util.gui.FontFactory;
+
 public class FontStyleRenderState extends RenderStateDelegator {
 	private final int style;
+	private final Integer superscript;
 	
 	public FontStyleRenderState(RenderState prevRenderState, int style) {
-		super(prevRenderState);
-		this.style = style;
+		this(prevRenderState, style, null);
 	}
 
+	FontStyleRenderState(RenderState prevRenderState, int style, Integer superscript) {
+		super(prevRenderState);
+		this.style = style;
+		this.superscript = superscript;
+	}
+
+	public static FontStyleRenderState createSuperscriptFontStyleRenderState (RenderState prevRenderState,
+			Integer superscript){
+		return new FontStyleRenderState(prevRenderState, prevRenderState.getFont().getStyle(), superscript); 
+	}
+
+	
 	private Font iFont;
 	
 	public Font getFont() {
@@ -43,7 +58,13 @@ public class FontStyleRenderState extends RenderStateDelegator {
 			return f;
 		}
 		Font parentFont = this.delegate.getFont();
+		if (parentFont.getStyle() != this.style){
 		f = parentFont.deriveFont(this.style | parentFont.getStyle());
+		} else {
+			f = parentFont;
+		}
+		f = FontFactory.superscriptFont(f, superscript);
+		
 		this.iFont = f;
 		return f;		
 	}
