@@ -44,8 +44,8 @@ public class HtmlValues {
 	private static final Logger logger = Logger.getLogger(HtmlValues.class
 			.getName());
 	public static final float DEFAULT_FONT_SIZE = 14.0f;
-	public static final int DEFAULT_FONT_SIZE_INT = 14;
-	public static final Float DEFAULT_FONT_SIZE_BOX = new Float(14.0f);
+	public static final int DEFAULT_FONT_SIZE_INT = (int) DEFAULT_FONT_SIZE;
+	public static final Float DEFAULT_FONT_SIZE_BOX = new Float(DEFAULT_FONT_SIZE);
 	public static final int DEFAULT_BORDER_WIDTH = 2;
 
 	public static final int BORDER_STYLE_NONE = 0;
@@ -81,9 +81,9 @@ public class HtmlValues {
 				|| tokenTL.equals("inset") || tokenTL.equals("outset");
 	}
 
-	public static Insets getMarginInsets(CSS2Properties cssProperties,
+	public static HtmlInsets getMarginInsets(CSS2Properties cssProperties,
 			RenderState renderState) {
-		Insets insets = null;
+	    HtmlInsets insets = null;
 		String topText = cssProperties.getMarginTop();
 		insets = updateTopInset(insets, topText, renderState);
 		String leftText = cssProperties.getMarginLeft();
@@ -95,9 +95,9 @@ public class HtmlValues {
 		return insets;
 	}
 
-	public static Insets getPaddingInsets(CSS2Properties cssProperties,
+	public static HtmlInsets getPaddingInsets(CSS2Properties cssProperties,
 			RenderState renderState) {
-		Insets insets = null;
+		HtmlInsets insets = null;
 		String topText = cssProperties.getPaddingTop();
 		insets = updateTopInset(insets, topText, renderState);
 		String leftText = cssProperties.getPaddingLeft();
@@ -109,17 +109,54 @@ public class HtmlValues {
 		return insets;
 	}
 
-	public static Insets getBorderInsets(Insets borderStyles,
+	public static HtmlInsets getBorderInsets(Insets borderStyles,
 			CSS2Properties cssProperties, RenderState renderState) {
-		int topWidth = getBorderWidth(cssProperties.getBorderTopWidth(),
-				borderStyles.top, renderState);
-		int leftWidth = getBorderWidth(cssProperties.getBorderLeftWidth(),
-				borderStyles.left, renderState);
-		int bottomWidth = getBorderWidth(cssProperties.getBorderBottomWidth(),
-				borderStyles.bottom, renderState);
-		int rightWidth = getBorderWidth(cssProperties.getBorderRightWidth(),
-				borderStyles.right, renderState);
-		return new Insets(topWidth, leftWidth, bottomWidth, rightWidth);
+        HtmlInsets insets = null;
+        if(borderStyles.top != HtmlValues.BORDER_STYLE_NONE) {
+            String topText = cssProperties.getBorderTopWidth();
+            insets = updateTopInset(insets, topText, renderState);
+        }
+        if(borderStyles.left != HtmlValues.BORDER_STYLE_NONE) {
+            String leftText = cssProperties.getBorderLeftWidth();
+            insets = updateLeftInset(insets, leftText, renderState);
+        }
+        if(borderStyles.bottom != HtmlValues.BORDER_STYLE_NONE) {
+            String bottomText = cssProperties.getBorderBottomWidth();
+            insets = updateBottomInset(insets, bottomText, renderState);
+        }
+        if(borderStyles.right != HtmlValues.BORDER_STYLE_NONE) {
+            String rightText = cssProperties.getBorderRightWidth();
+            insets = updateRightInset(insets, rightText, renderState);
+        }
+        return insets;
+	}
+
+	/**
+	 * Populates {@link BorderInfo.insets}.
+	 * @param binfo A BorderInfo with its styles already populated.
+	 * @param cssProperties The CSS properties object.
+	 * @param renderState The current render state.
+	 */
+	public static void populateBorderInsets(BorderInfo binfo,
+	        CSS2Properties cssProperties, RenderState renderState) {
+	    HtmlInsets insets = null;
+	    if(binfo.topStyle != HtmlValues.BORDER_STYLE_NONE) {
+	        String topText = cssProperties.getBorderTopWidth();
+	        insets = updateTopInset(insets, topText, renderState);
+	    }
+	    if(binfo.leftStyle != HtmlValues.BORDER_STYLE_NONE) {
+	        String leftText = cssProperties.getBorderLeftWidth();
+	        insets = updateLeftInset(insets, leftText, renderState);
+	    }
+	    if(binfo.bottomStyle != HtmlValues.BORDER_STYLE_NONE) {
+	        String bottomText = cssProperties.getBorderBottomWidth();
+	        insets = updateBottomInset(insets, bottomText, renderState);
+	    }
+	    if(binfo.rightStyle != HtmlValues.BORDER_STYLE_NONE) {
+	        String rightText = cssProperties.getBorderRightWidth();
+	        insets = updateRightInset(insets, rightText, renderState);
+	    }
+	    binfo.insets = insets;
 	}
 
 	private static int getBorderWidth(String sizeText, int borderStyle,
@@ -135,52 +172,124 @@ public class HtmlValues {
 		}
 	}
 
-	private static Insets updateTopInset(Insets insets, String sizeText,
+	private static HtmlInsets updateTopInset(HtmlInsets insets, String sizeText,
 			RenderState renderState) {
-		if (sizeText == null || sizeText.length() == 0) {
-			return insets;
+		if (sizeText == null) {
+		    return insets;
 		}
-		if (insets == null) {
-			insets = new Insets(0, 0, 0, 0);
+		sizeText = sizeText.trim();
+		if(sizeText.length() == 0) {
+		    return insets;
+		}
+		if(insets == null) {
+		    insets = new HtmlInsets();
+		}
+		if("auto".equalsIgnoreCase(sizeText)) {
+		    insets.topType = HtmlInsets.TYPE_AUTO;
+		}
+		else if(sizeText.endsWith("%")) {
+		    insets.topType = HtmlInsets.TYPE_PERCENT;
+		    try {
+		        insets.top = Integer.parseInt(sizeText.substring(0, sizeText.length() - 1));
+		    } catch(NumberFormatException nfe) {
+		        insets.top = 0;
+		    }
+		}
+		else {
+		    insets.topType = HtmlInsets.TYPE_PIXELS;
+		    insets.top = HtmlValues.getPixelSize(sizeText, renderState, 0);
 		}
-		insets.top = getPixelSize(sizeText, renderState, 0);
 		return insets;
 	}
 
-	private static Insets updateLeftInset(Insets insets, String sizeText,
+	private static HtmlInsets updateLeftInset(HtmlInsets insets, String sizeText,
 			RenderState renderState) {
-		if (sizeText == null || sizeText.length() == 0) {
-			return insets;
-		}
-		if (insets == null) {
-			insets = new Insets(0, 0, 0, 0);
-		}
-		insets.left = getPixelSize(sizeText, renderState, 0);
-		return insets;
+        if (sizeText == null) {
+            return insets;
+        }
+        sizeText = sizeText.trim();
+        if(sizeText.length() == 0) {
+            return insets;
+        }
+        if(insets == null) {
+            insets = new HtmlInsets();
+        }
+        if("auto".equalsIgnoreCase(sizeText)) {
+            insets.leftType = HtmlInsets.TYPE_AUTO;
+        }
+        else if(sizeText.endsWith("%")) {
+            insets.leftType = HtmlInsets.TYPE_PERCENT;
+            try {
+                insets.left = Integer.parseInt(sizeText.substring(0, sizeText.length() - 1));
+            } catch(NumberFormatException nfe) {
+                insets.left = 0;
+            }
+        }
+        else {
+            insets.leftType = HtmlInsets.TYPE_PIXELS;
+            insets.left = HtmlValues.getPixelSize(sizeText, renderState, 0);
+        }
+        return insets;
 	}
 
-	private static Insets updateBottomInset(Insets insets, String sizeText,
+	private static HtmlInsets updateBottomInset(HtmlInsets insets, String sizeText,
 			RenderState renderState) {
-		if (sizeText == null || sizeText.length() == 0) {
-			return insets;
-		}
-		if (insets == null) {
-			insets = new Insets(0, 0, 0, 0);
-		}
-		insets.bottom = getPixelSize(sizeText, renderState, 0);
-		return insets;
+        if (sizeText == null) {
+            return insets;
+        }
+        sizeText = sizeText.trim();
+        if(sizeText.length() == 0) {
+            return insets;
+        }
+        if(insets == null) {
+            insets = new HtmlInsets();
+        }
+        if("auto".equalsIgnoreCase(sizeText)) {
+            insets.bottomType = HtmlInsets.TYPE_AUTO;
+        }
+        else if(sizeText.endsWith("%")) {
+            insets.bottomType = HtmlInsets.TYPE_PERCENT;
+            try {
+                insets.bottom = Integer.parseInt(sizeText.substring(0, sizeText.length() - 1));
+            } catch(NumberFormatException nfe) {
+                insets.bottom = 0;
+            }
+        }
+        else {
+            insets.bottomType = HtmlInsets.TYPE_PIXELS;
+            insets.bottom = HtmlValues.getPixelSize(sizeText, renderState, 0);
+        }
+        return insets;
 	}
 
-	private static Insets updateRightInset(Insets insets, String sizeText,
+	private static HtmlInsets updateRightInset(HtmlInsets insets, String sizeText,
 			RenderState renderState) {
-		if (sizeText == null || sizeText.length() == 0) {
-			return insets;
-		}
-		if (insets == null) {
-			insets = new Insets(0, 0, 0, 0);
-		}
-		insets.right = getPixelSize(sizeText, renderState, 0);
-		return insets;
+        if (sizeText == null) {
+            return insets;
+        }
+        sizeText = sizeText.trim();
+        if(sizeText.length() == 0) {
+            return insets;
+        }
+        if(insets == null) {
+            insets = new HtmlInsets();
+        }
+        if("auto".equalsIgnoreCase(sizeText)) {
+            insets.rightType = HtmlInsets.TYPE_AUTO;
+        }
+        else if(sizeText.endsWith("%")) {
+            insets.rightType = HtmlInsets.TYPE_PERCENT;
+            try {
+                insets.right = Integer.parseInt(sizeText.substring(0, sizeText.length() - 1));
+            } catch(NumberFormatException nfe) {
+                insets.right = 0;
+            }
+        }
+        else {
+            insets.rightType = HtmlInsets.TYPE_PIXELS;
+            insets.right = HtmlValues.getPixelSize(sizeText, renderState, 0);
+        }
+        return insets;
 	}
 
 	public static Insets getInsets(String insetsSpec, RenderState renderState,
@@ -246,44 +355,6 @@ public class HtmlValues {
 		}
 	}
 
-	// /**
-	// * Gets an array of top-left-bottom-right colors
-	// * given a spec.
-	// */
-	// public static Color[] getColors(String colorsSpec) {
-	// Color[] colorsArray = new Color[4];
-	// String[] colorStrings = HtmlValues.splitCssValue(colorsSpec);
-	// int size = colorStrings.length;
-	// if(size > 4) {
-	// size = 4;
-	// }
-	// for(int i = 0; i < size; i++) {
-	// colorsArray[i] = ColorFactory.getInstance().getColor(colorStrings[i]);
-	// }
-	// if(size == 4) {
-	// return colorsArray;
-	// }
-	// else if (size == 1) {
-	// Color color = colorsArray[0];
-	// colorsArray[1] = color;
-	// colorsArray[2] = color;
-	// colorsArray[3] = color;
-	// return colorsArray;
-	// }
-	// else if(size == 2) {
-	// colorsArray[2] = colorsArray[0];
-	// colorsArray[3] = colorsArray[1];
-	// return colorsArray;
-	// }
-	// else if(size == 3) {
-	// colorsArray[3] = colorsArray[1];
-	// return colorsArray;
-	// }
-	// else {
-	// throw new IllegalStateException("size=" + size);
-	// }
-	// }
-
 	/**
 	 * Gets a number for 1 to 7.
 	 * 
@@ -337,17 +408,41 @@ public class HtmlValues {
 		}
 	}
 
+	public static final String getFontSizeSpec(int fontNumber) {
+	    switch (fontNumber) {
+	    case 1:
+	        return "10px";
+	    case 2:
+	        return "11px";
+	    case 3:
+	        return "13px";
+	    case 4:
+	        return "16px";
+	    case 5:
+	        return "21px";
+	    case 6:
+	        return "29px";
+	    case 7:
+	        return "42px";
+	    default:
+	        return "63px";
+	    }
+	}
+
 	public static final float getFontSize(String spec,
 			RenderState parentRenderState) {
 		String specTL = spec.toLowerCase();
 		if(specTL.endsWith("em")) {
+		    if(parentRenderState == null) {
+		        return DEFAULT_FONT_SIZE;
+		    }
 			Font font = parentRenderState.getFont();
 			String pxText = specTL.substring(0, specTL.length() - 2);
 			double value;
 			try {
 				value = Double.parseDouble(pxText);
 			} catch (NumberFormatException nfe) {
-				return DEFAULT_FONT_SIZE_INT;
+				return DEFAULT_FONT_SIZE;
 			}
 			return (int) Math.round(font.getSize() * value);
 		}
@@ -463,7 +558,7 @@ public class HtmlValues {
 			} catch (NumberFormatException nfe) {
 				return errorValue;
 			}
-			int dpi = Toolkit.getDefaultToolkit().getScreenResolution();
+			int dpi = GraphicsEnvironment.isHeadless() ? 72 : Toolkit.getDefaultToolkit().getScreenResolution();
 			double inches = val / 6;
 			return (int) Math.round(dpi * inches);
 		} else if (lcSpec.endsWith("cm")) {
@@ -474,7 +569,7 @@ public class HtmlValues {
 			} catch (NumberFormatException nfe) {
 				return errorValue;
 			}
-			int dpi = Toolkit.getDefaultToolkit().getScreenResolution();
+			int dpi = GraphicsEnvironment.isHeadless() ? 72 : Toolkit.getDefaultToolkit().getScreenResolution();
 			double inches = val / 2.54;
 			return (int) Math.round(dpi * inches);
 		} else if (lcSpec.endsWith("mm")) {
@@ -485,7 +580,7 @@ public class HtmlValues {
 			} catch (NumberFormatException nfe) {
 				return errorValue;
 			}
-			int dpi = Toolkit.getDefaultToolkit().getScreenResolution();
+			int dpi = GraphicsEnvironment.isHeadless() ? 72 : Toolkit.getDefaultToolkit().getScreenResolution();
 			double inches = val / 25.4;
 			return (int) Math.round(dpi * inches);
 		} else if (lcSpec.endsWith("ex") && renderState != null) {
@@ -529,6 +624,18 @@ public class HtmlValues {
 		}
 	}
 
+	public static int getOldSyntaxPixelSizeSimple(String spec, int errorValue) {
+	    if (spec == null) {
+	        return errorValue;
+	    }
+	    spec = spec.trim();
+	    try {
+	        return Integer.parseInt(spec);
+	    } catch (NumberFormatException nfe) {
+	        return errorValue;
+	    }
+	}
+
 	public static java.net.URL getURIFromStyleValue(String fullURLStyleValue) {
 		String start = "url(";
 		if (!fullURLStyleValue.toLowerCase().startsWith(start)) {
@@ -713,7 +820,10 @@ public class HtmlValues {
 
 	public static int getListStyleType(String token) {
 		String tokenTL = token.toLowerCase();
-		if ("disc".equals(tokenTL)) {
+		if("none".equals(tokenTL)) {
+			return ListStyle.TYPE_NONE;
+		}
+		else if ("disc".equals(tokenTL)) {
 			return ListStyle.TYPE_DISC;
 		} else if ("circle".equals(tokenTL)) {
 			return ListStyle.TYPE_CIRCLE;
@@ -806,77 +916,37 @@ public class HtmlValues {
 		}
 	}
 
-	// public static FontInfo createFontInfo(String fontSpec, RenderState
-	// parentRenderState) {
-	// String fontSpecTL = fontSpec.toLowerCase();
-	// FontInfo fontInfo = (FontInfo) SYSTEM_FONTS.get(fontSpecTL);
-	// if(fontInfo != null) {
-	// return fontInfo;
-	// }
-	// fontInfo = new FontInfo();
-	// String[] tokens = HtmlValues.splitCssValue(fontSpecTL);
-	// String token = null;
-	// int length = tokens.length;
-	// int i;
-	// for(i = 0; i < length; i++) {
-	// token = tokens[i];
-	// if(isFontStyle(token)) {
-	// fontInfo.fontStyle = token;
-	// continue;
-	// }
-	// if(isFontVariant(token)) {
-	// fontInfo.fontVariant = token;
-	// continue;
-	// }
-	// if(isFontWeight(token)) {
-	// fontInfo.fontWeight = token;
-	// continue;
-	// }
-	// // Otherwise exit loop
-	// break;
-	// }
-	// if(token != null) {
-	// int slashIdx = token.indexOf('/');
-	// String fontSizeText = slashIdx == -1 ? token : token.substring(0,
-	// slashIdx);
-	// fontInfo.fontSize = new
-	// java.lang.Float(HtmlValues.getFontSize(fontSizeText, parentRenderState));
-	// //TODO: Line height
-	// if(++i < length) {
-	// StringBuffer fontFamilyBuff = new StringBuffer();
-	// do {
-	// token = tokens[i];
-	// fontFamilyBuff.append(token);
-	// fontFamilyBuff.append(' ');
-	// } while(++i < length);
-	// fontInfo.fontFamily = fontFamilyBuff.toString();
-	// }
-	// }
-	// return fontInfo;
-	// }
-
-	// if(token != null) {
-	// int slashIdx = token.indexOf('/');
-	// String fontSizeText = slashIdx == -1 ? token : token.substring(0,
-	// slashIdx);
-	// try {
-	// fontSizeText = fontSizeText.toLowerCase();
-	// todo();
-	// } catch(Exception err) {
-	// // ignore
-	// }
-	//		
-	// // The rest is the font family
-	// if(hasMoreTokens) {
-	// StringBuffer fontFamilyBuf = new StringBuffer();
-	// while(tok.hasMoreTokens()) {
-	// fontFamilyBuf.append(tok.nextToken());
-	// fontFamilyBuf.append(' ');
-	// }
-	// fontInfo.fontFamily = fontFamilyBuf.toString();
-	// }
-	// }
-
+	public static BorderInfo getBorderInfo(CSS2Properties properties, RenderState renderState) {
+	    BorderInfo binfo = new BorderInfo();
+	    
+        binfo.topStyle = getBorderStyle(properties.getBorderTopStyle());
+        binfo.rightStyle = getBorderStyle(properties.getBorderRightStyle());
+        binfo.bottomStyle = getBorderStyle(properties.getBorderBottomStyle());
+        binfo.leftStyle = getBorderStyle(properties.getBorderLeftStyle());
+        
+        ColorFactory cf = ColorFactory.getInstance();
+        String topColorSpec = properties.getBorderTopColor();
+        if(topColorSpec != null) {
+            binfo.topColor = cf.getColor(topColorSpec);
+        }
+        String rightColorSpec = properties.getBorderRightColor();
+        if(rightColorSpec != null) {
+            binfo.rightColor = cf.getColor(rightColorSpec);
+        }
+        String bottomColorSpec = properties.getBorderBottomColor();
+        if(bottomColorSpec != null) {
+            binfo.bottomColor = cf.getColor(bottomColorSpec);
+        }
+        String leftColorSpec = properties.getBorderLeftColor();
+        if(leftColorSpec != null) {
+            binfo.leftColor = cf.getColor(leftColorSpec);
+        }
+        
+        HtmlValues.populateBorderInsets(binfo, properties, renderState);
+        
+        return binfo;
+	}	
+	
 	public static Insets getBorderStyles(CSS2Properties properties) {
 		int topStyle = getBorderStyle(properties.getBorderTopStyle());
 		int rightStyle = getBorderStyle(properties.getBorderRightStyle());
