@@ -81,12 +81,27 @@ public class StyleSheetAggregator {
 				StringTokenizer tok = new StringTokenizer(selectorPart, " \t\r\n");
 				if(tok.hasMoreTokens()) {
 					simpleSelectors = new ArrayList();
+					SimpleSelector prevSelector = null;
+					SELECTOR_FOR:
 					for(;;) {
 						String token = tok.nextToken();
+						if(">".equals(token)) {
+							if(prevSelector != null) {
+								prevSelector.setSelectorType(SimpleSelector.PARENT);
+							}
+							continue SELECTOR_FOR;
+						}
+						else if("+".equals(token)) {
+							if(prevSelector != null) {
+								prevSelector.setSelectorType(SimpleSelector.PRECEEDING_SIBLING);
+							}
+							continue SELECTOR_FOR;
+						}
 						int colonIdx = token.indexOf(':');
 						String simpleSelectorText = colonIdx == -1 ? token : token.substring(0, colonIdx);
-						String pseudoElement = colonIdx == -1 ? null : token.substring(colonIdx+1);					
-						simpleSelectors.add(new SimpleSelector(simpleSelectorText, pseudoElement));
+						String pseudoElement = colonIdx == -1 ? null : token.substring(colonIdx+1);
+						prevSelector = new SimpleSelector(simpleSelectorText, pseudoElement);
+						simpleSelectors.add(prevSelector);
 						if(!tok.hasMoreTokens()) {
 							lastSelectorText = simpleSelectorText;
 							break;
@@ -114,7 +129,7 @@ public class StyleSheetAggregator {
 					}
 				}
 			}
-			//TODO: Other types of selectors
+			//TODO: Attribute selectors
 		}
 		else if(rule instanceof CSSImportRule) {
 			UserAgentContext uacontext = document.getUserAgentContext();
@@ -182,7 +197,7 @@ public class StyleSheetAggregator {
 		rules.add(new StyleRuleInfo(ancestorSelectors, styleRule));
 	}
 		
-	public final Collection getStyleDeclarations(HTMLElementImpl element, String elementName, String elementId, String className, Set pseudoNames) {
+	public final Collection getActiveStyleDeclarations(HTMLElementImpl element, String elementName, String elementId, String className, Set pseudoNames) {
 		Collection styleDeclarations = null;
 		String elementTL = elementName.toLowerCase();
 		Collection elementRules = (Collection) this.rulesByElement.get(elementTL);
@@ -192,6 +207,10 @@ public class StyleSheetAggregator {
 				StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
 				if(styleRuleInfo.isSelectorMatch(element, pseudoNames)) {
 					CSSStyleRule styleRule = styleRuleInfo.styleRule;
+					CSSStyleSheet styleSheet = styleRule.getParentStyleSheet();
+					if(styleSheet != null && styleSheet.getDisabled()) {
+						continue;
+					}
 					if(styleDeclarations == null) {
 						styleDeclarations = new LinkedList();
 					}
@@ -208,6 +227,10 @@ public class StyleSheetAggregator {
 				StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
 				if(styleRuleInfo.isSelectorMatch(element, pseudoNames)) {
 					CSSStyleRule styleRule = styleRuleInfo.styleRule;
+					CSSStyleSheet styleSheet = styleRule.getParentStyleSheet();
+					if(styleSheet != null && styleSheet.getDisabled()) {
+						continue;
+					}
 					if(styleDeclarations == null) {
 						styleDeclarations = new LinkedList();
 					}
@@ -226,6 +249,10 @@ public class StyleSheetAggregator {
 						StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
 						if(styleRuleInfo.isSelectorMatch(element, pseudoNames)) {
 							CSSStyleRule styleRule = styleRuleInfo.styleRule;
+							CSSStyleSheet styleSheet = styleRule.getParentStyleSheet();
+							if(styleSheet != null && styleSheet.getDisabled()) {
+								continue;
+							}
 							if(styleDeclarations == null) {
 								styleDeclarations = new LinkedList();
 							}
@@ -243,6 +270,10 @@ public class StyleSheetAggregator {
 						StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
 						if(styleRuleInfo.isSelectorMatch(element, pseudoNames)) {
 							CSSStyleRule styleRule = styleRuleInfo.styleRule;
+							CSSStyleSheet styleSheet = styleRule.getParentStyleSheet();
+							if(styleSheet != null && styleSheet.getDisabled()) {
+								continue;
+							}
 							if(styleDeclarations == null) {
 								styleDeclarations = new LinkedList();
 							}
@@ -263,6 +294,10 @@ public class StyleSheetAggregator {
 						StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
 						if(styleRuleInfo.isSelectorMatch(element, pseudoNames)) {
 							CSSStyleRule styleRule = styleRuleInfo.styleRule;
+							CSSStyleSheet styleSheet = styleRule.getParentStyleSheet();
+							if(styleSheet != null && styleSheet.getDisabled()) {
+								continue;
+							}
 							if(styleDeclarations == null) {
 								styleDeclarations = new LinkedList();
 							}
@@ -281,6 +316,10 @@ public class StyleSheetAggregator {
 						StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
 						if(styleRuleInfo.isSelectorMatch(element, pseudoNames)) {
 							CSSStyleRule styleRule = styleRuleInfo.styleRule;
+							CSSStyleSheet styleSheet = styleRule.getParentStyleSheet();
+							if(styleSheet != null && styleSheet.getDisabled()) {
+								continue;
+							}
 							if(styleDeclarations == null) {
 								styleDeclarations = new LinkedList();
 							}
@@ -300,6 +339,10 @@ public class StyleSheetAggregator {
 			Iterator i = elementRules.iterator();
 			while(i.hasNext()) {
 				StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
+				CSSStyleSheet styleSheet = styleRuleInfo.styleRule.getParentStyleSheet();
+				if(styleSheet != null && styleSheet.getDisabled()) {
+					continue;
+				}
 				if(styleRuleInfo.affectedByPseudoNameInAncestor(element, ancestor, pseudoName)) {
 					return true;
 				}
@@ -310,6 +353,10 @@ public class StyleSheetAggregator {
 			Iterator i = elementRules.iterator();
 			while(i.hasNext()) {
 				StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
+				CSSStyleSheet styleSheet = styleRuleInfo.styleRule.getParentStyleSheet();
+				if(styleSheet != null && styleSheet.getDisabled()) {
+					continue;
+				}
 				if(styleRuleInfo.affectedByPseudoNameInAncestor(element, ancestor, pseudoName)) {
 					return true;
 				}
@@ -326,6 +373,10 @@ public class StyleSheetAggregator {
 						Iterator i = classRules.iterator();
 						while(i.hasNext()) {
 							StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
+							CSSStyleSheet styleSheet = styleRuleInfo.styleRule.getParentStyleSheet();
+							if(styleSheet != null && styleSheet.getDisabled()) {
+								continue;
+							}
 							if(styleRuleInfo.affectedByPseudoNameInAncestor(element, ancestor, pseudoName)) {
 								return true;
 							}
@@ -339,6 +390,10 @@ public class StyleSheetAggregator {
 						Iterator i = classRules.iterator();
 						while(i.hasNext()) {
 							StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
+							CSSStyleSheet styleSheet = styleRuleInfo.styleRule.getParentStyleSheet();
+							if(styleSheet != null && styleSheet.getDisabled()) {
+								continue;
+							}
 							if(styleRuleInfo.affectedByPseudoNameInAncestor(element, ancestor, pseudoName)) {
 								return true;
 							}
@@ -356,6 +411,10 @@ public class StyleSheetAggregator {
 					Iterator i = idRules.iterator();
 					while(i.hasNext()) {
 						StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
+						CSSStyleSheet styleSheet = styleRuleInfo.styleRule.getParentStyleSheet();
+						if(styleSheet != null && styleSheet.getDisabled()) {
+							continue;
+						}
 						if(styleRuleInfo.affectedByPseudoNameInAncestor(element, ancestor, pseudoName)) {
 							return true;
 						}
@@ -370,6 +429,10 @@ public class StyleSheetAggregator {
 					Iterator i = idRules.iterator();
 					while(i.hasNext()) {
 						StyleRuleInfo styleRuleInfo = (StyleRuleInfo) i.next();
+						CSSStyleSheet styleSheet = styleRuleInfo.styleRule.getParentStyleSheet();
+						if(styleSheet != null && styleSheet.getDisabled()) {
+							continue;
+						}
 						if(styleRuleInfo.affectedByPseudoNameInAncestor(element, ancestor, pseudoName)) {
 							return true;
 						}
@@ -460,39 +523,78 @@ public class StyleSheetAggregator {
 				}
 				String selectorText = simpleSelector.simpleSelectorText;
 				int dotIdx = selectorText.indexOf('.');
-				HTMLElementImpl ancestor;
+				int selectorType = simpleSelector.selectorType;
+				HTMLElementImpl priorElement;
 				if(dotIdx != -1) {
 					String elemtl = selectorText.substring(0, dotIdx);
 					String classtl = selectorText.substring(dotIdx+1);
-					ancestor = currentElement.getAncestorWithClass(elemtl, classtl);
+					if(selectorType == SimpleSelector.ANCESTOR) {
+						priorElement = currentElement.getAncestorWithClass(elemtl, classtl);
+					}
+					else if(selectorType == SimpleSelector.PARENT) {
+						priorElement = currentElement.getParentWithClass(elemtl, classtl);
+					}
+					else if(selectorType == SimpleSelector.PRECEEDING_SIBLING) {
+						priorElement = currentElement.getPreceedingSiblingWithClass(elemtl, classtl); 
+					}
+					else {
+						throw new IllegalStateException("selectorType=" + selectorType);
+					}
 				}
 				else {
 					int poundIdx = selectorText.indexOf('#');
 					if(poundIdx != -1) {
 						String elemtl = selectorText.substring(0, poundIdx);
 						String idtl = selectorText.substring(poundIdx+1);
-						ancestor = currentElement.getAncestorWithId(elemtl, idtl);
+						if(selectorType == SimpleSelector.ANCESTOR) {
+							priorElement = currentElement.getAncestorWithId(elemtl, idtl);
+						}
+						else if(selectorType == SimpleSelector.PARENT) {
+							priorElement = currentElement.getParentWithId(elemtl, idtl);
+						}
+						else if(selectorType == SimpleSelector.PRECEEDING_SIBLING) {
+							priorElement = currentElement.getPreceedingSiblingWithId(elemtl, idtl); 
+						}
+						else {
+							throw new IllegalStateException("selectorType=" + selectorType);
+						}
 					}
 					else {
 						String elemtl = selectorText;
-						ancestor = currentElement.getAncestor(elemtl);
+						if(selectorType == SimpleSelector.ANCESTOR) {
+							priorElement = currentElement.getAncestor(elemtl);
+						}
+						else if(selectorType == SimpleSelector.PARENT) {
+							priorElement = currentElement.getParent(elemtl);
+						}
+						else if(selectorType == SimpleSelector.PRECEEDING_SIBLING) {
+							priorElement = currentElement.getPreceedingSibling(elemtl); 
+						}
+						else {
+							throw new IllegalStateException("selectorType=" + selectorType);
+						}
 					}
 				}				
-				if(ancestor == null) {
+				if(priorElement == null) {
 					return false;
 				}
-				if(!simpleSelector.matches(ancestor)) {
+				if(!simpleSelector.matches(priorElement)) {
 					return false;
 				}
-				currentElement = ancestor;					
+				currentElement = priorElement;					
 			}
 			return true;
 		}
 	}
 	
-	private static class SimpleSelector {
-		public String simpleSelectorText;
-		public String pseudoElement;
+	static class SimpleSelector {
+		public static final int ANCESTOR = 0;
+		public static final int PARENT = 1;
+		public static final int PRECEEDING_SIBLING = 2;
+		
+		public final String simpleSelectorText;
+		public final String pseudoElement;
+		public int selectorType;
 
 		/**
 		 * 
@@ -503,6 +605,7 @@ public class StyleSheetAggregator {
 			super();
 			this.simpleSelectorText = simpleSelectorText;
 			this.pseudoElement = pseudoElement;
+			this.selectorType = ANCESTOR;
 		}
 		
 		public final boolean matches(HTMLElementImpl element) {
@@ -539,5 +642,13 @@ public class StyleSheetAggregator {
 		public final boolean hasPseudoName(String pseudoName) {
 			return pseudoName.equals(this.pseudoElement);
 		}
+
+		public int getSelectorType() {
+			return selectorType;
+		}
+
+		public void setSelectorType(int selectorType) {
+			this.selectorType = selectorType;
+		}
 	}
 }
