@@ -23,6 +23,7 @@
  */
 package org.lobobrowser.html.style;
 import java.awt.*;
+import java.awt.font.TextAttribute;
 import java.util.*;
 
 import org.lobobrowser.html.domimpl.*;
@@ -39,9 +40,10 @@ public class StyleSheetRenderState implements RenderState {
 	// Default font needs to be something that displays in all languages.
 	// Serif, SansSerif, Monospaced.
 	private static final String DEFAULT_FONT_FAMILY = "SansSerif"; 
-	private static final Font DEFAULT_FONT = FONT_FACTORY.getFont(DEFAULT_FONT_FAMILY, null, null, null, HtmlValues.DEFAULT_FONT_SIZE, null);
-	protected static final Insets INVALID_INSETS = new Insets(0, 0, 0, 0);
+	private static final Font DEFAULT_FONT = FONT_FACTORY.getFont(DEFAULT_FONT_FAMILY, null, null, null, HtmlValues.DEFAULT_FONT_SIZE, null, null);
+	protected static final HtmlInsets INVALID_INSETS = new HtmlInsets();
 	protected static final BackgroundInfo INVALID_BACKGROUND_INFO = new BackgroundInfo();
+    protected static final BorderInfo INVALID_BORDER_INFO = new BorderInfo();
 	protected static final Color INVALID_COLOR = new Color(100, 0, 100);
 
 	protected final HTMLElementImpl element;
@@ -54,12 +56,13 @@ public class StyleSheetRenderState implements RenderState {
 	private Color iBackgroundColor = INVALID_COLOR;
 	private Color iTextBackgroundColor = INVALID_COLOR;
 	private Color iOverlayColor = INVALID_COLOR;
-	private BackgroundInfo iBackgroundInfo = INVALID_BACKGROUND_INFO;
 	private int iTextDecoration = -1;
 	private int iTextTransform = -1;
 	private int iBlankWidth = -1;
 	private boolean iHighlight;
 
+	protected BackgroundInfo iBackgroundInfo = INVALID_BACKGROUND_INFO;
+
 	static {
 	}
 	
@@ -167,58 +170,12 @@ public class StyleSheetRenderState implements RenderState {
 		this.iWhiteSpace = null;
 		this.marginInsets = INVALID_INSETS;
 		this.paddingInsets = INVALID_INSETS;
+		this.overflowX = -1;
+		this.overflowY = -1;
+		this.borderInfo = INVALID_BORDER_INFO;
 		// Should NOT invalidate parent render state.
 	}
 		
-//	private final String getLocalFont() {
-//		String value = this.getCssProperties().getFont();
-//		if(value != null && value.length() == 0) {
-//			value = null;
-//		}
-//		return value;
-//	}
-//	
-//	private final String getLocalFontFamily() {
-//		String value = this.getCssProperties().getFontFamily();
-//		if(value != null && value.length() == 0) {
-//			value = null;
-//		}
-//		return value;
-//	}
-//	
-//	private final String getLocalFontStyle() {
-//		String value = this.getCssProperties().getFontStyle();
-//		if(value != null && value.length() == 0) {
-//			value = null;
-//		}
-//		return value;
-//	}
-//	
-//	private final String getLocalFontVariant() {
-//		String value = this.getCssProperties().getFontVariant();
-//		if(value != null && value.length() == 0) {
-//			value = null;
-//		}		
-//		return value;
-//	}
-//	
-//	private final String getLocalFontWeight() {
-//		String value = this.getCssProperties().getFontWeight();
-//		if(value != null && value.length() == 0) {
-//			value = null;
-//		}
-//		return value;
-//	}
-//	
-//	private final String getLocalFontSize() {
-//		String value = this.getCssProperties().getFontSize();
-//		if(value != null && value.length() == 0) {
-//			value = null;
-//		}		
-//		return value;
-//	}
-
-	
 	public Font getFont() {
 		Font f = this.iFont;
 		if(f != null) {
@@ -247,15 +204,20 @@ public class StyleSheetRenderState implements RenderState {
 		String newFontStyle = style == null ? null : style.getFontStyle();
 		String newFontVariant = style == null ? null : style.getFontVariant();
 		String newFontWeight = style == null ? null : style.getFontWeight();
+		String verticalAlign = style == null ? null : style.getVerticalAlign();
+		boolean isSuper = verticalAlign != null && verticalAlign.equalsIgnoreCase("super");
+		boolean isSub = verticalAlign != null && verticalAlign.equalsIgnoreCase("sub");
 		if(newFontSize == null && newFontWeight == null && newFontStyle == null && newFontFamily == null && newFontVariant == null) {
-			if(prs != null) {
-				f = prs.getFont();
-				this.iFont = f;
-				return f;
-			}
-			f = DEFAULT_FONT;
-			this.iFont = f;
-			return f;
+		    if (!isSuper && !isSub) {
+		        if(prs != null) {
+		            f = prs.getFont();
+		            this.iFont = f;
+		            return f;
+		        }
+		        f = DEFAULT_FONT;
+		        this.iFont = f;
+		        return f;
+		    }
 		}
 		if(newFontSize != null) {
 			try {
@@ -306,7 +268,17 @@ public class StyleSheetRenderState implements RenderState {
 		}
 		HTMLDocumentImpl document = this.document;
 		Set locales = document == null ? null : document.getLocales();
-		f = FONT_FACTORY.getFont(fontFamily, fontStyle, fontVariant, fontWeight, fontSize.floatValue(), locales);
+		
+		Integer superscript = null;
+		if(isSuper) {
+			superscript = new Integer(1);
+		} else if (isSub) {
+			superscript = new Integer(-1);
+		}
+		if (superscript == null && prs != null){
+			superscript = (Integer) prs.getFont().getAttributes().get(TextAttribute.SUPERSCRIPT);
+		}		
+		f = FONT_FACTORY.getFont(fontFamily, fontStyle, fontVariant, fontWeight, fontSize.floatValue(), locales, superscript);
 		this.iFont = f;
 		return f;
 	}
@@ -620,17 +592,19 @@ public class StyleSheetRenderState implements RenderState {
 		if(prs != null) {
 			prs.resetCount(counter, nesting, value);
 		}
-		Map counters = this.counters;
-		if(counters == null) {
-			counters = new HashMap(2);
-			this.counters = counters;
-			counters.put(counter, new ArrayList(0));
+		else {
+			Map counters = this.counters;
+			if(counters == null) {
+				counters = new HashMap(2);
+				this.counters = counters;
+				counters.put(counter, new ArrayList(0));
+			}
+			ArrayList counterArray = (ArrayList) counters.get(counter);
+			while(counterArray.size() <= nesting) {
+				counterArray.add(null);
+			}
+			counterArray.set(nesting, new Integer(value));
 		}
-		ArrayList counterArray = (ArrayList) counters.get(counter);
-		while(counterArray.size() <= nesting) {
-			counterArray.add(null);
-		}
-		counterArray.set(nesting, new Integer(value));
 	}
 
 	public int incrementCount(String counter, int nesting) {
@@ -747,19 +721,13 @@ public class StyleSheetRenderState implements RenderState {
 		String whiteSpaceText = props == null ? null : props.getWhiteSpace();
 		int wsValue;
 		if(whiteSpaceText == null) {
-//			HTMLElementImpl element = this.element;
-//			if(element != null && element.getAttributeAsBoolean("nowrap")) {
-//				wsValue = WS_NOWRAP;
-//			}
-//			else {
-				RenderState prs = this.prevRenderState;
-				if(prs != null) {
-					wsValue = prs.getWhiteSpace();
-				}
-				else {
-					wsValue = WS_NORMAL;
-				}
-//			}
+		    RenderState prs = this.prevRenderState;
+		    if(prs != null) {
+		        wsValue = prs.getWhiteSpace();
+		    }
+		    else {
+		        wsValue = WS_NORMAL;
+		    }
 		}
 		else {
 			String whiteSpaceTextTL = whiteSpaceText.toLowerCase();
@@ -777,11 +745,11 @@ public class StyleSheetRenderState implements RenderState {
 		return wsValue;
 	}
 	
-	protected java.awt.Insets marginInsets = INVALID_INSETS;
-	protected java.awt.Insets paddingInsets = INVALID_INSETS;
+	protected HtmlInsets marginInsets = INVALID_INSETS;
+	protected HtmlInsets paddingInsets = INVALID_INSETS;
 	
-	public java.awt.Insets getMarginInsets() {
-		Insets mi = this.marginInsets;
+	public HtmlInsets getMarginInsets() {
+		HtmlInsets mi = this.marginInsets;
 		if(mi != INVALID_INSETS) {
 			return mi;
 		}
@@ -796,8 +764,8 @@ public class StyleSheetRenderState implements RenderState {
 		return mi;
 	}
 	
-	public java.awt.Insets getPaddingInsets() {
-		Insets mi = this.paddingInsets;
+	public HtmlInsets getPaddingInsets() {
+		HtmlInsets mi = this.paddingInsets;
 		if(mi != INVALID_INSETS) {
 			return mi;
 		}
@@ -1054,8 +1022,109 @@ public class StyleSheetRenderState implements RenderState {
 		this.cachedFloat = new Integer(floatValue);
 		return floatValue;		
 	}
-
+	
 	public String toString() {
 		return "StyleSheetRenderState[font=" + this.getFont() + ",textDecoration=" + this.getTextDecorationMask() + "]";
 	}
+	
+	protected int overflowX = -1;
+	protected int overflowY = -1;
+
+    public int getOverflowX() {
+        int overflow = this.overflowX;
+        if(overflow != -1) {
+            return overflow;
+        }
+        AbstractCSS2Properties props = this.getCssProperties();
+        if(props == null) {
+            overflow = OVERFLOW_NONE;
+        }
+        else {
+            String overflowText = props.getPropertyValue("overflow-x");
+            if(overflowText == null) {
+                overflowText = props.getOverflow(); 
+            }
+            if(overflowText == null) {
+                overflow = OVERFLOW_NONE;
+            }
+            else {
+                String overflowTextTL = overflowText.toLowerCase();
+                if("scroll".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_SCROLL;
+                }
+                else if("auto".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_AUTO;
+                }
+                else if("hidden".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_HIDDEN;
+                }
+                else if("visible".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_VISIBLE;
+                }
+                else {
+                    overflow = OVERFLOW_NONE;
+                }
+            }
+        }
+        this.overflowX = overflow;
+        return overflow;
+    }
+
+    public int getOverflowY() {
+        int overflow = this.overflowY;
+        if(overflow != -1) {
+            return overflow;
+        }
+        AbstractCSS2Properties props = this.getCssProperties();
+        if(props == null) {
+            overflow = OVERFLOW_NONE;
+        }
+        else {
+            String overflowText = props.getPropertyValue("overflow-y");
+            if(overflowText == null) {
+                overflowText = props.getOverflow(); 
+            }
+            if(overflowText == null) {
+                overflow = OVERFLOW_NONE;
+            }
+            else {
+                String overflowTextTL = overflowText.toLowerCase();
+                if("scroll".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_SCROLL;
+                }
+                else if("auto".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_AUTO;
+                }
+                else if("hidden".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_HIDDEN;
+                }
+                else if("visible".equals(overflowTextTL)) {
+                    overflow = OVERFLOW_VISIBLE;
+                }
+                else {
+                    overflow = OVERFLOW_NONE;
+                }
+            }
+        }
+        this.overflowY = overflow;
+        return overflow;
+    }
+    
+    protected BorderInfo borderInfo = INVALID_BORDER_INFO;
+    
+    public BorderInfo getBorderInfo() {
+        BorderInfo binfo = this.borderInfo;
+        if(binfo != INVALID_BORDER_INFO) {
+            return binfo;
+        }
+        AbstractCSS2Properties props = this.getCssProperties();
+        if(props != null) {
+            binfo = HtmlValues.getBorderInfo(props, this);
+        }
+        else {
+            binfo = null;
+        }
+        this.borderInfo = binfo;
+        return binfo;
+    }
 }
