@@ -180,10 +180,10 @@ public class TableCellRenderState extends DisplayRenderState {
 		return (HTMLTableElement) ancestor;
 	}
 	
-	private Insets paddingInsets = INVALID_INSETS;
+    private HtmlInsets paddingInsets = INVALID_INSETS;
 	
-	public Insets getPaddingInsets() {
-		Insets insets = this.paddingInsets;
+	public HtmlInsets getPaddingInsets() {
+		HtmlInsets insets = this.paddingInsets;
 		if(insets != INVALID_INSETS) {
 			return insets;
 		}
@@ -196,9 +196,28 @@ public class TableCellRenderState extends DisplayRenderState {
 			}
 			String cellPaddingText = tableElement.getAttribute("cellpadding");
 			if(cellPaddingText != null && cellPaddingText.length() != 0) {
-				//TODO: Cellpadding as percent not implemented properly.
-				int cellPadding = HtmlValues.getOldSyntaxPixelSize(cellPaddingText, 600, 0);
-				insets = new Insets(cellPadding, cellPadding, cellPadding, cellPadding);
+			    cellPaddingText = cellPaddingText.trim();
+			    int cellPadding;
+			    int cellPaddingType;
+			    if(cellPaddingText.endsWith("%")) {
+			        cellPaddingType = HtmlInsets.TYPE_PERCENT;
+			        try {
+			            cellPadding = Integer.parseInt(cellPaddingText.substring(0, cellPaddingText.length()-1));
+			        } catch(NumberFormatException nfe) {
+			            cellPadding = 0;
+			        }
+			    }
+			    else {
+			        cellPaddingType = HtmlInsets.TYPE_PIXELS;
+                    try {
+                        cellPadding = Integer.parseInt(cellPaddingText);
+                    } catch(NumberFormatException nfe) {
+                        cellPadding = 0;
+                    }			        
+			    }
+				insets = new HtmlInsets();
+				insets.top = insets.left = insets.right = insets.bottom = cellPadding;
+				insets.topType = insets.leftType = insets.rightType = insets.bottomType = cellPaddingType;
 			}
 		}
 		this.paddingInsets = insets;
