@@ -16,7 +16,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 
 package org.lobobrowser.html.style;
@@ -31,7 +31,7 @@ public class TextDecorationRenderState extends RenderStateDelegator {
 	}
 
 	public int getTextDecorationMask() {
-		RenderState prs = this.prevRenderState;
+		RenderState prs = this.delegate;
 		int parentMask = prs == null ? 0 : prs.getTextDecorationMask();
 		return parentMask | this.textDecorationMask;
 	}
