@@ -24,6 +24,7 @@
 package org.lobobrowser.html.test;
 
 import java.awt.Component;
+import java.awt.Insets;
 import java.net.URL;
 
 import org.lobobrowser.html.*;
@@ -74,4 +75,17 @@ public class SimpleBrowserFrame extends HtmlPanel implements BrowserFrame {
 	public HtmlRendererContext getParentHtmlRendererContext() {
 		return this.parentRcontext;
 	}
+
+    public void setDefaultMarginInsets(Insets insets) {
+        // Current implementation is the frame HtmlPanel.
+        super.setDefaultMarginInsets(insets);
+    }
+
+    public void setDefaultOverflowX(int overflowX) {
+        super.setDefaultOverflowX(overflowX);
+    }
+
+    public void setDefaultOverflowY(int overflowY) {
+        super.setDefaultOverflowY(overflowY);
+    }
 }
