@@ -23,10 +23,10 @@
  */
 package org.lobobrowser.html.test;
 
+import java.awt.Component;
 import java.awt.event.MouseEvent;
 import java.io.*;
 import java.net.*;
-
 import javax.swing.JOptionPane;
 
 import org.lobobrowser.html.*;
@@ -46,6 +46,9 @@ import java.util.logging.*;
  * Note that this class provides rudimentary implementations
  * of most callback methods. Overridding some of the methods
  * in this class will usually be necessary in a professional application.
+ * <p>
+ * A simple way to load a URL into the {@link HtmlPanel} of the
+ * renderer context is to invoke {@link #navigate(String)}. 
  */
 public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	private static final Logger logger = Logger.getLogger(SimpleHtmlRendererContext.class.getName());
@@ -65,6 +68,7 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	/**
 	 * Constructs a SimpleHtmlRendererContext.
 	 * @param contextComponent The component that will render HTML.
+	 * @see SimpleUserAgentContext
 	 */
 	public SimpleHtmlRendererContext(HtmlPanel contextComponent, UserAgentContext ucontext) {
 		super();
@@ -86,6 +90,10 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 		this.bcontext = parentRcontext == null ? null : parentRcontext.getUserAgentContext();
 	}
 
+	public HtmlPanel getHtmlPanel() {
+	    return this.htmlPanel;
+	}
+	
 	private volatile String sourceCode;
 	
 	/**
@@ -195,6 +203,9 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 			HTMLCollection frames = topCtx.getFrames();
 			if(frames != null) {
 				org.w3c.dom.Node frame = frames.namedItem(target);
+				if(logger.isLoggable(Level.INFO)) {
+					logger.info("submitForm(): Frame matching target=" + target + " is " + frame);
+				} 
 				if(frame instanceof FrameNode) {
 					BrowserFrame bframe = ((FrameNode) frame).getBrowserFrame();
 					if(bframe == null) {
@@ -222,9 +233,12 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 				this.open(action, "cobra.blank", "", false);
 				return;
 			}
-			else {
+			else if("_this".equals(actualTarget)) {
 				// fall through
 			}
+			else {
+				logger.warning("submitForm(): Link target unrecognized: " + actualTarget);
+			}
 		}
 		
 		// Make request asynchronously.
@@ -260,6 +274,11 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 		return true;
 	}
 	
+	/**
+	 * The connection currently opened by openSync() if any.
+	 */
+	protected URLConnection currentConnection;
+	
 	/**
 	 * Submits a form and/or navigates by making
 	 * a <i>synchronous</i> request. This method is invoked
@@ -336,103 +355,108 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 		Proxy proxy = SimpleHtmlRendererContext.this.getProxy();
 		boolean isPost = "POST".equals(actualMethod);
 		URLConnection connection = proxy == null || proxy == Proxy.NO_PROXY ? urlForLoading.openConnection() : urlForLoading.openConnection(proxy);
-		connection.setRequestProperty("User-Agent", getUserAgentContext().getUserAgent());
-		connection.setRequestProperty("Cookie", "");
-		if (connection instanceof HttpURLConnection) {
-			HttpURLConnection hc = (HttpURLConnection) connection;
-			hc.setRequestMethod(actualMethod);
-			hc.setInstanceFollowRedirects(false);
-		}
-		if(isPost) {
-			connection.setDoOutput(true);
-	    	ByteArrayOutputStream bufOut = new ByteArrayOutputStream();
-	    	boolean firstParam = true;
-	    	if(formInputs != null) {
-	    		for(int i = 0; i < formInputs.length; i++) {
-	    			FormInput parameter = formInputs[i];
-	    			String name = parameter.getName();
-    				String encName = URLEncoder.encode(name, "UTF-8");
-	    			if(parameter.isText()) {
-	    				if(firstParam) {
-	    					firstParam = false;
-	    				}
-	    				else {
-	    					bufOut.write((byte) '&');				    					
-	    				}
-	    				String valueStr = parameter.getTextValue();
-	    				String encValue = URLEncoder.encode(valueStr, "UTF-8");
-	    				bufOut.write(encName.getBytes("UTF-8"));
-	    				bufOut.write((byte) '=');
-	    				bufOut.write(encValue.getBytes("UTF-8"));
-	    			}
-	    			else {
-	    				logger.warning("postData(): Ignoring non-textual parameter " + name + " for POST.");
-	    			}
-	    		}
-	    	}
-	    	// Do not add a line break to post content. Some servers
-	    	// can be picky about that (namely, java.net).
-			byte[] postContent = bufOut.toByteArray();
-			if(connection instanceof HttpURLConnection) {
-				((HttpURLConnection) connection).setFixedLengthStreamingMode(postContent.length);
-			}
-			connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
-			//connection.setRequestProperty("Content-Length", String.valueOf(postContent.length));
-	    	OutputStream postOut = connection.getOutputStream();
-			postOut.write(postContent);
-			postOut.flush();
-		}					
-		if (connection instanceof HttpURLConnection) {					
-			HttpURLConnection hc = (HttpURLConnection) connection;
-			int responseCode = hc.getResponseCode();
-			if(logger.isLoggable(Level.INFO)) {
-				logger.info("process(): HTTP response code: "
-						+ responseCode);
-			}
-			if(responseCode == HttpURLConnection.HTTP_MOVED_PERM || responseCode == HttpURLConnection.HTTP_MOVED_TEMP) {
-				String location = hc.getHeaderField("Location");
-				if(location == null) {
-					logger.warning("No Location header in redirect from " + action + ".");
-				}
-				else {
-					java.net.URL href;
-					href = Urls.createURL(action, location);
-					SimpleHtmlRendererContext.this.navigate(href, target);
-				}
-				return;
-			}
-		}
-		InputStream in = connection.getInputStream();
+		this.currentConnection = connection;
 		try {
-			SimpleHtmlRendererContext.this.sourceCode = null;
-			long time1 = System.currentTimeMillis();
-			RecordedInputStream rin = new RecordedInputStream(in, 1000000);
-			InputStream bin = new BufferedInputStream(rin, 8192);
-			String actualURI = urlForLoading.toExternalForm();
-			// Only create document, don't parse.
-			HTMLDocumentImpl document = this.createDocument(new InputSourceImpl(bin, actualURI, getDocumentCharset(connection)));
-			// Set document in HtmlPanel. Safe to call outside GUI thread.
-			HtmlPanel panel = htmlPanel; 
-			panel.setDocument(document, SimpleHtmlRendererContext.this);
-			// Now start loading.
-			document.load();
-			long time2 = System.currentTimeMillis();
-			if(logger.isLoggable(Level.INFO)) {
-				logger.info("Parsed URI=[" + urlForLoading + "]: Parse elapsed: "
-						+ (time2 - time1) + " ms. Connection elapsed: "
-						+ (time1 - time0) + " ms.");
+			connection.setRequestProperty("User-Agent", getUserAgentContext().getUserAgent());
+			connection.setRequestProperty("Cookie", "");
+			if (connection instanceof HttpURLConnection) {
+				HttpURLConnection hc = (HttpURLConnection) connection;
+				hc.setRequestMethod(actualMethod);
+				hc.setInstanceFollowRedirects(false);
 			}
-			String ref = urlForLoading.getRef();
-			if(ref != null && ref.length() != 0) {
-				panel.scrollToElement(ref);
+			if(isPost) {
+				connection.setDoOutput(true);
+				ByteArrayOutputStream bufOut = new ByteArrayOutputStream();
+				boolean firstParam = true;
+				if(formInputs != null) {
+					for(int i = 0; i < formInputs.length; i++) {
+						FormInput parameter = formInputs[i];
+						String name = parameter.getName();
+						String encName = URLEncoder.encode(name, "UTF-8");
+						if(parameter.isText()) {
+							if(firstParam) {
+								firstParam = false;
+							}
+							else {
+								bufOut.write((byte) '&');				    					
+							}
+							String valueStr = parameter.getTextValue();
+							String encValue = URLEncoder.encode(valueStr, "UTF-8");
+							bufOut.write(encName.getBytes("UTF-8"));
+							bufOut.write((byte) '=');
+							bufOut.write(encValue.getBytes("UTF-8"));
+						}
+						else {
+							logger.warning("postData(): Ignoring non-textual parameter " + name + " for POST.");
+						}
+					}
+				}
+				// Do not add a line break to post content. Some servers
+				// can be picky about that (namely, java.net).
+				byte[] postContent = bufOut.toByteArray();
+				if(connection instanceof HttpURLConnection) {
+					((HttpURLConnection) connection).setFixedLengthStreamingMode(postContent.length);
+				}
+				connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
+				//connection.setRequestProperty("Content-Length", String.valueOf(postContent.length));
+				OutputStream postOut = connection.getOutputStream();
+				postOut.write(postContent);
+				postOut.flush();
+			}					
+			if (connection instanceof HttpURLConnection) {					
+				HttpURLConnection hc = (HttpURLConnection) connection;
+				int responseCode = hc.getResponseCode();
+				if(logger.isLoggable(Level.INFO)) {
+					logger.info("process(): HTTP response code: "
+							+ responseCode);
+				}
+				if(responseCode == HttpURLConnection.HTTP_MOVED_PERM || responseCode == HttpURLConnection.HTTP_MOVED_TEMP || responseCode == HttpURLConnection.HTTP_SEE_OTHER) {
+					String location = hc.getHeaderField("Location");
+					if(location == null) {
+						logger.warning("No Location header in redirect from " + action + ".");
+					}
+					else {
+						java.net.URL href;
+						href = Urls.createURL(action, location);
+						SimpleHtmlRendererContext.this.navigate(href, target);
+					}
+					return;
+				}
 			}
+			InputStream in = connection.getInputStream();
 			try {
-				SimpleHtmlRendererContext.this.sourceCode = rin.getString("ISO-8859-1");
-			} catch(BufferExceededException bee) {
-				SimpleHtmlRendererContext.this.sourceCode = "[TOO BIG]";
+				SimpleHtmlRendererContext.this.sourceCode = null;
+				long time1 = System.currentTimeMillis();
+				RecordedInputStream rin = new RecordedInputStream(in, 1000000);
+				InputStream bin = new BufferedInputStream(rin, 8192);
+				String actualURI = urlForLoading.toExternalForm();
+				// Only create document, don't parse.
+				HTMLDocumentImpl document = this.createDocument(new InputSourceImpl(bin, actualURI, getDocumentCharset(connection)));
+				// Set document in HtmlPanel. Safe to call outside GUI thread.
+				HtmlPanel panel = htmlPanel; 
+				panel.setDocument(document, SimpleHtmlRendererContext.this);
+				// Now start loading.
+				document.load();
+				long time2 = System.currentTimeMillis();
+				if(logger.isLoggable(Level.INFO)) {
+					logger.info("Parsed URI=[" + urlForLoading + "]: Parse elapsed: "
+							+ (time2 - time1) + " ms. Connection elapsed: "
+							+ (time1 - time0) + " ms.");
+				}
+				String ref = urlForLoading.getRef();
+				if(ref != null && ref.length() != 0) {
+					panel.scrollToElement(ref);
+				}
+				try {
+					SimpleHtmlRendererContext.this.sourceCode = rin.getString("ISO-8859-1");
+				} catch(BufferExceededException bee) {
+					SimpleHtmlRendererContext.this.sourceCode = "[TOO BIG]";
+				}
+			} finally { 
+				in.close();
 			}
-		} finally { 
-			in.close();
+		} finally {
+			this.currentConnection = null;
 		}
 	}
 
@@ -468,14 +492,6 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	public void alert(String message) {
 		JOptionPane.showMessageDialog(this.htmlPanel, message);
 	}
-
-	/**
-	 * It should navigate back one page. This implementation does nothing
-	 * and should be overridden.
-	 */
-	public void back() {
-		this.warn("back(): Not overridden");
-	}
 	
 	/**
 	 * It should give up focus on the current browser window. This implementation does nothing
@@ -555,7 +571,11 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	public void scroll(int x, int y) {
 		this.htmlPanel.scroll(x, y);
 	}
-	
+
+	public void scrollBy(int x, int y) {
+		this.htmlPanel.scrollBy(x, y);
+	}
+
 	/**
 	 * Should return true if and only if the current browser window is closed. 
 	 * This implementation returns false and should be overridden.
@@ -681,7 +701,6 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	 * has been visited.
 	 */
 	public boolean isVisitedLink(HTMLLinkElement link) {
-		//TODO
 		return false;
 	}
 
@@ -717,4 +736,75 @@ public class SimpleHtmlRendererContext implements HtmlRendererContext {
 	public boolean onMouseClick(HTMLElement element, MouseEvent event) {
 		return true;
 	}
+	
+	private static java.awt.Window getWindow(Component c) {
+		java.awt.Component current = c;
+		while(current != null && !(current instanceof java.awt.Window)) {
+			current = current.getParent();
+		}
+		return (java.awt.Window) current;
+	}
+
+	public void resizeBy(int byWidth, int byHeight) {
+		java.awt.Window window = getWindow(this.htmlPanel);
+		if(window != null) {
+			window.setSize(window.getWidth() + byWidth, window.getHeight() + byHeight);
+		}
+	}
+ 
+	public void resizeTo(int width, int height) {
+		java.awt.Window window = getWindow(this.htmlPanel);
+		if(window != null) {
+			window.setSize(width, height);
+		}
+	}
+
+    /**
+     * It should navigate back one page. This implementation does nothing
+     * and should be overridden.
+     */
+    public void back() {
+        if(logger.isLoggable(Level.WARNING)) {
+            logger.log(Level.WARNING, "back() does nothing, unless overridden.");
+        }       
+    }
+
+    public void forward() {
+        if(logger.isLoggable(Level.WARNING)) {
+            logger.log(Level.WARNING, "forward() does nothing, unless overridden.");
+        }       
+    }
+
+    public String getCurrentURL() {
+        Object node = this.htmlPanel.getRootNode();
+        if(node instanceof HTMLDocumentImpl) {
+            HTMLDocumentImpl doc = (HTMLDocumentImpl) node;
+            return doc.getDocumentURI();
+        }
+        return null;
+    }
+
+    public int getHistoryLength() {
+        return 0;
+    }
+
+    public String getNextURL() {
+        return null;
+    }
+
+    public String getPreviousURL() {
+        return null;
+    }
+
+    public void goToHistoryURL(String url) {
+        if(logger.isLoggable(Level.WARNING)) {
+            logger.log(Level.WARNING, "goToHistoryURL() does nothing, unless overridden.");
+        }       
+    }
+
+    public void moveInHistory(int offset) {
+        if(logger.isLoggable(Level.WARNING)) {
+            logger.log(Level.WARNING, "moveInHistory() does nothing, unless overridden.");
+        }       
+    }
 }
