@@ -16,11 +16,11 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 package org.lobobrowser.js;
 
-import org.mozilla.javascript.Scriptable;
+import org.mozilla.javascript.*;
 
 public abstract class AbstractScriptableDelegate implements ScriptableDelegate {
 	private Scriptable scriptable;
@@ -31,5 +31,5 @@ public abstract class AbstractScriptableDelegate implements ScriptableDelegate {
 
 	public void setScriptable(Scriptable scriptable) {
 		this.scriptable = scriptable;
-	}
+	}	
 }
