@@ -117,6 +117,13 @@ public class JavaObjectWrapper extends ScriptableObject {
 				return f;
 			}
 			else {
+				// Should check properties set in context
+				// first. Consider element IDs should not
+				// override Window variables set by user.
+				Object result = super.get(name, start);
+				if(result != Scriptable.NOT_FOUND) {
+					return result;
+				}				
 				PropertyInfo ni = this.classWrapper.getNameIndexer();
 				if(ni != null) {
 					Method getter = ni.getGetter();
@@ -139,15 +146,8 @@ public class JavaObjectWrapper extends ScriptableObject {
 							throw new WrappedException(err);
 						}
 					}
-					else {
-						Object result = super.get(name, start);
-						return result;
-					}
-				}
-				else {
-					Object result = super.get(name, start);
-					return result;
 				}
+				return Scriptable.NOT_FOUND;
 			}
 		}
 	}
@@ -173,43 +173,48 @@ public class JavaObjectWrapper extends ScriptableObject {
 	}
 
 	public void put(String name, Scriptable start, Object value) {
-		PropertyInfo pinfo = this.classWrapper.getProperty(name);
-		if(pinfo != null) {
-			Method setter = pinfo.getSetter();
-			if(setter == null) {
-				throw new EvaluatorException("Property '" + name + "' is not settable in " + this.classWrapper.getClassName() + ".");
-			}
-			try {
-				Object actualValue;
-				actualValue = JavaScript.getInstance().getJavaObject(value, pinfo.getPropertyType());
-				setter.invoke(this.getJavaObject(), new Object[] { actualValue });
-			} catch(IllegalArgumentException iae) {
-				Exception newException = new IllegalArgumentException("Property named '" + name + "' could not be set with value " + value + ".", iae);
-				throw new WrappedException(newException);
-			} catch(Exception err) {
-				throw new WrappedException(err);
-			}
+		if(value instanceof org.mozilla.javascript.Undefined) {
+			super.put(name, start, value);
 		}
 		else {
-			PropertyInfo ni = this.classWrapper.getNameIndexer();
-			if(ni != null) {
-				Method setter = ni.getSetter();
-				if(setter != null) {
-					try {
-						Object actualValue;
-						actualValue = JavaScript.getInstance().getJavaObject(value, ni.getPropertyType());
-						setter.invoke(this.getJavaObject(), new Object[] { name, actualValue });
-					} catch(Exception err) {
-						throw new WrappedException(err);
+			PropertyInfo pinfo = this.classWrapper.getProperty(name);
+			if(pinfo != null) {
+				Method setter = pinfo.getSetter();
+				if(setter == null) {
+					throw new EvaluatorException("Property '" + name + "' is not settable in " + this.classWrapper.getClassName() + ".");
+				}
+				try {
+					Object actualValue;
+					actualValue = JavaScript.getInstance().getJavaObject(value, pinfo.getPropertyType());
+					setter.invoke(this.getJavaObject(), new Object[] { actualValue });
+				} catch(IllegalArgumentException iae) {
+					Exception newException = new IllegalArgumentException("Property named '" + name + "' could not be set with value " + value + ".", iae);
+					throw new WrappedException(newException);
+				} catch(Exception err) {
+					throw new WrappedException(err);
+				}
+			}
+			else {
+				PropertyInfo ni = this.classWrapper.getNameIndexer();
+				if(ni != null) {
+					Method setter = ni.getSetter();
+					if(setter != null) {
+						try {
+							Object actualValue;
+							actualValue = JavaScript.getInstance().getJavaObject(value, ni.getPropertyType());
+							setter.invoke(this.getJavaObject(), new Object[] { name, actualValue });
+						} catch(Exception err) {
+							throw new WrappedException(err);
+						}
+					}
+					else {
+						super.put(name, start, value);
 					}
 				}
 				else {
 					super.put(name, start, value);
 				}
 			}
-			else {
-				super.put(name, start, value);
-			}
 		}
 	}
 	
@@ -248,7 +253,7 @@ public class JavaObjectWrapper extends ScriptableObject {
 			return super.getDefaultValue(hint);
 		}
 	}
-
+	
 	public String toString() {
 		Object javaObject = this.getJavaObject();
 		String type = javaObject == null ? "<null>" : javaObject.getClass().getName();
