@@ -31,7 +31,7 @@ public class LRUCache implements java.io.Serializable {
 	private int approxMaxSize;
 	
 	private final Map cacheMap = new HashMap();
-	private final EventDispatch2 REMOVAL_EVENT = new RemovalDispatch(); 
+	private volatile transient EventDispatch2 removalEvent; 
 	
 	/**
 	 * Ascending timestamp order. First is least recently used.
@@ -41,6 +41,13 @@ public class LRUCache implements java.io.Serializable {
 	
 	public LRUCache(int approxMaxSize) {
 		this.approxMaxSize = approxMaxSize;
+		this.removalEvent = new RemovalDispatch();
+	}
+
+	private void readObject(java.io.ObjectInputStream in) throws java.io.IOException, ClassNotFoundException {
+		in.defaultReadObject();
+		// Need to initialize transient fields here.
+		this.removalEvent = new RemovalDispatch();
 	}
 	
 	public int getApproxMaxSize() {
@@ -59,7 +66,7 @@ public class LRUCache implements java.io.Serializable {
 		OrderedValue ordVal = (OrderedValue) this.cacheMap.get(key);
 		if(ordVal != null) {
 			if(ordVal.value != value) {
-				this.REMOVAL_EVENT.fireEvent(new RemovalEvent(this, ordVal.value));
+				this.removalEvent.fireEvent(new RemovalEvent(this, ordVal.value));
 			}
 			this.currentSize += (approxSize - ordVal.approximateSize);			
 			this.timedSet.remove(ordVal);
@@ -82,7 +89,7 @@ public class LRUCache implements java.io.Serializable {
 	private void removeLRU() {
 		OrderedValue ordVal = (OrderedValue) this.timedSet.first();
 		if(ordVal != null) {
-			this.REMOVAL_EVENT.fireEvent(new RemovalEvent(this, ordVal.value));
+			this.removalEvent.fireEvent(new RemovalEvent(this, ordVal.value));
 			if(this.timedSet.remove(ordVal)) {
 				this.cacheMap.remove(ordVal.key);
 				this.currentSize -= ordVal.approximateSize;
@@ -112,7 +119,7 @@ public class LRUCache implements java.io.Serializable {
 	public Object remove(Object key) {
 		OrderedValue ordVal = (OrderedValue) this.cacheMap.get(key);
 		if(ordVal != null) {
-			this.REMOVAL_EVENT.fireEvent(new RemovalEvent(this, ordVal.value));
+			this.removalEvent.fireEvent(new RemovalEvent(this, ordVal.value));
 			this.currentSize -= ordVal.approximateSize;
 			this.timedSet.remove(ordVal);
 			return ordVal.value;
@@ -123,11 +130,11 @@ public class LRUCache implements java.io.Serializable {
 	}
 	
 	public void addRemovalListener(RemovalListener listener) {
-		this.REMOVAL_EVENT.addListener(listener);
+		this.removalEvent.addListener(listener);
 	}
 	
 	public void removeRemovalListener(RemovalListener listener) {
-		this.REMOVAL_EVENT.removeListener(listener);
+		this.removalEvent.removeListener(listener);
 	}
 	
 	public int getApproxSize() {
