@@ -16,7 +16,7 @@
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-    Contact info: xamjadmin@users.sourceforge.net
+    Contact info: lobochief@users.sourceforge.net
 */
 /*
  * Created on Jun 12, 2005
@@ -26,7 +26,8 @@ package org.lobobrowser.util;
 /**
  * @author J. H. S.
  */
-public class NameValuePair {
+public class NameValuePair implements java.io.Serializable {
+    private static final long serialVersionUID = 22574500600001010L;
 	public final String name;
 	public final String value;
 	
