@@ -41,10 +41,88 @@ public class Strings
 
 	private Strings() {}
 
-	public static int compareVersions(String version1, String version2) {
-		return version1.compareTo(version2);
+	public static int compareVersions(String version1, String version2, boolean startsWithDigits) {
+		if(version1 == null) {
+			return version2 == null ? 0 : -1;
+		}
+		else if(version2 == null) {
+			return +1;
+		}
+		if(startsWithDigits) {
+			String v1prefix = leadingDigits(version1);
+			String v2prefix = leadingDigits(version2);
+			if(v1prefix.length() == 0) { 
+				if(v2prefix.length() == 0) {
+					return 0;
+				}
+				return -1;
+			}
+			else if(v2prefix.length() == 0) {
+				return +1;
+			}
+			int diff;
+			try {
+				diff = Integer.parseInt(v1prefix) - Integer.parseInt(v2prefix);
+			} catch(NumberFormatException nfe) {
+				diff = 0;
+			}
+			if(diff == 0) {
+				return compareVersions(version1.substring(v1prefix.length()), version2.substring(v2prefix.length()), false);
+			}
+			return diff;
+		}
+		else {
+			String v1prefix = leadingNonDigits(version1);
+			String v2prefix = leadingNonDigits(version2);
+			if(v1prefix.length() == 0) { 
+				if(v2prefix.length() == 0) {
+					return 0;
+				}
+				return -1;
+			}
+			else if(v2prefix.length() == 0) {
+				return +1;
+			}
+			int diff = v1prefix.compareTo(v2prefix);
+			if(diff == 0) {
+				return compareVersions(version1.substring(v1prefix.length()), version2.substring(v2prefix.length()), true);
+			}
+			return diff;			
+		}
 	}
-	
+
+	public static String leadingDigits(String text) {
+		int length = text.length();
+		StringBuffer buffer = null;
+		for(int i = 0; i < length; i++) {
+			char ch = text.charAt(i);
+			if(!Character.isDigit(ch)) {
+				break;
+			}
+			if(buffer == null) {
+				buffer = new StringBuffer(3);
+			}
+			buffer.append(ch);
+		}
+		return buffer == null ? "" : buffer.toString();
+	}
+
+	public static String leadingNonDigits(String text) {
+		int length = text.length();
+		StringBuffer buffer = null;
+		for(int i = 0; i < length; i++) {
+			char ch = text.charAt(i);
+			if(Character.isDigit(ch)) {
+				break;
+			}
+			if(buffer == null) {
+				buffer = new StringBuffer(3);
+			}
+			buffer.append(ch);
+		}
+		return buffer == null ? "" : buffer.toString();
+	}
+
 	public static boolean isBlank(String text) {
 	    return text == null || "".equals(text);
 	}
@@ -224,7 +302,8 @@ public class Strings
 	
 	public static String unquote(String text) {
 		if(text.startsWith("\"") && text.endsWith("\"")) {
-			return text.substring(1, text.length() - 2);
+			// substring works on indices
+			return text.substring(1, text.length() - 1);
 		}
 		return text;
 	}	
@@ -268,7 +347,7 @@ public class Strings
 		return text.substring(0, Math.max(maxLength - 3, 0)) + "...";
 	}
 	
-	public static String strictHtmlEncode(String rawText) {
+	public static String strictHtmlEncode(String rawText, boolean quotes) {
 		StringBuffer output = new StringBuffer();
 		int length = rawText.length();
 		for(int i = 0; i < length; i++) {
@@ -278,7 +357,12 @@ public class Strings
 				output.append("&amp;");
 				break;
 			case '"':
-				output.append("&quot;");
+				if(quotes) {
+					output.append("&quot;");
+				}
+				else {
+					output.append(ch);
+				}
 				break;
 			case '<':
 				output.append("&lt;");
