@@ -34,7 +34,8 @@ public class Urls {
 	public static final DateFormat PATTERN_RFC1123 = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz", Locale.US);
 
 	static {
-		PATTERN_RFC1123.setTimeZone(TimeZone.getTimeZone("GMT"));
+		DateFormat df = PATTERN_RFC1123;
+		df.setTimeZone(TimeZone.getTimeZone("GMT"));
 	}
 	
 	private Urls() {
@@ -136,12 +137,15 @@ public class Urls {
 		// Random access index recommended.
 		List headers = new ArrayList();
 		for(int n = 0; ; n++) {
-			String key = connection.getHeaderFieldKey(n);
-			if(key == null) {
-				break;
-			}
 			String value = connection.getHeaderField(n);
-			headers.add(new NameValuePair(key, value));
+			if(value == null) {
+			    break;
+			}
+            // Key may be null for n == 0.
+            String key = connection.getHeaderFieldKey(n);
+			if(key != null) {
+			    headers.add(new NameValuePair(key, value));
+			}
 		}
 		return headers;
 	}
